@@ -219,9 +219,6 @@ mm_modem_messaging_get_default_storage (MMModemMessaging *self)
 /*****************************************************************************/
 
 typedef struct {
-    MMModemMessaging *self;
-    GSimpleAsyncResult *result;
-    GCancellable *cancellable;
     gchar **sms_paths;
     GList *sms_objects;
     guint i;
@@ -230,20 +227,14 @@ typedef struct {
 static void
 sms_object_list_free (GList *list)
 {
-    g_list_free_full (list, (GDestroyNotify) g_object_unref);
+    g_list_free_full (list, g_object_unref);
 }
 
 static void
-list_sms_context_complete_and_free (ListSmsContext *ctx)
+list_sms_context_free (ListSmsContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-
     g_strfreev (ctx->sms_paths);
     sms_object_list_free (ctx->sms_objects);
-    g_object_unref (ctx->result);
-    if (ctx->cancellable)
-        g_object_unref (ctx->cancellable);
-    g_object_ref (ctx->self);
     g_slice_free (ListSmsContext, ctx);
 }
 
@@ -262,70 +253,69 @@ mm_modem_messaging_list_finish (MMModemMessaging *self,
                                 GAsyncResult *res,
                                 GError **error)
 {
-    GList *list;
-
     g_return_val_if_fail (MM_IS_MODEM_MESSAGING (self), FALSE);
 
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    list = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-
-    /* The list we got, including the objects within, is owned by the async result;
-     * so we'll make sure we return a new list */
-    g_list_foreach (list, (GFunc)g_object_ref, NULL);
-    return g_list_copy (list);
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
-static void create_next_sms (ListSmsContext *ctx);
+static void create_next_sms (GTask *task);
 
 static void
 list_build_object_ready (GDBusConnection *connection,
                          GAsyncResult *res,
-                         ListSmsContext *ctx)
+                         GTask *task)
 {
     GError *error = NULL;
     GObject *sms;
     GObject *source_object;
+    ListSmsContext *ctx;
 
     source_object = g_async_result_get_source_object (res);
     sms = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, &error);
     g_object_unref (source_object);
 
     if (error) {
-        g_simple_async_result_take_error (ctx->result, error);
-        list_sms_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
+    ctx = g_task_get_task_data (task);
+
     /* Keep the object */
     ctx->sms_objects = g_list_prepend (ctx->sms_objects, sms);
 
     /* If no more smss, just end here. */
     if (!ctx->sms_paths[++ctx->i]) {
-        g_simple_async_result_set_op_res_gpointer (ctx->result,
-                                                   ctx->sms_objects,
-                                                   (GDestroyNotify)sms_object_list_free);
-        ctx->sms_objects = NULL;
-        list_sms_context_complete_and_free (ctx);
+        GList *sms_objects;
+
+        sms_objects = g_list_copy_deep (ctx->sms_objects, (GCopyFunc)g_object_ref, NULL);
+        g_task_return_pointer (task, sms_objects, (GDestroyNotify)sms_object_list_free);
+        g_object_unref (task);
         return;
     }
 
     /* Keep on creating next object */
-    create_next_sms (ctx);
+    create_next_sms (task);
 }
 
 static void
-create_next_sms (ListSmsContext *ctx)
+create_next_sms (GTask *task)
 {
+    MMModemMessaging *self;
+    ListSmsContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     g_async_initable_new_async (MM_TYPE_SMS,
                                 G_PRIORITY_DEFAULT,
-                                ctx->cancellable,
+                                g_task_get_cancellable (task),
                                 (GAsyncReadyCallback)list_build_object_ready,
-                                ctx,
+                                task,
                                 "g-flags",          G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
                                 "g-name",           MM_DBUS_SERVICE,
-                                "g-connection",     g_dbus_proxy_get_connection (G_DBUS_PROXY (ctx->self)),
+                                "g-connection",     g_dbus_proxy_get_connection (G_DBUS_PROXY (self)),
                                 "g-object-path",    ctx->sms_paths[ctx->i],
                                 "g-interface-name", "org.freedesktop.ModemManager1.Sms",
                                 NULL);
@@ -352,30 +342,26 @@ mm_modem_messaging_list (MMModemMessaging *self,
                          gpointer user_data)
 {
     ListSmsContext *ctx;
+    GTask *task;
 
     g_return_if_fail (MM_IS_MODEM_MESSAGING (self));
 
     ctx = g_slice_new0 (ListSmsContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_modem_messaging_list);
-    if (cancellable)
-        ctx->cancellable = g_object_ref (cancellable);
-
     ctx->sms_paths = mm_gdbus_modem_messaging_dup_messages (MM_GDBUS_MODEM_MESSAGING (self));
 
+    task = g_task_new (self, cancellable, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)list_sms_context_free);
+
     /* If no SMS, just end here. */
     if (!ctx->sms_paths || !ctx->sms_paths[0]) {
-        g_simple_async_result_set_op_res_gpointer (ctx->result, NULL, NULL);
-        list_sms_context_complete_and_free (ctx);
+        g_task_return_pointer (task, NULL, NULL);
+        g_object_unref (task);
         return;
     }
 
     /* Got list of paths. If at least one found, start creating objects for each */
     ctx->i = 0;
-    create_next_sms (ctx);
+    create_next_sms (task);
 }
 
 /**
@@ -389,7 +375,7 @@ mm_modem_messaging_list (MMModemMessaging *self,
  * The calling thread is blocked until a reply is received. See mm_modem_messaging_list()
  * for the asynchronous version of this method.
  *
- * Returns: (element-type MMSms) (transfer full): A list of #MMSms objects, or #NULL if either not found or @error is set. The returned value should be freed with g_list_free_full() using g_object_unref() as #GDestroyNotify function.
+ * Returns: (element-type ModemManager.Sms) (transfer full): A list of #MMSms objects, or #NULL if either not found or @error is set. The returned value should be freed with g_list_free_full() using g_object_unref() as #GDestroyNotify function.
  */
 GList *
 mm_modem_messaging_list_sync (MMModemMessaging *self,
@@ -436,21 +422,6 @@ mm_modem_messaging_list_sync (MMModemMessaging *self,
 
 /*****************************************************************************/
 
-typedef struct {
-    GSimpleAsyncResult *result;
-    GCancellable *cancellable;
-} CreateSmsContext;
-
-static void
-create_sms_context_complete_and_free (CreateSmsContext *ctx)
-{
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
-    if (ctx->cancellable)
-        g_object_unref (ctx->cancellable);
-    g_slice_free (CreateSmsContext, ctx);
-}
-
 /**
  * mm_modem_messaging_create_finish:
  * @self: A #MMModemMessaging.
@@ -468,16 +439,13 @@ mm_modem_messaging_create_finish (MMModemMessaging *self,
 {
     g_return_val_if_fail (MM_IS_MODEM_MESSAGING (self), NULL);
 
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    return g_object_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
 static void
 new_sms_object_ready (GDBusConnection *connection,
                       GAsyncResult *res,
-                      CreateSmsContext *ctx)
+                      GTask *task)
 {
     GError *error = NULL;
     GObject *sms;
@@ -488,19 +456,17 @@ new_sms_object_ready (GDBusConnection *connection,
     g_object_unref (source_object);
 
     if (error)
-        g_simple_async_result_take_error (ctx->result, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gpointer (ctx->result,
-                                                   sms,
-                                                   (GDestroyNotify)g_object_unref);
+        g_task_return_pointer (task, sms, g_object_unref);
 
-    create_sms_context_complete_and_free (ctx);
+    g_object_unref (task);
 }
 
 static void
 create_sms_ready (MMModemMessaging *self,
                   GAsyncResult *res,
-                  CreateSmsContext *ctx)
+                  GTask *task)
 {
     GError *error = NULL;
     gchar *sms_path = NULL;
@@ -509,17 +475,17 @@ create_sms_ready (MMModemMessaging *self,
                                                       &sms_path,
                                                       res,
                                                       &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        create_sms_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         g_free (sms_path);
         return;
     }
 
     g_async_initable_new_async (MM_TYPE_SMS,
                                 G_PRIORITY_DEFAULT,
-                                ctx->cancellable,
+                                g_task_get_cancellable (task),
                                 (GAsyncReadyCallback)new_sms_object_ready,
-                                ctx,
+                                task,
                                 "g-flags",          G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
                                 "g-name",           MM_DBUS_SERVICE,
                                 "g-connection",     g_dbus_proxy_get_connection (G_DBUS_PROXY (self)),
@@ -551,26 +517,19 @@ mm_modem_messaging_create (MMModemMessaging *self,
                            GAsyncReadyCallback callback,
                            gpointer user_data)
 {
-    CreateSmsContext *ctx;
+    GTask *task;
     GVariant *dictionary;
 
     g_return_if_fail (MM_IS_MODEM_MESSAGING (self));
 
-    ctx = g_slice_new0 (CreateSmsContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_modem_messaging_create);
-    if (cancellable)
-        ctx->cancellable = g_object_ref (cancellable);
-
+    task = g_task_new (self, cancellable, callback, user_data);
     dictionary = (mm_sms_properties_get_dictionary (properties));
     mm_gdbus_modem_messaging_call_create (
         MM_GDBUS_MODEM_MESSAGING (self),
         dictionary,
         cancellable,
         (GAsyncReadyCallback)create_sms_ready,
-        ctx);
+        task);
 
     g_variant_unref (dictionary);
 }
