@@ -25,12 +25,6 @@
 #include "mm-modem-helpers.h"
 #include "mm-modem-helpers-cinterion.h"
 
-static gint
-sort_band (MMModemBand a, MMModemBand b)
-{
-    return a - b;
-}
-
 /*****************************************************************************/
 /* Test ^SCFG test responses */
 
@@ -52,8 +46,8 @@ common_test_scfg (const gchar *response,
     g_assert (res == TRUE);
     g_assert (bands != NULL);
 
-    g_array_sort (bands, (GCompareFunc)sort_band);
-    g_array_sort (expected_bands, (GCompareFunc)sort_band);
+    mm_common_bands_garray_sort (bands);
+    mm_common_bands_garray_sort (expected_bands);
 
     expected_bands_str = mm_common_build_bands_string ((const MMModemBand *)expected_bands->data,
                                                        expected_bands->len);
@@ -103,15 +97,15 @@ test_scfg (void)
         "^SCFG: \"URC/Ringline/ActiveTime\",(\"0\",\"1\",\"2\",\"keep\")\r\n";
 
     expected_bands = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), 9);
-    single = MM_MODEM_BAND_EGSM,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_DCS,   g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_PCS,   g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_G850,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U2100, g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U1900, g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U850,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U900,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U800,  g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_EGSM,    g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_DCS,     g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_PCS,     g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_G850,    g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_1, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_2, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_5, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_8, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_6, g_array_append_val (expected_bands, single);
 
     common_test_scfg (response, expected_bands);
 
@@ -176,10 +170,10 @@ test_scfg_ehs5 (void)
         "^SCFG: \"Userware/Watchdog\",(\"0\",\"1\",\"2\")\r\n";
 
     expected_bands = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), 4);
-    single = MM_MODEM_BAND_EGSM,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_DCS,   g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U2100, g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U900,  g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_EGSM,    g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_DCS,     g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_1, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_8, g_array_append_val (expected_bands, single);
 
     common_test_scfg (response, expected_bands);
 
@@ -205,8 +199,8 @@ common_test_scfg_response (const gchar *response,
     g_assert (res == TRUE);
     g_assert (bands != NULL);
 
-    g_array_sort (bands, (GCompareFunc)sort_band);
-    g_array_sort (expected_bands, (GCompareFunc)sort_band);
+    mm_common_bands_garray_sort (bands);
+    mm_common_bands_garray_sort (expected_bands);
 
     expected_bands_str = mm_common_build_bands_string ((const MMModemBand *)expected_bands->data,
                                                        expected_bands->len);
@@ -267,13 +261,13 @@ test_scfg_response_3g (void)
         "\r\n";
 
     expected_bands = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), 9);
-    single = MM_MODEM_BAND_EGSM,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_DCS,   g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_PCS,   g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_G850,  g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U2100, g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U1900, g_array_append_val (expected_bands, single);
-    single = MM_MODEM_BAND_U850,  g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_EGSM,    g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_DCS,     g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_PCS,     g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_G850,    g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_1, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_2, g_array_append_val (expected_bands, single);
+    single = MM_MODEM_BAND_UTRAN_5, g_array_append_val (expected_bands, single);
 
     common_test_scfg_response (response, MM_MODEM_CHARSET_UNKNOWN, expected_bands);
 
@@ -395,7 +389,206 @@ test_cnmi_phs8 (void)
     g_array_unref (expected_mt);
     g_array_unref (expected_bm);
     g_array_unref (expected_ds);
-    g_array_unref (expected_bfr);}
+    g_array_unref (expected_bfr);
+}
+
+static void
+test_cnmi_other (void)
+{
+    GArray *expected_mode;
+    GArray *expected_mt;
+    GArray *expected_bm;
+    GArray *expected_ds;
+    GArray *expected_bfr;
+    guint val;
+    const gchar *response =
+        "+CNMI: (0-3),(0,1),(0,2,3),(0,2),(1)\r\n"
+        "\r\n";
+
+    expected_mode = g_array_sized_new (FALSE, FALSE, sizeof (guint), 3);
+    val = 0, g_array_append_val (expected_mode, val);
+    val = 1, g_array_append_val (expected_mode, val);
+    val = 2, g_array_append_val (expected_mode, val);
+    val = 3, g_array_append_val (expected_mode, val);
+
+    expected_mt = g_array_sized_new (FALSE, FALSE, sizeof (guint), 2);
+    val = 0, g_array_append_val (expected_mt, val);
+    val = 1, g_array_append_val (expected_mt, val);
+
+    expected_bm = g_array_sized_new (FALSE, FALSE, sizeof (guint), 2);
+    val = 0, g_array_append_val (expected_bm, val);
+    val = 2, g_array_append_val (expected_bm, val);
+    val = 3, g_array_append_val (expected_bm, val);
+
+    expected_ds = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
+    val = 0, g_array_append_val (expected_ds, val);
+    val = 2, g_array_append_val (expected_ds, val);
+
+    expected_bfr = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
+    val = 1, g_array_append_val (expected_bfr, val);
+
+    common_test_cnmi (response,
+                      expected_mode,
+                      expected_mt,
+                      expected_bm,
+                      expected_ds,
+                      expected_bfr);
+
+    g_array_unref (expected_mode);
+    g_array_unref (expected_mt);
+    g_array_unref (expected_bm);
+    g_array_unref (expected_ds);
+    g_array_unref (expected_bfr);
+}
+
+/*****************************************************************************/
+/* Test ^SWWAN read */
+
+#define SWWAN_TEST_MAX_CIDS 2
+
+typedef struct {
+    guint                    cid;
+    MMBearerConnectionStatus state;
+} PdpContextState;
+
+typedef struct {
+    const gchar     *response;
+    PdpContextState  expected_items[SWWAN_TEST_MAX_CIDS];
+    gboolean         skip_test_other_cids;
+} SwwanTest;
+
+/* Note: all tests are based on checking CIDs 2 and 3 */
+static const SwwanTest swwan_tests[] = {
+    /* No active PDP context reported (all disconnected) */
+    {
+        .response = "",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED }
+        },
+        /* Don't test other CIDs because for those we would also return
+         * DISCONNECTED, not UNKNOWN. */
+        .skip_test_other_cids = TRUE
+    },
+    /* Single PDP context active (short version without interface index) */
+    {
+        .response = "^SWWAN: 3,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_UNKNOWN   },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED }
+        }
+    },
+    /* Single PDP context active (long version with interface index) */
+    {
+        .response = "^SWWAN: 3,1,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_UNKNOWN   },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED }
+        }
+    },
+    /* Single PDP context inactive (short version without interface index) */
+    {
+        .response = "^SWWAN: 3,0\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_UNKNOWN      },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED }
+        }
+    },
+    /* Single PDP context inactive (long version with interface index) */
+    {
+        .response = "^SWWAN: 3,0,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_UNKNOWN      },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED }
+        }
+    },
+    /* Multiple PDP contexts active (short version without interface index) */
+    {
+        .response = "^SWWAN: 2,1\r\n^SWWAN: 3,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED }
+        }
+    },
+    /* Multiple PDP contexts active (long version with interface index) */
+    {
+        .response = "^SWWAN: 2,1,3\r\n^SWWAN: 3,1,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED }
+        }
+    },
+    /* Multiple PDP contexts inactive (short version without interface index) */
+    {
+        .response = "^SWWAN: 2,0\r\n^SWWAN: 3,0\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED }
+        }
+    },
+    /* Multiple PDP contexts inactive (long version with interface index) */
+    {
+        .response = "^SWWAN: 2,0,3\r\n^SWWAN: 3,0,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED }
+        }
+    },
+    /* Multiple PDP contexts active/inactive (short version without interface index) */
+    {
+        .response = "^SWWAN: 2,0\r\n^SWWAN: 3,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED    }
+        }
+    },
+    /* Multiple PDP contexts active/inactive (long version with interface index) */
+    {
+        .response = "^SWWAN: 2,0,3\r\n^SWWAN: 3,1,1\r\n",
+        .expected_items = {
+            { .cid = 2, .state = MM_BEARER_CONNECTION_STATUS_DISCONNECTED },
+            { .cid = 3, .state = MM_BEARER_CONNECTION_STATUS_CONNECTED    }
+        }
+    }
+};
+
+static void
+test_swwan_pls8 (void)
+{
+    MMBearerConnectionStatus  read_state;
+    GError                   *error = NULL;
+    guint                     i;
+
+    /* Base tests for successful responses */
+    for (i = 0; i < G_N_ELEMENTS (swwan_tests); i++) {
+        guint j;
+
+        /* Query for the expected items (CIDs 2 and 3) */
+        for (j = 0; j < SWWAN_TEST_MAX_CIDS; j++) {
+            read_state = mm_cinterion_parse_swwan_response (swwan_tests[i].response, swwan_tests[i].expected_items[j].cid, &error);
+            if (swwan_tests[i].expected_items[j].state == MM_BEARER_CONNECTION_STATUS_UNKNOWN) {
+                g_assert_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED);
+                g_clear_error (&error);
+            } else
+                g_assert_no_error (error);
+            g_assert_cmpint (read_state, ==, swwan_tests[i].expected_items[j].state);
+        }
+
+        /* Query for a CID which isn't replied (e.g. 12) */
+        if (!swwan_tests[i].skip_test_other_cids) {
+            read_state = mm_cinterion_parse_swwan_response (swwan_tests[i].response, 12, &error);
+            g_assert_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED);
+            g_assert_cmpint (read_state, ==, MM_BEARER_CONNECTION_STATUS_UNKNOWN);
+            g_clear_error (&error);
+        }
+    }
+
+    /* Additional tests for errors */
+    read_state = mm_cinterion_parse_swwan_response ("^GARBAGE", 2, &error);
+    g_assert_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED);
+    g_assert_cmpint (read_state, ==, MM_BEARER_CONNECTION_STATUS_UNKNOWN);
+    g_clear_error (&error);
+}
 
 /*****************************************************************************/
 /* Test ^SIND responses */
@@ -500,7 +693,6 @@ int main (int argc, char **argv)
 {
     setlocale (LC_ALL, "");
 
-    g_type_init ();
     g_test_init (&argc, &argv, NULL);
 
     g_test_add_func ("/MM/cinterion/scfg",                    test_scfg);
@@ -509,6 +701,8 @@ int main (int argc, char **argv)
     g_test_add_func ("/MM/cinterion/scfg/response/2g",        test_scfg_response_2g);
     g_test_add_func ("/MM/cinterion/scfg/response/2g/ucs2",   test_scfg_response_2g_ucs2);
     g_test_add_func ("/MM/cinterion/cnmi/phs8",               test_cnmi_phs8);
+    g_test_add_func ("/MM/cinterion/cnmi/other",              test_cnmi_other);
+    g_test_add_func ("/MM/cinterion/swwan/pls8",              test_swwan_pls8);
     g_test_add_func ("/MM/cinterion/sind/response/simstatus", test_sind_response_simstatus);
     g_test_add_func ("/MM/cinterion/smong/response/tc63i",    test_smong_response_tc63i);
     g_test_add_func ("/MM/cinterion/smong/response/other",    test_smong_response_other);
