@@ -89,31 +89,24 @@ modem_create_bearer_finish (MMIfaceModem *self,
                             GAsyncResult *res,
                             GError **error)
 {
-    MMBaseBearer *bearer;
-
-    bearer = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    mm_dbg ("New MBM bearer created at DBus path '%s'", mm_base_bearer_get_path (bearer));
-
-    return g_object_ref (bearer);
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
 static void
 broadband_bearer_mbm_new_ready (GObject *source,
                                 GAsyncResult *res,
-                                GSimpleAsyncResult *simple)
+                                GTask *task)
 {
     MMBaseBearer *bearer = NULL;
     GError *error = NULL;
 
     bearer = mm_broadband_bearer_mbm_new_finish (res, &error);
     if (!bearer)
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gpointer (simple,
-                                                   bearer,
-                                                   (GDestroyNotify)g_object_unref);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_pointer (task, bearer, g_object_unref);
+
+    g_object_unref (task);
 }
 
 static void
@@ -122,19 +115,12 @@ modem_create_bearer (MMIfaceModem *self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_create_bearer);
-
     mm_dbg ("Creating MBM bearer...");
     mm_broadband_bearer_mbm_new (MM_BROADBAND_MODEM_MBM (self),
                                  properties,
                                  NULL, /* cancellable */
                                  (GAsyncReadyCallback)broadband_bearer_mbm_new_ready,
-                                 result);
+                                 g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -168,14 +154,14 @@ modem_after_sim_unlock_finish (MMIfaceModem *self,
                                GAsyncResult *res,
                                GError **error)
 {
-    return TRUE;
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static gboolean
-after_sim_unlock_wait_cb (GSimpleAsyncResult *result)
+after_sim_unlock_wait_cb (GTask *task)
 {
-    g_simple_async_result_complete (result);
-    g_object_unref (result);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
     return G_SOURCE_REMOVE;
 }
 
@@ -184,15 +170,12 @@ modem_after_sim_unlock (MMIfaceModem *self,
                         GAsyncReadyCallback callback,
                         gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_after_sim_unlock);
+    task = g_task_new (self, NULL, callback, user_data);
 
     /* wait so sim pin is done */
-    g_timeout_add (500, (GSourceFunc)after_sim_unlock_wait_cb, result);
+    g_timeout_add (500, (GSourceFunc)after_sim_unlock_wait_cb, task);
 }
 
 /*****************************************************************************/
@@ -275,45 +258,22 @@ load_current_modes_finish (MMIfaceModem *_self,
 {
     MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
     const gchar *response;
-    guint a;
+    gint mbm_mode = -1;
+
+    g_assert (allowed);
+    g_assert (preferred);
 
     response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, error);
-    if (!response)
+    if (!response || !mm_mbm_parse_cfun_query_current_modes (response, allowed, &mbm_mode, error))
         return FALSE;
 
-    if (mm_get_uint_from_str (mm_strip_tag (response, "+CFUN:"), &a)) {
-        /* No settings to set preferred */
-        *preferred = MM_MODEM_MODE_NONE;
+    /* No settings to set preferred */
+    *preferred = MM_MODEM_MODE_NONE;
 
-        switch (a) {
-        case MBM_NETWORK_MODE_OFFLINE:
-        case MBM_NETWORK_MODE_LOW_POWER:
-            /* Do not update internal mbm_mode */
-            *allowed = MM_MODEM_MODE_NONE;
-            break;
-        case MBM_NETWORK_MODE_2G:
-            self->priv->mbm_mode = MBM_NETWORK_MODE_2G;
-            *allowed = MM_MODEM_MODE_2G;
-            break;
-        case MBM_NETWORK_MODE_3G:
-            self->priv->mbm_mode = MBM_NETWORK_MODE_3G;
-            *allowed = MM_MODEM_MODE_3G;
-            break;
-        default:
-            /* Do not update internal mbm_mode */
-            *allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
-            break;
-        }
+    if (mbm_mode != -1)
+        self->priv->mbm_mode = mbm_mode;
 
-        return TRUE;
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_FAILED,
-                 "Couldn't parse +CFUN response: '%s'",
-                 response);
-    return FALSE;
+    return TRUE;
 }
 
 static void
@@ -333,45 +293,37 @@ load_current_modes (MMIfaceModem *self,
 /* Set allowed modes (Modem interface) */
 
 typedef struct {
-    MMBroadbandModemMbm *self;
-    GSimpleAsyncResult *result;
     gint mbm_mode;
 } SetCurrentModesContext;
 
-static void
-set_current_modes_context_complete_and_free (SetCurrentModesContext *ctx)
-{
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_slice_free (SetCurrentModesContext, ctx);
-}
-
 static gboolean
 set_current_modes_finish (MMIfaceModem *self,
                           GAsyncResult *res,
                           GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 allowed_mode_update_ready (MMBaseModem *self,
                            GAsyncResult *res,
-                           SetCurrentModesContext *ctx)
+                           GTask *task)
 {
+    SetCurrentModesContext *ctx;
     GError *error = NULL;
 
+    ctx = g_task_get_task_data (task);
+
     mm_base_modem_at_command_finish (self, res, &error);
     if (error)
         /* Let the error be critical. */
-        g_simple_async_result_take_error (ctx->result, error);
+        g_task_return_error (task, error);
     else {
         /* Cache current allowed mode */
-        ctx->self->priv->mbm_mode = ctx->mbm_mode;
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        MM_BROADBAND_MODEM_MBM (self)->priv->mbm_mode = ctx->mbm_mode;
+        g_task_return_boolean (task, TRUE);
     }
-    set_current_modes_context_complete_and_free (ctx);
+    g_object_unref (task);
 }
 
 static void
@@ -382,16 +334,15 @@ set_current_modes (MMIfaceModem *self,
                    gpointer user_data)
 {
     SetCurrentModesContext *ctx;
+    GTask *task;
     gchar *command;
 
-    ctx = g_slice_new (SetCurrentModesContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             set_current_modes);
+    ctx = g_new (SetCurrentModesContext, 1);
     ctx->mbm_mode = -1;
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
     if (allowed == MM_MODEM_MODE_2G)
         ctx->mbm_mode = MBM_NETWORK_MODE_2G;
     else if (allowed == MM_MODEM_MODE_3G)
@@ -407,17 +358,16 @@ set_current_modes (MMIfaceModem *self,
 
         allowed_str = mm_modem_mode_build_string_from_mask (allowed);
         preferred_str = mm_modem_mode_build_string_from_mask (preferred);
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Requested mode (allowed: '%s', preferred: '%s') not "
-                                         "supported by the modem.",
-                                         allowed_str,
-                                         preferred_str);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Requested mode (allowed: '%s', preferred: '%s') not "
+                                 "supported by the modem.",
+                                 allowed_str,
+                                 preferred_str);
+        g_object_unref (task);
         g_free (allowed_str);
         g_free (preferred_str);
-
-        set_current_modes_context_complete_and_free (ctx);
         return;
     }
 
@@ -428,44 +378,30 @@ set_current_modes (MMIfaceModem *self,
         3,
         FALSE,
         (GAsyncReadyCallback)allowed_mode_update_ready,
-        ctx);
+        task);
     g_free (command);
 }
 
 /*****************************************************************************/
 /* Initializing the modem (during first enabling) */
 
-typedef struct {
-    GSimpleAsyncResult *result;
-    MMBroadbandModemMbm *self;
-} EnablingModemInitContext;
-
-static void
-enabling_modem_init_context_complete_and_free (EnablingModemInitContext *ctx)
-{
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_slice_free (EnablingModemInitContext, ctx);
-}
-
 static gboolean
 enabling_modem_init_finish (MMBroadbandModem *self,
                             GAsyncResult *res,
                             GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 enabling_init_sequence_ready (MMBaseModem *self,
                               GAsyncResult *res,
-                              EnablingModemInitContext *ctx)
+                              GTask *task)
 {
     /* Ignore errors */
     mm_base_modem_at_sequence_full_finish (self, res, NULL, NULL);
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    enabling_modem_init_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 static const MMBaseModemAtCommand enabling_modem_init_sequence[] = {
@@ -477,22 +413,25 @@ static const MMBaseModemAtCommand enabling_modem_init_sequence[] = {
 };
 
 static void
-run_enabling_init_sequence (EnablingModemInitContext *ctx)
+run_enabling_init_sequence (GTask *task)
 {
-    mm_base_modem_at_sequence_full (MM_BASE_MODEM (ctx->self),
-                                    mm_base_modem_peek_port_primary (MM_BASE_MODEM (ctx->self)),
+    MMBaseModem *self;
+
+    self = g_task_get_source_object (task);
+    mm_base_modem_at_sequence_full (self,
+                                    mm_base_modem_peek_port_primary (self),
                                     enabling_modem_init_sequence,
                                     NULL,  /* response_processor_context */
                                     NULL,  /* response_processor_context_free */
                                     NULL, /* cancellable */
                                     (GAsyncReadyCallback)enabling_init_sequence_ready,
-                                    ctx);
+                                    task);
 }
 
 static void
 emrdy_ready (MMBaseModem *self,
              GAsyncResult *res,
-             EnablingModemInitContext *ctx)
+             GTask *task)
 {
     GError *error = NULL;
 
@@ -511,30 +450,26 @@ emrdy_ready (MMBaseModem *self,
                              MM_SERIAL_ERROR_RESPONSE_TIMEOUT))
             mm_warn ("timed out waiting for EMRDY response.");
         else
-            ctx->self->priv->have_emrdy = TRUE;
+            MM_BROADBAND_MODEM_MBM (self)->priv->have_emrdy = TRUE;
         g_error_free (error);
     }
 
-    run_enabling_init_sequence (ctx);
+    run_enabling_init_sequence (task);
 }
 
 static void
-enabling_modem_init (MMBroadbandModem *self,
+enabling_modem_init (MMBroadbandModem *_self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
-    EnablingModemInitContext *ctx;
+    MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
+    GTask *task;
 
-    ctx = g_slice_new0 (EnablingModemInitContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             enabling_modem_init);
-    ctx->self = g_object_ref (self);
+    task = g_task_new (self, NULL, callback, user_data);
 
     /* Modem is ready?, no need to check EMRDY */
-    if (ctx->self->priv->have_emrdy) {
-        run_enabling_init_sequence (ctx);
+    if (self->priv->have_emrdy) {
+        run_enabling_init_sequence (task);
         return;
     }
 
@@ -543,7 +478,7 @@ enabling_modem_init (MMBroadbandModem *self,
                               3,
                               FALSE,
                               (GAsyncReadyCallback)emrdy_ready,
-                              ctx);
+                              task);
 }
 
 /*****************************************************************************/
@@ -615,36 +550,14 @@ load_power_state_finish (MMIfaceModem *self,
                          GAsyncResult *res,
                          GError **error)
 {
-    const gchar *response;
-    guint a;
+    const gchar       *response;
+    MMModemPowerState  state;
 
     response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, error);
-    if (!response)
-        return FALSE;
+    if (!response || !mm_mbm_parse_cfun_query_power_state (response, &state, error))
+        return MM_MODEM_POWER_STATE_UNKNOWN;
 
-    if (mm_get_uint_from_str (mm_strip_tag (response, "+CFUN:"), &a)) {
-        switch (a) {
-        case MBM_NETWORK_MODE_OFFLINE:
-            return MM_MODEM_POWER_STATE_OFF;
-
-        case MBM_NETWORK_MODE_LOW_POWER:
-            return MM_MODEM_POWER_STATE_LOW;
-
-        case MBM_NETWORK_MODE_ANY:
-        case MBM_NETWORK_MODE_2G:
-        case MBM_NETWORK_MODE_3G:
-            return MM_MODEM_POWER_STATE_ON;
-        default:
-            break;
-        }
-    }
-
-    g_set_error (error,
-                 MM_CORE_ERROR,
-                 MM_CORE_ERROR_FAILED,
-                 "Couldn't parse +CFUN response: '%s'",
-                 response);
-    return MM_MODEM_POWER_STATE_UNKNOWN;
+    return state;
 }
 
 static void
@@ -907,7 +820,7 @@ set_unsolicited_events_handlers (MMBroadbandModemMbm *self,
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
     /* Enable unsolicited events in given port */
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
@@ -940,26 +853,24 @@ modem_3gpp_setup_cleanup_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                                     GAsyncResult *res,
                                                     GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 parent_setup_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                        GAsyncResult *res,
-                                       GSimpleAsyncResult *simple)
+                                       GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->setup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else {
         /* Our own setup now */
         set_unsolicited_events_handlers (MM_BROADBAND_MODEM_MBM (self), TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_task_return_boolean (task, TRUE);
     }
-
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 static void
@@ -967,33 +878,26 @@ modem_3gpp_setup_unsolicited_events (MMIfaceModem3gpp *self,
                                      GAsyncReadyCallback callback,
                                      gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_setup_unsolicited_events);
-
     /* Chain up parent's setup */
     iface_modem_3gpp_parent->setup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_setup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 static void
 parent_cleanup_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                          GAsyncResult *res,
-                                         GSimpleAsyncResult *simple)
+                                         GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->cleanup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
@@ -1001,13 +905,6 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp *self,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_cleanup_unsolicited_events);
-
     /* Our own cleanup first */
     set_unsolicited_events_handlers (MM_BROADBAND_MODEM_MBM (self), FALSE);
 
@@ -1015,7 +912,7 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp *self,
     iface_modem_3gpp_parent->cleanup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_cleanup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -1026,23 +923,23 @@ modem_3gpp_enable_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                              GAsyncResult *res,
                                              GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 own_enable_unsolicited_events_ready (MMBaseModem *self,
                                      GAsyncResult *res,
-                                     GSimpleAsyncResult *simple)
+                                     GTask *task)
 {
     GError *error = NULL;
 
     mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
     if (error)
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static const MMBaseModemAtCommand unsolicited_enable_sequence[] = {
@@ -1054,14 +951,13 @@ static const MMBaseModemAtCommand unsolicited_enable_sequence[] = {
 static void
 parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                         GAsyncResult *res,
-                                        GSimpleAsyncResult *simple)
+                                        GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->enable_unsolicited_events_finish (self, res, &error)) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -1074,7 +970,7 @@ parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
         NULL,  /* response_processor_context_free */
         NULL, /* cancellable */
         (GAsyncReadyCallback)own_enable_unsolicited_events_ready,
-        simple);
+        task);
 }
 
 static void
@@ -1082,18 +978,11 @@ modem_3gpp_enable_unsolicited_events (MMIfaceModem3gpp *self,
                                       GAsyncReadyCallback callback,
                                       gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_enable_unsolicited_events);
-
     /* Chain up parent's enable */
     iface_modem_3gpp_parent->enable_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_enable_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -1104,36 +993,35 @@ modem_3gpp_disable_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                               GAsyncResult *res,
                                               GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 parent_disable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                          GAsyncResult *res,
-                                         GSimpleAsyncResult *simple)
+                                         GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->disable_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
 own_disable_unsolicited_events_ready (MMBaseModem *self,
                                       GAsyncResult *res,
-                                      GSimpleAsyncResult *simple)
+                                      GTask *task)
 {
     GError *error = NULL;
 
     mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
     if (error) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -1141,7 +1029,7 @@ own_disable_unsolicited_events_ready (MMBaseModem *self,
     iface_modem_3gpp_parent->disable_unsolicited_events (
         MM_IFACE_MODEM_3GPP (self),
         (GAsyncReadyCallback)parent_disable_unsolicited_events_ready,
-        simple);
+        task);
 }
 
 static const MMBaseModemAtCommand unsolicited_disable_sequence[] = {
@@ -1155,13 +1043,6 @@ modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp *self,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_disable_unsolicited_events);
-
     /* Our own disable first */
     mm_base_modem_at_sequence_full (
         MM_BASE_MODEM (self),
@@ -1171,7 +1052,7 @@ modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp *self,
         NULL, /* response_processor_context_free */
         NULL, /* cancellable */
         (GAsyncReadyCallback)own_disable_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -1182,25 +1063,29 @@ location_load_capabilities_finish (MMIfaceModemLocation *self,
                                    GAsyncResult *res,
                                    GError **error)
 {
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return MM_MODEM_LOCATION_SOURCE_NONE;
+    GError *inner_error = NULL;
+    gssize value;
 
-    return (MMModemLocationSource) GPOINTER_TO_UINT (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    value = g_task_propagate_int (G_TASK (res), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return MM_MODEM_LOCATION_SOURCE_NONE;
+    }
+    return (MMModemLocationSource)value;
 }
 
 static void
 parent_load_capabilities_ready (MMIfaceModemLocation *self,
                                 GAsyncResult *res,
-                                GSimpleAsyncResult *simple)
+                                GTask *task)
 {
     MMModemLocationSource sources;
     GError *error = NULL;
 
     sources = iface_modem_location_parent->load_capabilities_finish (self, res, &error);
     if (error) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -1211,11 +1096,8 @@ parent_load_capabilities_ready (MMIfaceModemLocation *self,
                     MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED);
 
     /* So we're done, complete */
-    g_simple_async_result_set_op_res_gpointer (simple,
-                                               GUINT_TO_POINTER (sources),
-                                               NULL);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_task_return_int (task, sources);
+    g_object_unref (task);
 }
 
 static void
@@ -1223,37 +1105,20 @@ location_load_capabilities (MMIfaceModemLocation *self,
                             GAsyncReadyCallback callback,
                             gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        location_load_capabilities);
-
     /* Chain up parent's setup */
-    iface_modem_location_parent->load_capabilities (self,
-                                                    (GAsyncReadyCallback)parent_load_capabilities_ready,
-                                                    result);
+    iface_modem_location_parent->load_capabilities (
+        self,
+        (GAsyncReadyCallback)parent_load_capabilities_ready,
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
 /* Enable/Disable location gathering (Location interface) */
 
 typedef struct {
-    MMBroadbandModemMbm *self;
-    GSimpleAsyncResult *result;
     MMModemLocationSource source;
 } LocationGatheringContext;
 
-static void
-location_gathering_context_complete_and_free (LocationGatheringContext *ctx)
-{
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_slice_free (LocationGatheringContext, ctx);
-}
-
 /******************************/
 /* Disable location gathering */
 
@@ -1262,21 +1127,21 @@ disable_location_gathering_finish (MMIfaceModemLocation *self,
                                    GAsyncResult *res,
                                    GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 gps_disabled_ready (MMBaseModem *self,
                     GAsyncResult *res,
-                    LocationGatheringContext *ctx)
+                    GTask *task)
 {
+    LocationGatheringContext *ctx;
     MMPortSerialGps *gps_port;
     GError *error = NULL;
 
-    if (!mm_base_modem_at_command_full_finish (self, res, &error))
-        g_simple_async_result_take_error (ctx->result, error);
-    else
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+    ctx = g_task_get_task_data (task);
+
+    mm_base_modem_at_command_full_finish (self, res, &error);
 
     /* Only use the GPS port in NMEA/RAW setups */
     if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
@@ -1287,7 +1152,12 @@ gps_disabled_ready (MMBaseModem *self,
             mm_port_serial_close (MM_PORT_SERIAL (gps_port));
     }
 
-    location_gathering_context_complete_and_free (ctx);
+    if (error)
+        g_task_return_error (task, error);
+    else
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
@@ -1299,15 +1169,14 @@ disable_location_gathering (MMIfaceModemLocation *_self,
     MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
     gboolean stop_gps = FALSE;
     LocationGatheringContext *ctx;
+    GTask *task;
 
-    ctx = g_slice_new (LocationGatheringContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             disable_location_gathering);
+    ctx = g_new (LocationGatheringContext, 1);
     ctx->source = source;
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
     /* Only stop GPS engine if no GPS-related sources enabled */
     if (source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
                   MM_MODEM_LOCATION_SOURCE_GPS_RAW |
@@ -1329,13 +1198,13 @@ disable_location_gathering (MMIfaceModemLocation *_self,
                                        FALSE, /* raw */
                                        NULL, /* cancellable */
                                        (GAsyncReadyCallback)gps_disabled_ready,
-                                       ctx);
+                                       task);
         return;
     }
 
     /* For any other location (e.g. 3GPP), or if still some GPS needed, just return */
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    location_gathering_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 /*****************************************************************************/
@@ -1346,23 +1215,26 @@ enable_location_gathering_finish (MMIfaceModemLocation *self,
                                   GAsyncResult *res,
                                   GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 gps_enabled_ready (MMBaseModem *self,
                    GAsyncResult *res,
-                   LocationGatheringContext *ctx)
+                   GTask *task)
 {
+    LocationGatheringContext *ctx;
     GError *error = NULL;
     MMPortSerialGps *gps_port;
 
     if (!mm_base_modem_at_command_full_finish (self, res, &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        location_gathering_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
+    ctx = g_task_get_task_data (task);
+
     /* Only use the GPS port in NMEA/RAW setups */
     if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
                        MM_MODEM_LOCATION_SOURCE_GPS_RAW)) {
@@ -1370,12 +1242,12 @@ gps_enabled_ready (MMBaseModem *self,
         if (!gps_port ||
             !mm_port_serial_open (MM_PORT_SERIAL (gps_port), &error)) {
             if (error)
-                g_simple_async_result_take_error (ctx->result, error);
+                g_task_return_error (task, error);
             else
-                g_simple_async_result_set_error (ctx->result,
-                                                 MM_CORE_ERROR,
-                                                 MM_CORE_ERROR_FAILED,
-                                                 "Couldn't open raw GPS serial port");
+                g_task_return_new_error (task,
+                                         MM_CORE_ERROR,
+                                         MM_CORE_ERROR_FAILED,
+                                         "Couldn't open raw GPS serial port");
         } else {
             GByteArray *buf;
             const gchar *command = "ATE0*E2GPSNPD\r\n";
@@ -1395,41 +1267,44 @@ gps_enabled_ready (MMBaseModem *self,
                                     NULL,
                                     NULL);
             g_byte_array_unref (buf);
-            g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+            g_task_return_boolean (task, TRUE);
         }
 
     } else
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        g_task_return_boolean (task, TRUE);
 
-    location_gathering_context_complete_and_free (ctx);
+    g_object_unref (task);
 }
 
 static void
-parent_enable_location_gathering_ready (MMIfaceModemLocation *self,
+parent_enable_location_gathering_ready (MMIfaceModemLocation *_self,
                                         GAsyncResult *res,
-                                        LocationGatheringContext *ctx)
+                                        GTask *task)
 {
+    MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
+    LocationGatheringContext *ctx;
     gboolean start_gps = FALSE;
     GError *error = NULL;
 
-    if (!iface_modem_location_parent->enable_location_gathering_finish (self, res, &error)) {
-        g_simple_async_result_take_error (ctx->result, error);
-        location_gathering_context_complete_and_free (ctx);
+    if (!iface_modem_location_parent->enable_location_gathering_finish (_self, res, &error)) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Now our own enabling */
 
     /* NMEA and RAW are both enabled in the same way */
+    ctx = g_task_get_task_data (task);
     if (ctx->source & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
                        MM_MODEM_LOCATION_SOURCE_GPS_RAW |
                        MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)) {
         /* Only start GPS engine if not done already */
-        if (!(ctx->self->priv->enabled_sources & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
-                                                  MM_MODEM_LOCATION_SOURCE_GPS_RAW |
-                                                  MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)))
+        if (!(self->priv->enabled_sources & (MM_MODEM_LOCATION_SOURCE_GPS_NMEA |
+                                             MM_MODEM_LOCATION_SOURCE_GPS_RAW |
+                                             MM_MODEM_LOCATION_SOURCE_GPS_UNMANAGED)))
             start_gps = TRUE;
-        ctx->self->priv->enabled_sources |= ctx->source;
+        self->priv->enabled_sources |= ctx->source;
     }
 
     if (start_gps) {
@@ -1441,13 +1316,13 @@ parent_enable_location_gathering_ready (MMIfaceModemLocation *self,
                                        FALSE, /* raw */
                                        NULL, /* cancellable */
                                        (GAsyncReadyCallback)gps_enabled_ready,
-                                       ctx);
+                                       task);
         return;
     }
 
     /* For any other location (e.g. 3GPP), or if GPS already running just return */
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    location_gathering_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 static void
@@ -1457,20 +1332,19 @@ enable_location_gathering (MMIfaceModemLocation *self,
                            gpointer user_data)
 {
     LocationGatheringContext *ctx;
+    GTask *task;
 
-    ctx = g_slice_new (LocationGatheringContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             enable_location_gathering);
+    ctx = g_new (LocationGatheringContext, 1);
     ctx->source = source;
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
     /* Chain up parent's gathering enable */
     iface_modem_location_parent->enable_location_gathering (self,
                                                             source,
                                                             (GAsyncReadyCallback)parent_enable_location_gathering_ready,
-                                                            ctx);
+                                                            task);
 }
 
 /*****************************************************************************/
@@ -1507,7 +1381,7 @@ setup_ports (MMBroadbandModem *_self)
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
     /* Setup unsolicited handlers which should be always on */
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
