@@ -65,16 +65,13 @@ load_supported_modes_finish (MMIfaceModem *self,
                              GAsyncResult *res,
                              GError **error)
 {
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    return g_array_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
 static void
 parent_load_supported_modes_ready (MMIfaceModem *self,
                                    GAsyncResult *res,
-                                   GSimpleAsyncResult *simple)
+                                   GTask *task)
 {
     GError *error = NULL;
     GArray *all;
@@ -84,9 +81,8 @@ parent_load_supported_modes_ready (MMIfaceModem *self,
 
     all = iface_modem_parent->load_supported_modes_finish (self, res, &error);
     if (!all) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -119,9 +115,8 @@ parent_load_supported_modes_ready (MMIfaceModem *self,
     g_array_unref (all);
     g_array_unref (combinations);
 
-    g_simple_async_result_set_op_res_gpointer (simple, filtered, (GDestroyNotify) g_array_unref);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_task_return_pointer (task, filtered, (GDestroyNotify) g_array_unref);
+    g_object_unref (task);
 }
 
 static void
@@ -133,10 +128,7 @@ load_supported_modes (MMIfaceModem *self,
     iface_modem_parent->load_supported_modes (
         MM_IFACE_MODEM (self),
         (GAsyncReadyCallback)parent_load_supported_modes_ready,
-        g_simple_async_result_new (G_OBJECT (self),
-                                   callback,
-                                   user_data,
-                                   load_supported_modes));
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -222,24 +214,24 @@ set_current_modes_finish (MMIfaceModem *self,
                           GAsyncResult *res,
                           GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
-allowed_mode_update_ready (MMBroadbandModemOption *self,
+allowed_mode_update_ready (MMBaseModem *self,
                            GAsyncResult *res,
-                           GSimpleAsyncResult *operation_result)
+                           GTask *task)
 {
     GError *error = NULL;
 
-    mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
+    mm_base_modem_at_command_finish (self, res, &error);
     if (error)
         /* Let the error be critical. */
-        g_simple_async_result_take_error (operation_result, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (operation_result, TRUE);
-    g_simple_async_result_complete (operation_result);
-    g_object_unref (operation_result);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
@@ -249,14 +241,11 @@ set_current_modes (MMIfaceModem *self,
                    GAsyncReadyCallback callback,
                    gpointer user_data)
 {
-    GSimpleAsyncResult *result;
+    GTask *task;
     gchar *command;
     gint option_mode = -1;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        set_current_modes);
+    task = g_task_new (self, NULL, callback, user_data);
 
     if (allowed == MM_MODEM_MODE_2G)
         option_mode = 0;
@@ -278,18 +267,16 @@ set_current_modes (MMIfaceModem *self,
 
         allowed_str = mm_modem_mode_build_string_from_mask (allowed);
         preferred_str = mm_modem_mode_build_string_from_mask (preferred);
-        g_simple_async_result_set_error (result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Requested mode (allowed: '%s', preferred: '%s') not "
-                                         "supported by the modem.",
-                                         allowed_str,
-                                         preferred_str);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Requested mode (allowed: '%s', preferred: '%s') not "
+                                 "supported by the modem.",
+                                 allowed_str,
+                                 preferred_str);
+        g_object_unref (task);
         g_free (allowed_str);
         g_free (preferred_str);
-
-        g_simple_async_result_complete_in_idle (result);
-        g_object_unref (result);
         return;
     }
 
@@ -300,7 +287,7 @@ set_current_modes (MMIfaceModem *self,
         3,
         FALSE,
         (GAsyncReadyCallback)allowed_mode_update_ready,
-        result);
+        task);
     g_free (command);
 }
 
@@ -316,24 +303,13 @@ typedef enum {
 } AccessTechnologiesStep;
 
 typedef struct {
-    MMBroadbandModemOption *self;
-    GSimpleAsyncResult *result;
     MMModemAccessTechnology access_technology;
     gboolean check_2g;
     gboolean check_3g;
     AccessTechnologiesStep step;
 } AccessTechnologiesContext;
 
-static void load_access_technologies_step (AccessTechnologiesContext *ctx);
-
-static void
-access_technologies_context_complete_and_free (AccessTechnologiesContext *ctx)
-{
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_free (ctx);
-}
+static void load_access_technologies_step (GTask *task);
 
 static gboolean
 load_access_technologies_finish (MMIfaceModem *self,
@@ -342,12 +318,17 @@ load_access_technologies_finish (MMIfaceModem *self,
                                  guint *mask,
                                  GError **error)
 {
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+    GError *inner_error = NULL;
+    gssize value;
+
+    value = g_task_propagate_int (G_TASK (res), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
         return FALSE;
+    }
 
     /* We are reporting ALL 3GPP access technologies here */
-    *access_technologies = (MMModemAccessTechnology) GPOINTER_TO_UINT (
-        g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+    *access_technologies = (MMModemAccessTechnology) value;
     *mask = MM_IFACE_MODEM_3GPP_ALL_ACCESS_TECHNOLOGIES_MASK;
     return TRUE;
 }
@@ -407,10 +388,13 @@ parse_ossys_response (const gchar *response,
 static void
 ossys_query_ready (MMBaseModem *self,
                    GAsyncResult *res,
-                   AccessTechnologiesContext *ctx)
+                   GTask *task)
 {
+    AccessTechnologiesContext *ctx;
     const gchar *response;
 
+    ctx = g_task_get_task_data (task);
+
     /* If for some reason the OSSYS request failed, still try to check
      * explicit 2G/3G mode with OCTI and OWCTI; maybe we'll get something.
      */
@@ -429,7 +413,7 @@ ossys_query_ready (MMBaseModem *self,
 
     /* Go on to next step */
     ctx->step++;
-    load_access_technologies_step (ctx);
+    load_access_technologies_step (task);
 }
 
 static gboolean
@@ -487,11 +471,14 @@ parse_octi_response (const gchar *response,
 static void
 octi_query_ready (MMBaseModem *self,
                   GAsyncResult *res,
-                  AccessTechnologiesContext *ctx)
+                  GTask *task)
 {
+    AccessTechnologiesContext *ctx;
     MMModemAccessTechnology octi = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
     const gchar *response;
 
+    ctx = g_task_get_task_data (task);
+
     response = mm_base_modem_at_command_finish (self, res, NULL);
     if (response &&
         parse_octi_response (response, &octi)) {
@@ -504,7 +491,7 @@ octi_query_ready (MMBaseModem *self,
 
     /* Go on to next step */
     ctx->step++;
-    load_access_technologies_step (ctx);
+    load_access_technologies_step (task);
 }
 
 static gboolean
@@ -544,11 +531,14 @@ parse_owcti_response (const gchar *response,
 static void
 owcti_query_ready (MMBaseModem *self,
                    GAsyncResult *res,
-                   AccessTechnologiesContext *ctx)
+                   GTask *task)
 {
+    AccessTechnologiesContext *ctx;
     MMModemAccessTechnology owcti = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
     const gchar *response;
 
+    ctx = g_task_get_task_data (task);
+
     response = mm_base_modem_at_command_finish (self, res, NULL);
     if (response &&
         parse_owcti_response (response, &owcti)) {
@@ -557,34 +547,40 @@ owcti_query_ready (MMBaseModem *self,
 
     /* Go on to next step */
     ctx->step++;
-    load_access_technologies_step (ctx);
+    load_access_technologies_step (task);
 }
 
 static void
-load_access_technologies_step (AccessTechnologiesContext *ctx)
+load_access_technologies_step (GTask *task)
 {
+    MMBroadbandModemOption *self;
+    AccessTechnologiesContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     switch (ctx->step) {
     case ACCESS_TECHNOLOGIES_STEP_FIRST:
         /* Go on to next step */
         ctx->step++;
 
     case ACCESS_TECHNOLOGIES_STEP_OSSYS:
-        mm_base_modem_at_command (MM_BASE_MODEM (ctx->self),
+        mm_base_modem_at_command (MM_BASE_MODEM (self),
                                   "_OSSYS?",
                                   3,
                                   FALSE,
                                   (GAsyncReadyCallback)ossys_query_ready,
-                                  ctx);
+                                  task);
         break;
 
     case ACCESS_TECHNOLOGIES_STEP_OCTI:
         if (ctx->check_2g) {
-            mm_base_modem_at_command (MM_BASE_MODEM (ctx->self),
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
                                       "_OCTI?",
                                       3,
                                       FALSE,
                                       (GAsyncReadyCallback)octi_query_ready,
-                                      ctx);
+                                      task);
             return;
         }
         /* Go on to next step */
@@ -592,12 +588,12 @@ load_access_technologies_step (AccessTechnologiesContext *ctx)
 
     case ACCESS_TECHNOLOGIES_STEP_OWCTI:
         if (ctx->check_3g) {
-            mm_base_modem_at_command (MM_BASE_MODEM (ctx->self),
+            mm_base_modem_at_command (MM_BASE_MODEM (self),
                                       "_OWCTI?",
                                       3,
                                       FALSE,
                                       (GAsyncReadyCallback)owcti_query_ready,
-                                      ctx);
+                                      task);
             return;
         }
         /* Go on to next step */
@@ -605,10 +601,8 @@ load_access_technologies_step (AccessTechnologiesContext *ctx)
 
     case ACCESS_TECHNOLOGIES_STEP_LAST:
         /* All done, set result and complete */
-        g_simple_async_result_set_op_res_gpointer (ctx->result,
-                                                   GUINT_TO_POINTER (ctx->access_technology),
-                                                   NULL);
-        access_technologies_context_complete_and_free (ctx);
+        g_task_return_int (task, ctx->access_technology);
+        g_object_unref (task);
         break;
     }
 }
@@ -622,19 +616,18 @@ run_access_technology_loading_sequence (MMIfaceModem *self,
                                         gpointer user_data)
 {
     AccessTechnologiesContext *ctx;
+    GTask *task;
 
     ctx = g_new (AccessTechnologiesContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             run_access_technology_loading_sequence);
     ctx->step = first;
     ctx->check_2g = check_2g;
     ctx->check_3g = check_3g;
     ctx->access_technology = MM_MODEM_ACCESS_TECHNOLOGY_UNKNOWN;
 
-    load_access_technologies_step (ctx);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
+
+    load_access_technologies_step (task);
 }
 
 static void
@@ -658,46 +651,38 @@ modem_after_power_up_finish (MMIfaceModem *self,
                              GAsyncResult *res,
                              GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static gboolean
-after_power_up_wait_cb (GSimpleAsyncResult *result)
+after_power_up_wait_cb (GTask *task)
 {
-    MMBroadbandModemOption *option;
+    MMBroadbandModemOption *self;
 
-    option = MM_BROADBAND_MODEM_OPTION (g_async_result_get_source_object (G_ASYNC_RESULT (result)));
+    self = g_task_get_source_object (task);
+    self->priv->after_power_up_wait_id = 0;
 
-    g_simple_async_result_set_op_res_gboolean (result, TRUE);
-    g_simple_async_result_complete (result);
-    g_object_unref (result);
-
-    option->priv->after_power_up_wait_id = 0;
-    g_object_unref (option);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 
     return G_SOURCE_REMOVE;
 }
 
 static void
-modem_after_power_up (MMIfaceModem *self,
+modem_after_power_up (MMIfaceModem *_self,
                       GAsyncReadyCallback callback,
                       gpointer user_data)
 {
-    MMBroadbandModemOption *option = MM_BROADBAND_MODEM_OPTION (self);
-    GSimpleAsyncResult *result;
+    MMBroadbandModemOption *self = MM_BROADBAND_MODEM_OPTION (_self);
 
     /* Some Option devices return OK on +CFUN=1 right away but need some time
      * to finish initialization.
      */
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_after_power_up);
-    g_warn_if_fail (option->priv->after_power_up_wait_id == 0);
-    option->priv->after_power_up_wait_id =
+    g_warn_if_fail (self->priv->after_power_up_wait_id == 0);
+    self->priv->after_power_up_wait_id =
         g_timeout_add_seconds (10,
                                (GSourceFunc)after_power_up_wait_cb,
-                               result);
+                               g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -850,7 +835,7 @@ set_unsolicited_events_handlers (MMBroadbandModemOption *self,
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
     /* Enable unsolicited events in given port */
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
@@ -896,26 +881,25 @@ modem_3gpp_setup_cleanup_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                                     GAsyncResult *res,
                                                     GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 parent_setup_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                        GAsyncResult *res,
-                                       GSimpleAsyncResult *simple)
+                                       GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->setup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else {
         /* Our own setup now */
         set_unsolicited_events_handlers (MM_BROADBAND_MODEM_OPTION (self), TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_task_return_boolean (task, TRUE);
     }
 
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 static void
@@ -923,33 +907,26 @@ modem_3gpp_setup_unsolicited_events (MMIfaceModem3gpp *self,
                                      GAsyncReadyCallback callback,
                                      gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_setup_unsolicited_events);
-
     /* Chain up parent's setup */
     iface_modem_3gpp_parent->setup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_setup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 static void
 parent_cleanup_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                          GAsyncResult *res,
-                                         GSimpleAsyncResult *simple)
+                                         GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->cleanup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
@@ -957,13 +934,6 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp *self,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_cleanup_unsolicited_events);
-
     /* Our own cleanup first */
     set_unsolicited_events_handlers (MM_BROADBAND_MODEM_OPTION (self), FALSE);
 
@@ -971,7 +941,7 @@ modem_3gpp_cleanup_unsolicited_events (MMIfaceModem3gpp *self,
     iface_modem_3gpp_parent->cleanup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_cleanup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -982,23 +952,23 @@ modem_3gpp_enable_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                              GAsyncResult *res,
                                              GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 own_enable_unsolicited_events_ready (MMBaseModem *self,
                                      GAsyncResult *res,
-                                     GSimpleAsyncResult *simple)
+                                     GTask *task)
 {
     GError *error = NULL;
 
     mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
     if (error)
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static const MMBaseModemAtCommand unsolicited_enable_sequence[] = {
@@ -1012,14 +982,13 @@ static const MMBaseModemAtCommand unsolicited_enable_sequence[] = {
 static void
 parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                         GAsyncResult *res,
-                                        GSimpleAsyncResult *simple)
+                                        GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->enable_unsolicited_events_finish (self, res, &error)) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
     }
 
     /* Our own enable now */
@@ -1031,7 +1000,7 @@ parent_enable_unsolicited_events_ready (MMIfaceModem3gpp *self,
         NULL, /* response_processor_context_free */
         NULL, /* cancellable */
         (GAsyncReadyCallback)own_enable_unsolicited_events_ready,
-        simple);
+        task);
 }
 
 static void
@@ -1039,18 +1008,11 @@ modem_3gpp_enable_unsolicited_events (MMIfaceModem3gpp *self,
                                       GAsyncReadyCallback callback,
                                       gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_enable_unsolicited_events);
-
     /* Chain up parent's enable */
     iface_modem_3gpp_parent->enable_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_enable_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -1061,7 +1023,7 @@ modem_3gpp_disable_unsolicited_events_finish (MMIfaceModem3gpp *self,
                                               GAsyncResult *res,
                                               GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static const MMBaseModemAtCommand unsolicited_disable_sequence[] = {
@@ -1075,30 +1037,29 @@ static const MMBaseModemAtCommand unsolicited_disable_sequence[] = {
 static void
 parent_disable_unsolicited_events_ready (MMIfaceModem3gpp *self,
                                          GAsyncResult *res,
-                                         GSimpleAsyncResult *simple)
+                                         GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_3gpp_parent->disable_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (simple, TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
 own_disable_unsolicited_events_ready (MMBaseModem *self,
                                       GAsyncResult *res,
-                                      GSimpleAsyncResult *simple)
+                                      GTask *task)
 {
     GError *error = NULL;
 
     mm_base_modem_at_sequence_full_finish (self, res, NULL, &error);
     if (error) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -1106,7 +1067,7 @@ own_disable_unsolicited_events_ready (MMBaseModem *self,
     iface_modem_3gpp_parent->disable_unsolicited_events (
         MM_IFACE_MODEM_3GPP (self),
         (GAsyncReadyCallback)parent_disable_unsolicited_events_ready,
-        simple);
+        task);
 }
 
 static void
@@ -1114,13 +1075,6 @@ modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp *self,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_3gpp_disable_unsolicited_events);
-
     /* Our own disable first */
     mm_base_modem_at_sequence_full (
         MM_BASE_MODEM (self),
@@ -1130,7 +1084,7 @@ modem_3gpp_disable_unsolicited_events (MMIfaceModem3gpp *self,
         NULL, /* response_processor_context_free */
         NULL, /* cancellable */
         (GAsyncReadyCallback)own_disable_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
