@@ -31,30 +31,23 @@
 
 #include "mm-sim-sierra.h"
 
-G_DEFINE_TYPE (MMSimSierra, mm_sim_sierra, MM_TYPE_SIM);
+G_DEFINE_TYPE (MMSimSierra, mm_sim_sierra, MM_TYPE_BASE_SIM)
 
 /*****************************************************************************/
 /* SIM identifier loading */
 
 static gchar *
-load_sim_identifier_finish (MMSim *self,
+load_sim_identifier_finish (MMBaseSim *self,
                             GAsyncResult *res,
                             GError **error)
 {
-    gchar *iccid;
-
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    iccid = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    mm_dbg ("loaded SIM identifier: %s", iccid);
-    return g_strdup (iccid);
+    return g_task_propagate_pointer (G_TASK (res), error);
 }
 
 static void
 iccid_read_ready (MMBaseModem *modem,
                   GAsyncResult *res,
-                  GSimpleAsyncResult *simple)
+                  GTask *task)
 {
     GError *error = NULL;
     const gchar *response;
@@ -64,62 +57,59 @@ iccid_read_ready (MMBaseModem *modem,
 
     response = mm_base_modem_at_command_finish (modem, res, &error);
     if (!response) {
-        g_simple_async_result_take_error (simple, error);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     p = mm_strip_tag (response, "!ICCID:");
     if (!p) {
-        g_simple_async_result_set_error (simple,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Failed to parse !ICCID response: '%s'",
-                                         response);
-        g_simple_async_result_complete (simple);
-        g_object_unref (simple);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Failed to parse !ICCID response: '%s'",
+                                 response);
+        g_object_unref (task);
         return;
     }
 
     parsed = mm_3gpp_parse_iccid (p, &local);
     if (parsed)
-        g_simple_async_result_set_op_res_gpointer (simple, parsed, g_free);
+        g_task_return_pointer (task, parsed, g_free);
     else
-        g_simple_async_result_take_error (simple, local);
+        g_task_return_error (task, local);
 
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 static void
-load_sim_identifier (MMSim *self,
+load_sim_identifier (MMBaseSim *self,
                      GAsyncReadyCallback callback,
                      gpointer user_data)
 {
     MMBaseModem *modem = NULL;
+    GTask *task;
 
     g_object_get (self,
-                  MM_SIM_MODEM, &modem,
+                  MM_BASE_SIM_MODEM, &modem,
                   NULL);
 
+    task = g_task_new (self, NULL, callback, user_data);
+
     mm_dbg ("loading (Sierra) SIM identifier...");
     mm_base_modem_at_command (
-        MM_BASE_MODEM (modem),
+        modem,
         "!ICCID?",
         3,
         FALSE,
         (GAsyncReadyCallback)iccid_read_ready,
-        g_simple_async_result_new (G_OBJECT (self),
-                                   callback,
-                                   user_data,
-                                   load_sim_identifier));
+        task);
     g_object_unref (modem);
 }
 
 /*****************************************************************************/
 
-MMSim *
+MMBaseSim *
 mm_sim_sierra_new_finish (GAsyncResult  *res,
                           GError       **error)
 {
@@ -134,9 +124,9 @@ mm_sim_sierra_new_finish (GAsyncResult  *res,
         return NULL;
 
     /* Only export valid SIMs */
-    mm_sim_export (MM_SIM (sim));
+    mm_base_sim_export (MM_BASE_SIM (sim));
 
-    return MM_SIM (sim);
+    return MM_BASE_SIM (sim);
 }
 
 void
@@ -150,7 +140,7 @@ mm_sim_sierra_new (MMBaseModem *modem,
                                 cancellable,
                                 callback,
                                 user_data,
-                                MM_SIM_MODEM, modem,
+                                MM_BASE_SIM_MODEM, modem,
                                 NULL);
 }
 
@@ -162,8 +152,8 @@ mm_sim_sierra_init (MMSimSierra *self)
 static void
 mm_sim_sierra_class_init (MMSimSierraClass *klass)
 {
-    MMSimClass *sim_class = MM_SIM_CLASS (klass);
+    MMBaseSimClass *base_sim_class = MM_BASE_SIM_CLASS (klass);
 
-    sim_class->load_sim_identifier = load_sim_identifier;
-    sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
+    base_sim_class->load_sim_identifier = load_sim_identifier;
+    base_sim_class->load_sim_identifier_finish = load_sim_identifier_finish;
 }
