@@ -67,80 +67,74 @@ typedef struct {
 } SetupRegistrationChecksResults;
 
 static gboolean
-setup_registration_checks_finish (MMIfaceModemCdma *self,
-                                  GAsyncResult *res,
-                                  gboolean *skip_qcdm_call_manager_step,
-                                  gboolean *skip_qcdm_hdr_step,
-                                  gboolean *skip_at_cdma_service_status_step,
-                                  gboolean *skip_at_cdma1x_serving_system_step,
-                                  gboolean *skip_detailed_registration_state,
-                                  GError **error)
+setup_registration_checks_finish (MMIfaceModemCdma  *self,
+                                  GAsyncResult      *res,
+                                  gboolean          *skip_qcdm_call_manager_step,
+                                  gboolean          *skip_qcdm_hdr_step,
+                                  gboolean          *skip_at_cdma_service_status_step,
+                                  gboolean          *skip_at_cdma1x_serving_system_step,
+                                  gboolean          *skip_detailed_registration_state,
+                                  GError           **error)
 {
     SetupRegistrationChecksResults *results;
 
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+    results = g_task_propagate_pointer (G_TASK (res), error);
+    if (!results)
         return FALSE;
 
-    results = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    *skip_qcdm_call_manager_step = results->skip_qcdm_call_manager_step;
-    *skip_qcdm_hdr_step = results->skip_qcdm_hdr_step;
-    *skip_at_cdma_service_status_step = results->skip_at_cdma_service_status_step;
+    *skip_qcdm_call_manager_step        = results->skip_qcdm_call_manager_step;
+    *skip_qcdm_hdr_step                 = results->skip_qcdm_hdr_step;
+    *skip_at_cdma_service_status_step   = results->skip_at_cdma_service_status_step;
     *skip_at_cdma1x_serving_system_step = results->skip_at_cdma1x_serving_system_step;
-    *skip_detailed_registration_state = results->skip_detailed_registration_state;
+    *skip_detailed_registration_state   = results->skip_detailed_registration_state;
+
+    g_free (results);
+
     return TRUE;
 }
 
 static void
 parent_setup_registration_checks_ready (MMIfaceModemCdma *self,
-                                        GAsyncResult *res,
-                                        GSimpleAsyncResult *simple)
+                                        GAsyncResult     *res,
+                                        GTask            *task)
 {
-    GError *error = NULL;
-    SetupRegistrationChecksResults results = { 0 };
+    GError                         *error = NULL;
+    SetupRegistrationChecksResults *results;
+
+    results = g_new0 (SetupRegistrationChecksResults, 1);
 
     if (!iface_modem_cdma_parent->setup_registration_checks_finish (self,
                                                                     res,
-                                                                    &results.skip_qcdm_call_manager_step,
-                                                                    &results.skip_qcdm_hdr_step,
-                                                                    &results.skip_at_cdma_service_status_step,
-                                                                    &results.skip_at_cdma1x_serving_system_step,
-                                                                    &results.skip_detailed_registration_state,
+                                                                    &results->skip_qcdm_call_manager_step,
+                                                                    &results->skip_qcdm_hdr_step,
+                                                                    &results->skip_at_cdma_service_status_step,
+                                                                    &results->skip_at_cdma1x_serving_system_step,
+                                                                    &results->skip_detailed_registration_state,
                                                                     &error)) {
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     } else {
         /* Skip +CSS */
-        results.skip_at_cdma1x_serving_system_step = TRUE;
+        results->skip_at_cdma1x_serving_system_step = TRUE;
         /* Skip +CAD */
-        results.skip_at_cdma_service_status_step = TRUE;
-
+        results->skip_at_cdma_service_status_step = TRUE;
         /* Force to always use the detailed registration checks, as we have
          * ^SYSINFO for that */
-        results.skip_detailed_registration_state = FALSE;
-
-        g_simple_async_result_set_op_res_gpointer (simple, &results, NULL);
+        results->skip_detailed_registration_state = FALSE;
+        g_task_return_pointer (task, results, g_free);
     }
-
-    /* All done. NOTE: complete NOT in idle! */
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 static void
-setup_registration_checks (MMIfaceModemCdma *self,
-                           GAsyncReadyCallback callback,
-                           gpointer user_data)
+setup_registration_checks (MMIfaceModemCdma    *self,
+                           GAsyncReadyCallback  callback,
+                           gpointer             user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        setup_registration_checks);
-
     /* Run parent's checks first */
-    iface_modem_cdma_parent->setup_registration_checks (self,
-                                                        (GAsyncReadyCallback)parent_setup_registration_checks_ready,
-                                                        result);
+    iface_modem_cdma_parent->setup_registration_checks (
+        self,
+        (GAsyncReadyCallback)parent_setup_registration_checks_ready,
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -151,61 +145,48 @@ typedef struct {
     MMModemCdmaRegistrationState detailed_evdo_state;
 } DetailedRegistrationStateResults;
 
-typedef struct {
-    MMBroadbandModem *self;
-    GSimpleAsyncResult *result;
-    DetailedRegistrationStateResults state;
-} DetailedRegistrationStateContext;
-
-static void
-detailed_registration_state_context_complete_and_free (DetailedRegistrationStateContext *ctx)
-{
-    /* Always not in idle! we're passing a struct in stack as result */
-    g_simple_async_result_complete (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
-    g_free (ctx);
-}
-
 static gboolean
-get_detailed_registration_state_finish (MMIfaceModemCdma *self,
-                                        GAsyncResult *res,
-                                        MMModemCdmaRegistrationState *detailed_cdma1x_state,
-                                        MMModemCdmaRegistrationState *detailed_evdo_state,
-                                        GError **error)
+get_detailed_registration_state_finish (MMIfaceModemCdma              *self,
+                                        GAsyncResult                  *res,
+                                        MMModemCdmaRegistrationState  *detailed_cdma1x_state,
+                                        MMModemCdmaRegistrationState  *detailed_evdo_state,
+                                        GError                       **error)
 {
     DetailedRegistrationStateResults *results;
 
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+    results = g_task_propagate_pointer (G_TASK (res), error);
+    if (!results)
         return FALSE;
 
-    results = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
     *detailed_cdma1x_state = results->detailed_cdma1x_state;
-    *detailed_evdo_state = results->detailed_evdo_state;
+    *detailed_evdo_state   = results->detailed_evdo_state;
+    g_free (results);
     return TRUE;
 }
 
 static void
-sysinfo_ready (MMIfaceModemCdma *self,
+sysinfo_ready (MMBaseModem  *self,
                GAsyncResult *res,
-               DetailedRegistrationStateContext *ctx)
-{
-    GError *error = NULL;
-    const gchar *response;
-    GRegex *r;
-    GMatchInfo *match_info;
+               GTask        *task)
 
-    response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, &error);
+{
+    DetailedRegistrationStateResults *ctx;
+    DetailedRegistrationStateResults *results;
+    const gchar                      *response;
+    GRegex                           *r;
+    GMatchInfo                       *match_info;
+    MMModemCdmaRegistrationState      reg_state;
+    guint                             val = 0;
+
+    ctx = g_task_get_task_data (task);
+
+    /* Set input detailed states as fallback */
+    results = g_memdup (ctx, sizeof (*ctx));
 
     /* If error, leave superclass' reg state alone if AT^SYSINFO isn't supported. */
-    if (error) {
-        g_error_free (error);
-
-        /* NOTE: always complete NOT in idle here */
-        g_simple_async_result_set_op_res_gpointer (ctx->result, &ctx->state, NULL);
-        detailed_registration_state_context_complete_and_free (ctx);
-        return;
-    }
+    response = mm_base_modem_at_command_finish (self, res, NULL);
+    if (!response)
+        goto out;
 
     response = mm_strip_tag (response, "^SYSINFO:");
 
@@ -215,82 +196,77 @@ sysinfo_ready (MMIfaceModemCdma *self,
     g_assert (r != NULL);
 
     /* Try to parse the results */
-
     g_regex_match (r, response, 0, &match_info);
-
     if (g_match_info_get_match_count (match_info) < 6) {
         mm_warn ("Via: failed to parse ^SYSINFO response: '%s'", response);
+        goto out;
+    }
+
+    /* At this point the generic code already knows we've been registered */
+    reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED;
+
+    if (mm_get_uint_from_match_info (match_info, 1, &val)) {
+        if (val == 2) {
+            /* Service available, check roaming state */
+            val = 0;
+            if (mm_get_uint_from_match_info (match_info, 3, &val)) {
+                if (val == 0)
+                    reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_HOME;
+                else if (val == 1)
+                    reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING;
+            }
+        }
+    }
+
+    /* Check service type */
+    val = 0;
+    if (mm_get_uint_from_match_info (match_info, 4, &val)) {
+        if (val == 2) /* CDMA */
+            results->detailed_cdma1x_state = reg_state;
+        else if (val == 4) /* HDR */
+            results->detailed_evdo_state = reg_state;
+        else if (val == 8) { /* Hybrid */
+            results->detailed_cdma1x_state = reg_state;
+            results->detailed_evdo_state = reg_state;
+        }
     } else {
-        MMModemCdmaRegistrationState reg_state;
-        guint val = 0;
-
-        /* At this point the generic code already knows we've been registered */
-        reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED;
-
-        if (mm_get_uint_from_match_info (match_info, 1, &val)) {
-            if (val == 2) {
-                /* Service available, check roaming state */
-                val = 0;
-                if (mm_get_uint_from_match_info (match_info, 3, &val)) {
-                    if (val == 0)
-                        reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_HOME;
-                    else if (val == 1)
-                        reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING;
-                }
-            }
-        }
-
-        /* Check service type */
-        val = 0;
-        if (mm_get_uint_from_match_info (match_info, 4, &val)) {
-            if (val == 2) /* CDMA */
-                ctx->state.detailed_cdma1x_state = reg_state;
-            else if (val == 4) /* HDR */
-                ctx->state.detailed_evdo_state = reg_state;
-            else if (val == 8) { /* Hybrid */
-                ctx->state.detailed_cdma1x_state = reg_state;
-                ctx->state.detailed_evdo_state = reg_state;
-            }
-        } else {
-            /* Say we're registered to something even though sysmode parsing failed */
-            mm_dbg ("SYSMODE parsing failed: assuming registered at least in CDMA1x");
-            ctx->state.detailed_cdma1x_state = reg_state;
-        }
+        /* Say we're registered to something even though sysmode parsing failed */
+        mm_dbg ("SYSMODE parsing failed: assuming registered at least in CDMA1x");
+        results->detailed_cdma1x_state = reg_state;
     }
 
     g_match_info_free (match_info);
     g_regex_unref (r);
 
-    /* NOTE: always complete NOT in idle here */
-    g_simple_async_result_set_op_res_gpointer (ctx->result, &ctx->state, NULL);
-    detailed_registration_state_context_complete_and_free (ctx);
+out:
+    g_task_return_pointer (task, results, NULL);
+    g_object_unref (task);
 }
 
 static void
-get_detailed_registration_state (MMIfaceModemCdma *self,
-                                 MMModemCdmaRegistrationState cdma1x_state,
-                                 MMModemCdmaRegistrationState evdo_state,
-                                 GAsyncReadyCallback callback,
-                                 gpointer user_data)
+get_detailed_registration_state (MMIfaceModemCdma             *self,
+                                 MMModemCdmaRegistrationState  cdma1x_state,
+                                 MMModemCdmaRegistrationState  evdo_state,
+                                 GAsyncReadyCallback           callback,
+                                 gpointer                      user_data)
 {
-    DetailedRegistrationStateContext *ctx;
+    GTask                            *task;
+    DetailedRegistrationStateResults *ctx;
 
     /* Setup context */
-    ctx = g_new0 (DetailedRegistrationStateContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             get_detailed_registration_state);
-    ctx->state.detailed_cdma1x_state = cdma1x_state;
-    ctx->state.detailed_evdo_state = evdo_state;
+    ctx = g_new0 (DetailedRegistrationStateResults, 1);
+    ctx->detailed_cdma1x_state = cdma1x_state;
+    ctx->detailed_evdo_state   = evdo_state;
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, g_free);
 
     mm_base_modem_at_command (MM_BASE_MODEM (self),
                               "^SYSINFO",
                               3,
                               FALSE,
                               (GAsyncReadyCallback)sysinfo_ready,
-                              ctx);
+                              task);
 }
 
 /*****************************************************************************/
@@ -321,7 +297,7 @@ set_unsolicited_events_handlers (MMBroadbandModemVia *self,
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
     /* Enable unsolicited events in given port */
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
@@ -340,26 +316,24 @@ modem_cdma_setup_cleanup_unsolicited_events_finish (MMIfaceModemCdma *self,
                                                     GAsyncResult *res,
                                                     GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 parent_cdma_setup_unsolicited_events_ready (MMIfaceModemCdma *self,
                                             GAsyncResult *res,
-                                            GSimpleAsyncResult *simple)
+                                            GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_cdma_parent->setup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else {
         /* Our own setup now */
         set_unsolicited_events_handlers (MM_BROADBAND_MODEM_VIA (self), TRUE);
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
+        g_task_return_boolean (task, TRUE);
     }
-
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+    g_object_unref (task);
 }
 
 static void
@@ -367,33 +341,26 @@ modem_cdma_setup_unsolicited_events (MMIfaceModemCdma *self,
                                      GAsyncReadyCallback callback,
                                      gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_cdma_setup_unsolicited_events);
-
     /* Chain up parent's setup */
     iface_modem_cdma_parent->setup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_cdma_setup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 static void
 parent_cdma_cleanup_unsolicited_events_ready (MMIfaceModemCdma *self,
                                               GAsyncResult *res,
-                                              GSimpleAsyncResult *simple)
+                                              GTask *task)
 {
     GError *error = NULL;
 
     if (!iface_modem_cdma_parent->cleanup_unsolicited_events_finish (self, res, &error))
-        g_simple_async_result_take_error (simple, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res), TRUE);
-    g_simple_async_result_complete (simple);
-    g_object_unref (simple);
+        g_task_return_boolean (task, TRUE);
+
+    g_object_unref (task);
 }
 
 static void
@@ -401,13 +368,6 @@ modem_cdma_cleanup_unsolicited_events (MMIfaceModemCdma *self,
                                        GAsyncReadyCallback callback,
                                        gpointer user_data)
 {
-    GSimpleAsyncResult *result;
-
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        modem_cdma_cleanup_unsolicited_events);
-
     /* Our own cleanup first */
     set_unsolicited_events_handlers (MM_BROADBAND_MODEM_VIA (self), FALSE);
 
@@ -415,7 +375,7 @@ modem_cdma_cleanup_unsolicited_events (MMIfaceModemCdma *self,
     iface_modem_cdma_parent->cleanup_unsolicited_events (
         self,
         (GAsyncReadyCallback)parent_cdma_cleanup_unsolicited_events_ready,
-        result);
+        g_task_new (self, NULL, callback, user_data));
 }
 
 /*****************************************************************************/
@@ -431,7 +391,7 @@ set_ignored_unsolicited_events_handlers (MMBroadbandModemVia *self)
     ports[1] = mm_base_modem_peek_port_secondary (MM_BASE_MODEM (self));
 
     /* Enable unsolicited events in given port */
-    for (i = 0; i < 2; i++) {
+    for (i = 0; i < G_N_ELEMENTS (ports); i++) {
         if (!ports[i])
             continue;
 
