@@ -23,6 +23,8 @@
 #include <string.h>
 
 #include <ModemManager.h>
+#include <ModemManager-tags.h>
+
 #include <mm-errors-types.h>
 #include <mm-gdbus-modem.h>
 
@@ -36,6 +38,10 @@
 
 G_DEFINE_ABSTRACT_TYPE (MMBaseModem, mm_base_modem, MM_GDBUS_TYPE_OBJECT_SKELETON);
 
+/* If we get 10 consecutive timeouts in a serial port, we consider the modem
+ * invalid and we request re-probing. */
+#define DEFAULT_MAX_TIMEOUTS 10
+
 enum {
     PROP_0,
     PROP_VALID,
@@ -195,6 +201,8 @@ mm_base_modem_grab_port (MMBaseModem         *self,
     }
     /* Serial ports... */
     else if (g_str_equal (subsys, "tty")) {
+        const gchar *flow_control_tag;
+
         if (ptype == MM_PORT_TYPE_QCDM)
             /* QCDM port */
             port = MM_PORT (mm_port_serial_qcdm_new (name));
@@ -207,7 +215,19 @@ mm_base_modem_grab_port (MMBaseModem         *self,
                                                    mm_serial_parser_v1_parse,
                                                    mm_serial_parser_v1_new (),
                                                    mm_serial_parser_v1_destroy);
-            /* Store flags already */
+            /* Prefer plugin-provided flags to the generic ones */
+            if (at_pflags == MM_PORT_SERIAL_AT_FLAG_NONE) {
+                if (mm_kernel_device_get_property_as_boolean (kernel_device, ID_MM_PORT_TYPE_AT_PRIMARY)) {
+                    mm_dbg ("AT port '%s/%s' flagged as primary", subsys, name);
+                    at_pflags = MM_PORT_SERIAL_AT_FLAG_PRIMARY;
+                } else if (mm_kernel_device_get_property_as_boolean (kernel_device, ID_MM_PORT_TYPE_AT_SECONDARY)) {
+                    mm_dbg ("AT port '%s/%s' flagged as secondary", subsys, name);
+                    at_pflags = MM_PORT_SERIAL_AT_FLAG_SECONDARY;
+                } else if (mm_kernel_device_get_property_as_boolean (kernel_device, ID_MM_PORT_TYPE_AT_PPP)) {
+                    mm_dbg ("AT port '%s/%s' flagged as PPP", subsys, name);
+                    at_pflags = MM_PORT_SERIAL_AT_FLAG_PPP;
+                }
+            }
             mm_port_serial_at_set_flags (MM_PORT_SERIAL_AT (port), at_pflags);
         } else if (ptype == MM_PORT_TYPE_GPS) {
             /* Raw GPS port */
@@ -230,11 +250,28 @@ mm_base_modem_grab_port (MMBaseModem         *self,
                               G_CALLBACK (serial_port_timed_out_cb),
                               self);
 
-        /* For serial ports, optionally use a specific baudrate */
-        if (mm_kernel_device_has_property (kernel_device, "ID_MM_TTY_BAUDRATE"))
+        /* For serial ports, optionally use a specific baudrate and flow control */
+        if (mm_kernel_device_has_property (kernel_device, ID_MM_TTY_BAUDRATE))
             g_object_set (port,
-                          MM_PORT_SERIAL_BAUD, mm_kernel_device_get_property_as_int (kernel_device, "ID_MM_TTY_BAUDRATE"),
+                          MM_PORT_SERIAL_BAUD, mm_kernel_device_get_property_as_int (kernel_device, ID_MM_TTY_BAUDRATE),
                           NULL);
+
+        flow_control_tag = mm_kernel_device_get_property (kernel_device, ID_MM_TTY_FLOW_CONTROL);
+        if (flow_control_tag) {
+            MMFlowControl flow_control;
+            GError *inner_error = NULL;
+
+            flow_control = mm_flow_control_from_string (flow_control_tag, &inner_error);
+            if (flow_control == MM_FLOW_CONTROL_UNKNOWN) {
+                mm_warn ("(%s/%s) unsupported flow control settings in port: %s",
+                         subsys, name, inner_error->message);
+                g_error_free (inner_error);
+            } else {
+                g_object_set (port,
+                              MM_PORT_SERIAL_FLOW_CONTROL, flow_control,
+                              NULL);
+            }
+        }
     }
     /* Net ports... */
     else if (g_str_equal (subsys, "net")) {
@@ -342,7 +379,7 @@ disable_ready (MMBaseModem  *self,
     }
     g_clear_error (&error);
 
-    g_list_free_full (disable_tasks, (GDestroyNotify)g_object_unref);
+    g_list_free_full (disable_tasks, g_object_unref);
 }
 
 void
@@ -402,7 +439,7 @@ enable_ready (MMBaseModem  *self,
     }
     g_clear_error (&error);
 
-    g_list_free_full (enable_tasks, (GDestroyNotify)g_object_unref);
+    g_list_free_full (enable_tasks, g_object_unref);
 }
 
 void
@@ -922,6 +959,9 @@ mm_base_modem_find_ports (MMBaseModem *self,
     gpointer value;
     gpointer key;
 
+    if (!self->priv->ports)
+        return NULL;
+
     /* We'll iterate the ht of ports, looking for any port which is matches
      * the compare function */
     g_hash_table_iter_init (&iter, self->priv->ports);
@@ -1396,6 +1436,8 @@ mm_base_modem_init (MMBaseModem *self)
                                                g_str_equal,
                                                g_free,
                                                g_object_unref);
+
+    self->priv->max_timeouts = DEFAULT_MAX_TIMEOUTS;
 }
 
 static void
@@ -1578,7 +1620,7 @@ mm_base_modem_class_init (MMBaseModemClass *klass)
                            "Max timeouts",
                            "Maximum number of consecutive timed out commands sent to "
                            "the modem before disabling it. If 0, this feature is disabled.",
-                           0, G_MAXUINT, 0,
+                           0, G_MAXUINT, DEFAULT_MAX_TIMEOUTS,
                            G_PARAM_READWRITE);
     g_object_class_install_property (object_class, PROP_MAX_TIMEOUTS, properties[PROP_MAX_TIMEOUTS]);
 
