@@ -25,6 +25,7 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
+#include "mm-modem-helpers.h"
 #include "mm-base-bearer.h"
 #include "mm-broadband-modem.h"
 
@@ -39,6 +40,8 @@ typedef struct _MMBroadbandBearer MMBroadbandBearer;
 typedef struct _MMBroadbandBearerClass MMBroadbandBearerClass;
 typedef struct _MMBroadbandBearerPrivate MMBroadbandBearerPrivate;
 
+#define MM_BROADBAND_BEARER_FLOW_CONTROL "broadband-bearer-flow-control"
+
 struct _MMBroadbandBearer {
     MMBaseBearer parent;
     MMBroadbandBearerPrivate *priv;
@@ -47,7 +50,7 @@ struct _MMBroadbandBearer {
 struct _MMBroadbandBearerClass {
     MMBaseBearerClass parent;
 
-    /* Full 3GPP connection sequence */
+    /* Full 3GPP connection sequence (cid selection, dial, get ip config) */
     void     (* connect_3gpp)        (MMBroadbandBearer *self,
                                       MMBroadbandModem *modem,
                                       MMPortSerialAt *primary,
@@ -59,6 +62,17 @@ struct _MMBroadbandBearerClass {
                                                      GAsyncResult *res,
                                                      GError **error);
 
+    /* CID selection sub-part of 3GPP connection */
+    void  (* cid_selection_3gpp)        (MMBroadbandBearer *self,
+                                         MMBaseModem *modem,
+                                         MMPortSerialAt *primary,
+                                         GCancellable *cancellable,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data);
+    guint (* cid_selection_3gpp_finish) (MMBroadbandBearer *self,
+                                         GAsyncResult *res,
+                                         GError **error);
+
     /* Dialing sub-part of 3GPP connection */
     void     (* dial_3gpp)        (MMBroadbandBearer *self,
                                    MMBaseModem *modem,
