@@ -24,6 +24,7 @@
 
 #include <ModemManager.h>
 
+#include "mm-modem-helpers.h"
 #include "mm-charsets.h"
 #include "mm-base-modem.h"
 
@@ -38,6 +39,8 @@ typedef struct _MMBroadbandModem MMBroadbandModem;
 typedef struct _MMBroadbandModemClass MMBroadbandModemClass;
 typedef struct _MMBroadbandModemPrivate MMBroadbandModemPrivate;
 
+#define MM_BROADBAND_MODEM_FLOW_CONTROL "broadband-modem-flow-control"
+
 struct _MMBroadbandModem {
     MMBaseModem parent;
     MMBroadbandModemPrivate *priv;
@@ -127,5 +130,7 @@ gboolean mm_broadband_modem_lock_sms_storages_finish (MMBroadbandModem *self,
 void     mm_broadband_modem_unlock_sms_storages      (MMBroadbandModem *self,
                                                       gboolean mem1,
                                                       gboolean mem2);
+/* Helper to update SIM hot swap */
+void mm_broadband_modem_update_sim_hot_swap_detected (MMBroadbandModem *self);
 
 #endif /* MM_BROADBAND_MODEM_H */
