@@ -194,15 +194,17 @@ handle_respond_ready (MMIfaceModem3gppUssd *self,
                       HandleRespondContext *ctx)
 {
     GError *error = NULL;
-    const gchar *reply;
+    gchar *reply;
 
-    reply = MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->send_finish (self, res,&error);
+    reply = MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->send_finish (self, res, &error);
     if (!reply)
         g_dbus_method_invocation_take_error (ctx->invocation, error);
-    else
+    else {
         mm_gdbus_modem3gpp_ussd_complete_respond (ctx->skeleton,
                                                   ctx->invocation,
                                                   reply);
+        g_free (reply);
+    }
     handle_respond_context_free (ctx);
 }
 
@@ -298,15 +300,17 @@ handle_initiate_ready (MMIfaceModem3gppUssd *self,
                        HandleInitiateContext *ctx)
 {
     GError *error = NULL;
-    const gchar *reply;
+    gchar *reply;
 
     reply = MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->send_finish (self, res, &error);
     if (!reply)
         g_dbus_method_invocation_take_error (ctx->invocation, error);
-    else
+    else {
         mm_gdbus_modem3gpp_ussd_complete_initiate (ctx->skeleton,
                                                    ctx->invocation,
                                                    reply);
+        g_free (reply);
+    }
     handle_initiate_context_free (ctx);
 }
 
@@ -398,6 +402,24 @@ mm_iface_modem_3gpp_ussd_decode (MMIfaceModem3gppUssd *self,
 
 /*****************************************************************************/
 
+MMModem3gppUssdSessionState
+mm_iface_modem_3gpp_ussd_get_state (MMIfaceModem3gppUssd *self)
+{
+    MmGdbusModem3gppUssd        *skeleton = NULL;
+    MMModem3gppUssdSessionState  state;
+
+    g_object_get (self,
+                  MM_IFACE_MODEM_3GPP_USSD_DBUS_SKELETON, &skeleton,
+                  NULL);
+
+    if (!skeleton)
+        return MM_MODEM_3GPP_USSD_SESSION_STATE_UNKNOWN;
+
+    state = (MMModem3gppUssdSessionState) mm_gdbus_modem3gpp_ussd_get_state (skeleton);
+    g_object_unref (skeleton);
+    return state;
+}
+
 void
 mm_iface_modem_3gpp_ussd_update_state (MMIfaceModem3gppUssd *self,
                                        MMModem3gppUssdSessionState new_state)
@@ -459,28 +481,23 @@ mm_iface_modem_3gpp_ussd_update_network_request (MMIfaceModem3gppUssd *self,
 /*****************************************************************************/
 
 typedef struct _DisablingContext DisablingContext;
-static void interface_disabling_step (DisablingContext *ctx);
+static void interface_disabling_step (GTask *task);
 
 typedef enum {
     DISABLING_STEP_FIRST,
-    DISABLING_STEP_DISABLE_UNSOLICITED_RESULT_CODES,
-    DISABLING_STEP_CLEANUP_UNSOLICITED_RESULT_CODES,
+    DISABLING_STEP_DISABLE_UNSOLICITED_EVENTS,
+    DISABLING_STEP_CLEANUP_UNSOLICITED_EVENTS,
     DISABLING_STEP_LAST
 } DisablingStep;
 
 struct _DisablingContext {
-    MMIfaceModem3gppUssd *self;
     DisablingStep step;
-    GSimpleAsyncResult *result;
     MmGdbusModem3gppUssd *skeleton;
 };
 
 static void
-disabling_context_complete_and_free (DisablingContext *ctx)
+disabling_context_free (DisablingContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
     if (ctx->skeleton)
         g_object_unref (ctx->skeleton);
     g_free (ctx);
@@ -491,79 +508,85 @@ mm_iface_modem_3gpp_ussd_disable_finish (MMIfaceModem3gppUssd *self,
                                          GAsyncResult *res,
                                          GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
-disable_unsolicited_result_codes_ready (MMIfaceModem3gppUssd *self,
-                                        GAsyncResult *res,
-                                        DisablingContext *ctx)
+disable_unsolicited_events_ready (MMIfaceModem3gppUssd *self,
+                                  GAsyncResult *res,
+                                  GTask *task)
 {
+    DisablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->disable_unsolicited_result_codes_finish (self,
-                                                                                            res,
-                                                                                            &error);
+    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->disable_unsolicited_events_finish (self, res, &error);
     if (error) {
         /* This error shouldn't be treated as critical */
-        mm_dbg ("Couldn't disable unsolicited result codes: '%s'", error->message);
+        mm_dbg ("Couldn't disable unsolicited USSD events: '%s'", error->message);
         g_error_free (error);
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 static void
-cleanup_unsolicited_result_codes_ready (MMIfaceModem3gppUssd *self,
-                                        GAsyncResult *res,
-                                        DisablingContext *ctx)
+cleanup_unsolicited_events_ready (MMIfaceModem3gppUssd *self,
+                                  GAsyncResult *res,
+                                  GTask *task)
 {
+    DisablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->cleanup_unsolicited_result_codes_finish (self,
-                                                                                            res,
-                                                                                            &error);
+    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->cleanup_unsolicited_events_finish (self, res, &error);
     if (error) {
         /* This error shouldn't be treated as critical */
-        mm_dbg ("Couldn't cleanup unsolicited result codes: '%s'", error->message);
+        mm_dbg ("Couldn't cleanup unsolicited USSD events: '%s'", error->message);
         g_error_free (error);
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 static void
-interface_disabling_step (DisablingContext *ctx)
+interface_disabling_step (GTask *task)
 {
+    MMIfaceModem3gppUssd *self;
+    DisablingContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     switch (ctx->step) {
     case DISABLING_STEP_FIRST:
         /* Fall down to next step */
         ctx->step++;
 
-    case DISABLING_STEP_DISABLE_UNSOLICITED_RESULT_CODES:
-        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->disable_unsolicited_result_codes (
-            ctx->self,
-            (GAsyncReadyCallback)disable_unsolicited_result_codes_ready,
-            ctx);
+    case DISABLING_STEP_DISABLE_UNSOLICITED_EVENTS:
+        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->disable_unsolicited_events (
+            self,
+            (GAsyncReadyCallback)disable_unsolicited_events_ready,
+            task);
         return;
 
-    case DISABLING_STEP_CLEANUP_UNSOLICITED_RESULT_CODES:
-        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->cleanup_unsolicited_result_codes (
-            ctx->self,
-            (GAsyncReadyCallback)cleanup_unsolicited_result_codes_ready,
-            ctx);
+    case DISABLING_STEP_CLEANUP_UNSOLICITED_EVENTS:
+        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->cleanup_unsolicited_events (
+            self,
+            (GAsyncReadyCallback)cleanup_unsolicited_events_ready,
+            task);
         return;
 
     case DISABLING_STEP_LAST:
         /* We are done without errors! */
-        mm_iface_modem_3gpp_ussd_update_state (MM_IFACE_MODEM_3GPP_USSD (ctx->self),
+        mm_iface_modem_3gpp_ussd_update_state (MM_IFACE_MODEM_3GPP_USSD (self),
                                                MM_MODEM_3GPP_USSD_SESSION_STATE_UNKNOWN);
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        disabling_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -576,54 +599,49 @@ mm_iface_modem_3gpp_ussd_disable (MMIfaceModem3gppUssd *self,
                                   gpointer user_data)
 {
     DisablingContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (DisablingContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_3gpp_ussd_disable);
     ctx->step = DISABLING_STEP_FIRST;
-    g_object_get (ctx->self,
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)disabling_context_free);
+
+    g_object_get (self,
                   MM_IFACE_MODEM_3GPP_USSD_DBUS_SKELETON, &ctx->skeleton,
                   NULL);
     if (!ctx->skeleton) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Couldn't get interface skeleton");
-        disabling_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Couldn't get interface skeleton");
+        g_object_unref (task);
         return;
     }
 
-    interface_disabling_step (ctx);
+    interface_disabling_step (task);
 }
 
 /*****************************************************************************/
 
 typedef struct _EnablingContext EnablingContext;
-static void interface_enabling_step (EnablingContext *ctx);
+static void interface_enabling_step (GTask *task);
 
 typedef enum {
     ENABLING_STEP_FIRST,
-    ENABLING_STEP_SETUP_UNSOLICITED_RESULT_CODES,
-    ENABLING_STEP_ENABLE_UNSOLICITED_RESULT_CODES,
+    ENABLING_STEP_SETUP_UNSOLICITED_EVENTS,
+    ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS,
     ENABLING_STEP_LAST
 } EnablingStep;
 
 struct _EnablingContext {
-    MMIfaceModem3gppUssd *self;
     EnablingStep step;
-    GSimpleAsyncResult *result;
     MmGdbusModem3gppUssd *skeleton;
 };
 
 static void
-enabling_context_complete_and_free (EnablingContext *ctx)
+enabling_context_free (EnablingContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
     if (ctx->skeleton)
         g_object_unref (ctx->skeleton);
     g_free (ctx);
@@ -634,79 +652,85 @@ mm_iface_modem_3gpp_ussd_enable_finish (MMIfaceModem3gppUssd *self,
                                         GAsyncResult *res,
                                         GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
-setup_unsolicited_result_codes_ready (MMIfaceModem3gppUssd *self,
-                                      GAsyncResult *res,
-                                      EnablingContext *ctx)
+setup_unsolicited_events_ready (MMIfaceModem3gppUssd *self,
+                                GAsyncResult *res,
+                                GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->setup_unsolicited_result_codes_finish (self,
-                                                                                          res,
-                                                                                          &error);
+    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->setup_unsolicited_events_finish (self, res, &error);
     if (error) {
         /* This error shouldn't be treated as critical */
-        mm_dbg ("Couldn't setup unsolicited result codes: '%s'", error->message);
+        mm_dbg ("Couldn't setup unsolicited USSD events: '%s'", error->message);
         g_error_free (error);
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 static void
-enable_unsolicited_result_codes_ready (MMIfaceModem3gppUssd *self,
-                                       GAsyncResult *res,
-                                       EnablingContext *ctx)
+enable_unsolicited_events_ready (MMIfaceModem3gppUssd *self,
+                                 GAsyncResult *res,
+                                 GTask *task)
 {
+    EnablingContext *ctx;
     GError *error = NULL;
 
-    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->enable_unsolicited_result_codes_finish (self,
-                                                                                           res,
-                                                                                           &error);
+    MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->enable_unsolicited_events_finish (self, res, &error);
     if (error) {
         /* This error shouldn't be treated as critical */
-        mm_dbg ("Couldn't enable unsolicited result codes: '%s'", error->message);
+        mm_dbg ("Couldn't enable unsolicited USSD events: '%s'", error->message);
         g_error_free (error);
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 static void
-interface_enabling_step (EnablingContext *ctx)
+interface_enabling_step (GTask *task)
 {
+    MMIfaceModem3gppUssd *self;
+    EnablingContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     switch (ctx->step) {
     case ENABLING_STEP_FIRST:
         /* Fall down to next step */
         ctx->step++;
 
-    case ENABLING_STEP_SETUP_UNSOLICITED_RESULT_CODES:
-        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->setup_unsolicited_result_codes (
-            ctx->self,
-            (GAsyncReadyCallback)setup_unsolicited_result_codes_ready,
-            ctx);
+    case ENABLING_STEP_SETUP_UNSOLICITED_EVENTS:
+        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->setup_unsolicited_events (
+            self,
+            (GAsyncReadyCallback)setup_unsolicited_events_ready,
+            task);
         return;
 
-    case ENABLING_STEP_ENABLE_UNSOLICITED_RESULT_CODES:
-        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->enable_unsolicited_result_codes (
-            ctx->self,
-            (GAsyncReadyCallback)enable_unsolicited_result_codes_ready,
-            ctx);
+    case ENABLING_STEP_ENABLE_UNSOLICITED_EVENTS:
+        MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->enable_unsolicited_events (
+            self,
+            (GAsyncReadyCallback)enable_unsolicited_events_ready,
+            task);
         return;
 
     case ENABLING_STEP_LAST:
         /* We are done without errors! */
-        mm_iface_modem_3gpp_ussd_update_state (MM_IFACE_MODEM_3GPP_USSD (ctx->self),
+        mm_iface_modem_3gpp_ussd_update_state (MM_IFACE_MODEM_3GPP_USSD (self),
                                                MM_MODEM_3GPP_USSD_SESSION_STATE_IDLE);
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        enabling_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -719,33 +743,33 @@ mm_iface_modem_3gpp_ussd_enable (MMIfaceModem3gppUssd *self,
                                  gpointer user_data)
 {
     EnablingContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (EnablingContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_3gpp_ussd_enable);
     ctx->step = ENABLING_STEP_FIRST;
-    g_object_get (ctx->self,
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)enabling_context_free);
+
+    g_object_get (self,
                   MM_IFACE_MODEM_3GPP_USSD_DBUS_SKELETON, &ctx->skeleton,
                   NULL);
     if (!ctx->skeleton) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Couldn't get interface skeleton");
-        enabling_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Couldn't get interface skeleton");
+        g_object_unref (task);
         return;
     }
 
-    interface_enabling_step (ctx);
+    interface_enabling_step (task);
 }
 
 /*****************************************************************************/
 
 typedef struct _InitializationContext InitializationContext;
-static void interface_initialization_step (InitializationContext *ctx);
+static void interface_initialization_step (GTask *task);
 
 typedef enum {
     INITIALIZATION_STEP_FIRST,
@@ -755,18 +779,13 @@ typedef enum {
 } InitializationStep;
 
 struct _InitializationContext {
-    MMIfaceModem3gppUssd *self;
     MmGdbusModem3gppUssd *skeleton;
-    GSimpleAsyncResult *result;
     InitializationStep step;
 };
 
 static void
-initialization_context_complete_and_free (InitializationContext *ctx)
+initialization_context_free (InitializationContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
     g_object_unref (ctx->skeleton);
     g_free (ctx);
 }
@@ -774,8 +793,9 @@ initialization_context_complete_and_free (InitializationContext *ctx)
 static void
 check_support_ready (MMIfaceModem3gppUssd *self,
                      GAsyncResult *res,
-                     InitializationContext *ctx)
+                     GTask *task)
 {
+    InitializationContext *ctx;
     GError *error = NULL;
 
     if (!MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->check_support_finish (self,
@@ -794,13 +814,20 @@ check_support_ready (MMIfaceModem3gppUssd *self,
     }
 
     /* Go on to next step */
+    ctx = g_task_get_task_data (task);
     ctx->step++;
-    interface_initialization_step (ctx);
+    interface_initialization_step (task);
 }
 
 static void
-interface_initialization_step (InitializationContext *ctx)
+interface_initialization_step (GTask *task)
 {
+    MMIfaceModem3gppUssd *self;
+    InitializationContext *ctx;
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
     switch (ctx->step) {
     case INITIALIZATION_STEP_FIRST:
         /* Setup quarks if we didn't do it before */
@@ -815,23 +842,23 @@ interface_initialization_step (InitializationContext *ctx)
         ctx->step++;
 
     case INITIALIZATION_STEP_CHECK_SUPPORT:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
+        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
                                                    support_checked_quark))) {
             /* Set the checked flag so that we don't run it again */
-            g_object_set_qdata (G_OBJECT (ctx->self),
+            g_object_set_qdata (G_OBJECT (self),
                                 support_checked_quark,
                                 GUINT_TO_POINTER (TRUE));
             /* Initially, assume we don't support it */
-            g_object_set_qdata (G_OBJECT (ctx->self),
+            g_object_set_qdata (G_OBJECT (self),
                                 supported_quark,
                                 GUINT_TO_POINTER (FALSE));
 
-            if (MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->check_support &&
-                MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->check_support_finish) {
-                MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (ctx->self)->check_support (
-                    ctx->self,
+            if (MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->check_support &&
+                MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->check_support_finish) {
+                MM_IFACE_MODEM_3GPP_USSD_GET_INTERFACE (self)->check_support (
+                    self,
                     (GAsyncReadyCallback)check_support_ready,
-                    ctx);
+                    task);
                 return;
             }
 
@@ -843,13 +870,13 @@ interface_initialization_step (InitializationContext *ctx)
         ctx->step++;
 
     case INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
+        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (self),
                                                    supported_quark))) {
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_UNSUPPORTED,
-                                             "USSD not supported");
-            initialization_context_complete_and_free (ctx);
+            g_task_return_new_error (task,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_UNSUPPORTED,
+                                     "USSD not supported");
+            g_object_unref (task);
             return;
         }
 
@@ -863,22 +890,22 @@ interface_initialization_step (InitializationContext *ctx)
         g_signal_connect (ctx->skeleton,
                           "handle-initiate",
                           G_CALLBACK (handle_initiate),
-                          ctx->self);
+                          self);
         g_signal_connect (ctx->skeleton,
                           "handle-respond",
                           G_CALLBACK (handle_respond),
-                          ctx->self);
+                          self);
         g_signal_connect (ctx->skeleton,
                           "handle-cancel",
                           G_CALLBACK (handle_cancel),
-                          ctx->self);
+                          self);
 
         /* Finally, export the new interface */
-        mm_gdbus_object_skeleton_set_modem3gpp_ussd (MM_GDBUS_OBJECT_SKELETON (ctx->self),
+        mm_gdbus_object_skeleton_set_modem3gpp_ussd (MM_GDBUS_OBJECT_SKELETON (self),
                                                      MM_GDBUS_MODEM3GPP_USSD (ctx->skeleton));
 
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        initialization_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
@@ -890,7 +917,7 @@ mm_iface_modem_3gpp_ussd_initialize_finish (MMIfaceModem3gppUssd *self,
                                             GAsyncResult *res,
                                             GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 void
@@ -900,6 +927,7 @@ mm_iface_modem_3gpp_ussd_initialize (MMIfaceModem3gppUssd *self,
 {
     InitializationContext *ctx;
     MmGdbusModem3gppUssd *skeleton = NULL;
+    GTask *task;
 
     /* Did we already create it? */
     g_object_get (self,
@@ -921,15 +949,13 @@ mm_iface_modem_3gpp_ussd_initialize (MMIfaceModem3gppUssd *self,
     /* Perform async initialization here */
 
     ctx = g_new0 (InitializationContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_3gpp_ussd_initialize);
     ctx->step = INITIALIZATION_STEP_FIRST;
     ctx->skeleton = skeleton;
 
-    interface_initialization_step (ctx);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)initialization_context_free);
+
+    interface_initialization_step (task);
 }
 
 void
