@@ -21,12 +21,6 @@
 #include "mm-iface-modem-firmware.h"
 #include "mm-log.h"
 
-#define SUPPORT_CHECKED_TAG "firmware-support-checked-tag"
-#define SUPPORTED_TAG       "firmware-supported-tag"
-
-static GQuark support_checked_quark;
-static GQuark supported_quark;
-
 /*****************************************************************************/
 
 void
@@ -50,7 +44,7 @@ static void
 handle_list_context_free (HandleListContext *ctx)
 {
     if (ctx->list)
-        g_list_free_full (ctx->list, (GDestroyNotify)g_object_unref);
+        g_list_free_full (ctx->list, g_object_unref);
     if (ctx->current)
         g_object_unref (ctx->current);
     g_object_unref (ctx->skeleton);
@@ -68,16 +62,20 @@ load_current_ready (MMIfaceModemFirmware *self,
     GList *l;
     GError *error = NULL;
 
-    /* reported current may be NULL and we don't treat it as error */
     ctx->current = MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current_finish (self, res, &error);
-    if (error) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
-        handle_list_context_free (ctx);
-        return;
+    if (!ctx->current) {
+        /* Not found isn't fatal */
+        if (!g_error_matches (error, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND)) {
+            g_dbus_method_invocation_take_error (ctx->invocation, error);
+            handle_list_context_free (ctx);
+            return;
+        }
+        mm_dbg ("Couldn't load current firmware image: %s", error->message);
+        g_clear_error (&error);
     }
 
     /* Build array of dicts */
-    g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
+    g_variant_builder_init (&builder, G_VARIANT_TYPE ("aa{sv}"));
     for (l = ctx->list; l; l = g_list_next (l))
         g_variant_builder_add_value (
             &builder,
@@ -99,10 +97,15 @@ load_list_ready (MMIfaceModemFirmware *self,
     GError *error = NULL;
 
     ctx->list = MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list_finish (self, res, &error);
-    if (error) {
-        g_dbus_method_invocation_take_error (ctx->invocation, error);
-        handle_list_context_free (ctx);
-        return;
+    if (!ctx->list) {
+        /* Not found isn't fatal */
+        if (!g_error_matches (error, MM_CORE_ERROR, MM_CORE_ERROR_NOT_FOUND)) {
+            g_dbus_method_invocation_take_error (ctx->invocation, error);
+            handle_list_context_free (ctx);
+            return;
+        }
+        mm_dbg ("Couldn't load firmware image list: %s", error->message);
+        g_clear_error (&error);
     }
 
     MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current (MM_IFACE_MODEM_FIRMWARE (self),
@@ -123,6 +126,18 @@ list_auth_ready (MMBaseModem *self,
         return;
     }
 
+    if (!MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list ||
+        !MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list_finish ||
+        !MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current ||
+        !MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current_finish) {
+        g_dbus_method_invocation_return_error (ctx->invocation,
+                                               MM_CORE_ERROR,
+                                               MM_CORE_ERROR_UNSUPPORTED,
+                                               "Cannot list firmware: operation not supported");
+        handle_list_context_free (ctx);
+        return;
+    }
+
     MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list (MM_IFACE_MODEM_FIRMWARE (self),
                                                              (GAsyncReadyCallback)load_list_ready,
                                                              ctx);
@@ -135,11 +150,6 @@ handle_list (MmGdbusModemFirmware *skeleton,
 {
     HandleListContext *ctx;
 
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list != NULL);
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_list_finish != NULL);
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current != NULL);
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_current_finish != NULL);
-
     ctx = g_slice_new (HandleListContext);
     ctx->skeleton = g_object_ref (skeleton);
     ctx->invocation = g_object_ref (invocation);
@@ -201,6 +211,17 @@ select_auth_ready (MMBaseModem *self,
         return;
     }
 
+
+    if (!MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->change_current ||
+        !MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->change_current_finish) {
+        g_dbus_method_invocation_return_error (ctx->invocation,
+                                               MM_CORE_ERROR,
+                                               MM_CORE_ERROR_UNSUPPORTED,
+                                               "Cannot select firmware: operation not supported");
+        handle_select_context_free (ctx);
+        return;
+    }
+
     MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->change_current (MM_IFACE_MODEM_FIRMWARE (self),
                                                                   ctx->name,
                                                                   (GAsyncReadyCallback)change_current_ready,
@@ -215,9 +236,6 @@ handle_select (MmGdbusModemFirmware *skeleton,
 {
     HandleSelectContext *ctx;
 
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->change_current != NULL);
-    g_assert (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->change_current_finish != NULL);
-
     ctx = g_slice_new (HandleSelectContext);
     ctx->skeleton = g_object_ref (skeleton);
     ctx->invocation = g_object_ref (invocation);
@@ -236,130 +254,175 @@ handle_select (MmGdbusModemFirmware *skeleton,
 /*****************************************************************************/
 
 typedef struct _InitializationContext InitializationContext;
-static void interface_initialization_step (InitializationContext *ctx);
+static void interface_initialization_step (GTask *task);
 
 typedef enum {
     INITIALIZATION_STEP_FIRST,
-    INITIALIZATION_STEP_CHECK_SUPPORT,
-    INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED,
+    INITIALIZATION_STEP_UPDATE_SETTINGS,
     INITIALIZATION_STEP_LAST
 } InitializationStep;
 
 struct _InitializationContext {
-    MMIfaceModemFirmware *self;
     MmGdbusModemFirmware *skeleton;
-    GCancellable *cancellable;
-    GSimpleAsyncResult *result;
-    InitializationStep step;
+    InitializationStep    step;
 };
 
 static void
-initialization_context_complete_and_free (InitializationContext *ctx)
+initialization_context_free (InitializationContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->self);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->cancellable);
     g_object_unref (ctx->skeleton);
     g_free (ctx);
 }
 
-static gboolean
-initialization_context_complete_and_free_if_cancelled (InitializationContext *ctx)
+gboolean
+mm_iface_modem_firmware_initialize_finish (MMIfaceModemFirmware  *self,
+                                           GAsyncResult          *res,
+                                           GError               **error)
 {
-    if (!g_cancellable_is_cancelled (ctx->cancellable))
-        return FALSE;
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
 
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_CANCELLED,
-                                     "Interface initialization cancelled");
-    initialization_context_complete_and_free (ctx);
+static gboolean
+add_generic_version (MMBaseModem               *self,
+                     MMFirmwareUpdateSettings  *update_settings,
+                     GError                   **error)
+{
+    const gchar *revision;
+
+    revision = mm_iface_modem_get_revision (MM_IFACE_MODEM (self));
+    if (!revision) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Unknown revision");
+        return FALSE;
+    }
+
+    mm_firmware_update_settings_set_version (update_settings, revision);
+    return TRUE;
+}
+
+static gboolean
+add_generic_device_ids (MMBaseModem               *self,
+                        MMFirmwareUpdateSettings  *update_settings,
+                        GError                   **error)
+{
+    guint16      vid;
+    guint16      pid;
+    guint16      rid;
+    GPtrArray   *ids;
+    MMPort      *primary = NULL;
+    const gchar *subsystem;
+
+    vid = mm_base_modem_get_vendor_id (self);
+    pid = mm_base_modem_get_product_id (self);
+
+#if defined WITH_QMI
+    primary = MM_PORT (mm_base_modem_peek_port_qmi (self));
+#endif
+#if defined WITH_MBIM
+    if (!primary)
+        primary = MM_PORT (mm_base_modem_peek_port_mbim (self));
+#endif
+    if (!primary)
+        primary = MM_PORT (mm_base_modem_peek_port_primary (self));
+    g_assert (primary != NULL);
+    rid = mm_kernel_device_get_physdev_revision (mm_port_peek_kernel_device (primary));
+
+    subsystem = mm_kernel_device_get_physdev_subsystem (mm_port_peek_kernel_device (primary));
+    if (g_strcmp0 (subsystem, "usb")) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Unsupported subsystem: %s", subsystem);
+        return FALSE;
+    }
+
+    ids = g_ptr_array_new_with_free_func ((GDestroyNotify)g_free);
+    g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X&PID_%04X&REV_%04X", vid, pid, rid));
+    g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X&PID_%04X", vid, pid));
+    g_ptr_array_add (ids, g_strdup_printf ("USB\\VID_%04X", vid));
+    g_ptr_array_add (ids, NULL);
+
+    mm_firmware_update_settings_set_device_ids (update_settings, (const gchar **)ids->pdata);
+    g_ptr_array_unref (ids);
     return TRUE;
 }
 
 static void
-check_support_ready (MMIfaceModemFirmware *self,
-                     GAsyncResult *res,
-                     InitializationContext *ctx)
+load_update_settings_ready (MMIfaceModemFirmware *self,
+                            GAsyncResult         *res,
+                            GTask                *task)
 {
-    GError *error = NULL;
+    InitializationContext    *ctx;
+    MMFirmwareUpdateSettings *update_settings;
+    GError                   *error = NULL;
+    GVariant                 *variant = NULL;
 
-    if (!MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->check_support_finish (self,
-                                                                             res,
-                                                                             &error)) {
-        if (error) {
-            /* This error shouldn't be treated as critical */
-            mm_dbg ("Firmware support check failed: '%s'", error->message);
-            g_error_free (error);
-        }
-    } else {
-        /* Firmware is supported! */
-        g_object_set_qdata (G_OBJECT (self),
-                            supported_quark,
-                            GUINT_TO_POINTER (TRUE));
+    ctx = g_task_get_task_data (task);
+
+    update_settings = MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_update_settings_finish (self, res, &error);
+    if (!update_settings) {
+        mm_dbg ("Couldn't load update settings: '%s'", error->message);
+        g_error_free (error);
+        goto out;
     }
 
+    /* If the plugin didn't specify custom device ids, add the default ones ourselves */
+    if (!mm_firmware_update_settings_get_device_ids (update_settings) &&
+        !add_generic_device_ids (MM_BASE_MODEM (self), update_settings, &error)) {
+        mm_warn ("Couldn't build device ids: '%s'", error->message);
+        g_error_free (error);
+        g_clear_object (&update_settings);
+        goto out;
+    }
+
+    /* If the plugin didn't specify custom version, add the default one ourselves */
+    if (!mm_firmware_update_settings_get_version (update_settings) &&
+        !add_generic_version (MM_BASE_MODEM (self), update_settings, &error)) {
+        mm_warn ("Couldn't set version: '%s'", error->message);
+        g_error_free (error);
+        g_clear_object (&update_settings);
+        goto out;
+    }
+
+out:
+    if (update_settings) {
+        variant = mm_firmware_update_settings_get_variant (update_settings);
+        g_object_unref (update_settings);
+    }
+    mm_gdbus_modem_firmware_set_update_settings (ctx->skeleton, variant);
+    if (variant)
+        g_variant_unref (variant);
+
     /* Go on to next step */
     ctx->step++;
-    interface_initialization_step (ctx);
+    interface_initialization_step (task);
 }
 
 static void
-interface_initialization_step (InitializationContext *ctx)
+interface_initialization_step (GTask *task)
 {
+    MMIfaceModemFirmware *self;
+    InitializationContext *ctx;
+
     /* Don't run new steps if we're cancelled */
-    if (initialization_context_complete_and_free_if_cancelled (ctx))
+    if (g_task_return_error_if_cancelled (task)) {
+        g_object_unref (task);
         return;
+    }
+
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
 
     switch (ctx->step) {
     case INITIALIZATION_STEP_FIRST:
-        /* Setup quarks if we didn't do it before */
-        if (G_UNLIKELY (!support_checked_quark))
-            support_checked_quark = (g_quark_from_static_string (
-                                         SUPPORT_CHECKED_TAG));
-        if (G_UNLIKELY (!supported_quark))
-            supported_quark = (g_quark_from_static_string (
-                                   SUPPORTED_TAG));
-
         /* Fall down to next step */
         ctx->step++;
 
-    case INITIALIZATION_STEP_CHECK_SUPPORT:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
-                                                   support_checked_quark))) {
-            /* Set the checked flag so that we don't run it again */
-            g_object_set_qdata (G_OBJECT (ctx->self),
-                                support_checked_quark,
-                                GUINT_TO_POINTER (TRUE));
-            /* Initially, assume we don't support it */
-            g_object_set_qdata (G_OBJECT (ctx->self),
-                                supported_quark,
-                                GUINT_TO_POINTER (FALSE));
-
-            if (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (ctx->self)->check_support &&
-                MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (ctx->self)->check_support_finish) {
-                MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (ctx->self)->check_support (
-                    ctx->self,
-                    (GAsyncReadyCallback)check_support_ready,
-                    ctx);
-                return;
-            }
-
-            /* If there is no implementation to check support, assume we DON'T
-             * support it. */
-        }
-        /* Fall down to next step */
-        ctx->step++;
-
-    case INITIALIZATION_STEP_FAIL_IF_UNSUPPORTED:
-        if (!GPOINTER_TO_UINT (g_object_get_qdata (G_OBJECT (ctx->self),
-                                                   supported_quark))) {
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_UNSUPPORTED,
-                                             "Firmware interface not available");
-            initialization_context_complete_and_free (ctx);
+    case INITIALIZATION_STEP_UPDATE_SETTINGS:
+        if (MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_update_settings &&
+            MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_update_settings_finish) {
+            MM_IFACE_MODEM_FIRMWARE_GET_INTERFACE (self)->load_update_settings (
+                self,
+                (GAsyncReadyCallback)load_update_settings_ready,
+                task);
             return;
         }
         /* Fall down to next step */
@@ -369,43 +432,32 @@ interface_initialization_step (InitializationContext *ctx)
         /* We are done without errors! */
 
         /* Handle method invocations */
-        g_signal_connect (ctx->skeleton,
-                          "handle-list",
-                          G_CALLBACK (handle_list),
-                          ctx->self);
-        g_signal_connect (ctx->skeleton,
-                          "handle-select",
-                          G_CALLBACK (handle_select),
-                          ctx->self);
+        g_object_connect (ctx->skeleton,
+                          "signal::handle-list",   G_CALLBACK (handle_list),   self,
+                          "signal::handle-select", G_CALLBACK (handle_select), self,
+                          NULL);
 
         /* Finally, export the new interface */
-        mm_gdbus_object_skeleton_set_modem_firmware (MM_GDBUS_OBJECT_SKELETON (ctx->self),
+        mm_gdbus_object_skeleton_set_modem_firmware (MM_GDBUS_OBJECT_SKELETON (self),
                                                      MM_GDBUS_MODEM_FIRMWARE (ctx->skeleton));
 
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        initialization_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
     g_assert_not_reached ();
 }
 
-gboolean
-mm_iface_modem_firmware_initialize_finish (MMIfaceModemFirmware *self,
-                                           GAsyncResult *res,
-                                           GError **error)
-{
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
-}
-
 void
 mm_iface_modem_firmware_initialize (MMIfaceModemFirmware *self,
-                                    GCancellable *cancellable,
-                                    GAsyncReadyCallback callback,
-                                    gpointer user_data)
+                                    GCancellable         *cancellable,
+                                    GAsyncReadyCallback   callback,
+                                    gpointer              user_data)
 {
     InitializationContext *ctx;
-    MmGdbusModemFirmware *skeleton = NULL;
+    MmGdbusModemFirmware  *skeleton = NULL;
+    GTask                 *task;
 
     /* Did we already create it? */
     g_object_get (self,
@@ -419,18 +471,14 @@ mm_iface_modem_firmware_initialize (MMIfaceModemFirmware *self,
     }
 
     /* Perform async initialization here */
-
     ctx = g_new0 (InitializationContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_iface_modem_firmware_initialize);
     ctx->step = INITIALIZATION_STEP_FIRST;
     ctx->skeleton = skeleton;
 
-    interface_initialization_step (ctx);
+    task = g_task_new (self, cancellable, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)initialization_context_free);
+
+    interface_initialization_step (task);
 }
 
 void
