@@ -23,13 +23,13 @@
 #include <unistd.h>
 #include <string.h>
 
-#include <gudev/gudev.h>
-
 #include <ModemManager.h>
 #include <mm-errors-types.h>
 
 #include "mm-plugin.h"
 #include "mm-device.h"
+#include "mm-kernel-device.h"
+#include "mm-kernel-device-generic.h"
 #include "mm-port-serial-at.h"
 #include "mm-port-serial-qcdm.h"
 #include "mm-serial-parsers.h"
@@ -39,7 +39,7 @@
 
 G_DEFINE_TYPE (MMPlugin, mm_plugin, G_TYPE_OBJECT)
 
-/* Virtual port corresponding to the embeded modem */
+/* Virtual port corresponding to the embedded modem */
 static gchar *virtual_port[] = {"smd0", NULL};
 
 #define HAS_POST_PROBING_FILTERS(self)          \
@@ -48,6 +48,8 @@ static gchar *virtual_port[] = {"smd0", NULL};
      self->priv->forbidden_product_strings ||   \
      self->priv->allowed_icera ||               \
      self->priv->forbidden_icera ||             \
+     self->priv->allowed_xmm ||                 \
+     self->priv->forbidden_xmm ||               \
      self->priv->custom_init)
 
 
@@ -70,6 +72,8 @@ struct _MMPluginPrivate {
     mm_str_pair *forbidden_product_strings;
     gboolean allowed_icera;
     gboolean forbidden_icera;
+    gboolean allowed_xmm;
+    gboolean forbidden_xmm;
 
     /* Probing setup */
     gboolean at;
@@ -78,6 +82,7 @@ struct _MMPluginPrivate {
     gboolean qmi;
     gboolean mbim;
     gboolean icera_probe;
+    gboolean xmm_probe;
     MMPortProbeAtCommand *custom_at_probe;
     guint64 send_delay;
     gboolean remove_echo;
@@ -110,6 +115,9 @@ enum {
     PROP_ICERA_PROBE,
     PROP_ALLOWED_ICERA,
     PROP_FORBIDDEN_ICERA,
+    PROP_XMM_PROBE,
+    PROP_ALLOWED_XMM,
+    PROP_FORBIDDEN_XMM,
     PROP_CUSTOM_AT_PROBE,
     PROP_CUSTOM_INIT,
     PROP_SEND_DELAY,
@@ -162,14 +170,14 @@ is_virtual_port (const gchar *device_name)
 
 /* Returns TRUE if the support check request was filtered out */
 static gboolean
-apply_subsystem_filter (MMPlugin *self,
-                        GUdevDevice *port)
+apply_subsystem_filter (MMPlugin       *self,
+                        MMKernelDevice *port)
 {
     if (self->priv->subsystems) {
         const gchar *subsys;
         guint i;
 
-        subsys = g_udev_device_get_subsystem (port);
+        subsys = mm_kernel_device_get_subsystem (port);
         for (i = 0; self->priv->subsystems[i]; i++) {
             if (g_str_equal (subsys, self->priv->subsystems[i]))
                 break;
@@ -189,11 +197,11 @@ apply_subsystem_filter (MMPlugin *self,
 
 /* Returns TRUE if the support check request was filtered out */
 static gboolean
-apply_pre_probing_filters (MMPlugin *self,
-                           MMDevice *device,
-                           GUdevDevice *port,
-                           gboolean *need_vendor_probing,
-                           gboolean *need_product_probing)
+apply_pre_probing_filters (MMPlugin       *self,
+                           MMDevice       *device,
+                           MMKernelDevice *port,
+                           gboolean       *need_vendor_probing,
+                           gboolean       *need_product_probing)
 {
     guint16 vendor;
     guint16 product;
@@ -209,7 +217,7 @@ apply_pre_probing_filters (MMPlugin *self,
     if (apply_subsystem_filter (self, port)) {
         mm_dbg ("(%s) [%s] filtered by subsystem",
                 self->priv->name,
-                g_udev_device_get_name (port));
+                mm_kernel_device_get_name (port));
         return TRUE;
     }
 
@@ -232,7 +240,7 @@ apply_pre_probing_filters (MMPlugin *self,
         const gchar **drivers;
 
         /* Detect any modems accessible through the list of virtual ports */
-        drivers = (is_virtual_port (g_udev_device_get_name (port)) ?
+        drivers = (is_virtual_port (mm_kernel_device_get_name (port)) ?
                    virtual_drivers :
                    mm_device_get_drivers (device));
 
@@ -240,7 +248,7 @@ apply_pre_probing_filters (MMPlugin *self,
         if (!drivers) {
             mm_dbg ("(%s) [%s] filtered as couldn't retrieve drivers",
                     self->priv->name,
-                    g_udev_device_get_name (port));
+                    mm_kernel_device_get_name (port));
             return TRUE;
         }
 
@@ -261,7 +269,7 @@ apply_pre_probing_filters (MMPlugin *self,
             if (!found) {
                 mm_dbg ("(%s) [%s] filtered by drivers",
                         self->priv->name,
-                        g_udev_device_get_name (port));
+                        mm_kernel_device_get_name (port));
                 return TRUE;
             }
         }
@@ -276,7 +284,7 @@ apply_pre_probing_filters (MMPlugin *self,
                     if (g_str_equal (drivers[j], self->priv->forbidden_drivers[i])) {
                         mm_dbg ("(%s) [%s] filtered by forbidden drivers",
                                 self->priv->name,
-                                g_udev_device_get_name (port));
+                                mm_kernel_device_get_name (port));
                         return TRUE;
                     }
                 }
@@ -292,7 +300,7 @@ apply_pre_probing_filters (MMPlugin *self,
                 if (g_str_equal (drivers[j], "qmi_wwan")) {
                     mm_dbg ("(%s) [%s] filtered by implicit QMI driver",
                             self->priv->name,
-                            g_udev_device_get_name (port));
+                            mm_kernel_device_get_name (port));
                     return TRUE;
                 }
             }
@@ -307,7 +315,7 @@ apply_pre_probing_filters (MMPlugin *self,
                 if (g_str_equal (drivers[j], "cdc_mbim")) {
                     mm_dbg ("(%s) [%s] filtered by implicit MBIM driver",
                             self->priv->name,
-                            g_udev_device_get_name (port));
+                            mm_kernel_device_get_name (port));
                     return TRUE;
                 }
             }
@@ -369,11 +377,11 @@ apply_pre_probing_filters (MMPlugin *self,
         ((!self->priv->vendor_strings &&
           !self->priv->product_strings &&
           !self->priv->forbidden_product_strings) ||
-         g_str_equal (g_udev_device_get_subsystem (port), "net") ||
-         g_str_has_prefix (g_udev_device_get_name (port), "cdc-wdm"))) {
+         g_str_equal (mm_kernel_device_get_subsystem (port), "net") ||
+         g_str_has_prefix (mm_kernel_device_get_name (port), "cdc-wdm"))) {
         mm_dbg ("(%s) [%s] filtered by vendor/product IDs",
                 self->priv->name,
-                g_udev_device_get_name (port));
+                mm_kernel_device_get_name (port));
         return TRUE;
     }
 
@@ -385,7 +393,7 @@ apply_pre_probing_filters (MMPlugin *self,
                 product == self->priv->forbidden_product_ids[i].r) {
                 mm_dbg ("(%s) [%s] filtered by forbidden vendor/product IDs",
                         self->priv->name,
-                        g_udev_device_get_name (port));
+                        mm_kernel_device_get_name (port));
                 return TRUE;
             }
         }
@@ -417,9 +425,8 @@ apply_pre_probing_filters (MMPlugin *self,
      * supported. If that is the case, filter by udev tag */
     if (self->priv->udev_tags) {
         for (i = 0; self->priv->udev_tags[i]; i++) {
-            /* Check if the port was tagged */
-            if (g_udev_device_get_property_as_boolean (port,
-                                                       self->priv->udev_tags[i]))
+            /* Check if the port or device was tagged */
+            if (mm_kernel_device_get_global_property_as_boolean (port, self->priv->udev_tags[i]))
                 break;
         }
 
@@ -427,7 +434,7 @@ apply_pre_probing_filters (MMPlugin *self,
         if (!self->priv->udev_tags[i]) {
             mm_dbg ("(%s) [%s] filtered by udev tags",
                     self->priv->name,
-                    g_udev_device_get_name (port));
+                    mm_kernel_device_get_name (port));
             return TRUE;
         }
     }
@@ -536,8 +543,8 @@ apply_post_probing_filters (MMPlugin *self,
                 gchar *casefolded_vendor;
                 gchar *casefolded_product;
 
-                casefolded_vendor = g_utf8_casefold (self->priv->product_strings[i].l, -1);
-                casefolded_product = g_utf8_casefold (self->priv->product_strings[i].r, -1);
+                casefolded_vendor = g_utf8_casefold (self->priv->forbidden_product_strings[i].l, -1);
+                casefolded_product = g_utf8_casefold (self->priv->forbidden_product_strings[i].r, -1);
                 found = (!!strstr (vendor, casefolded_vendor) &&
                          !!strstr (product, casefolded_product));
                 g_free (casefolded_vendor);
@@ -577,6 +584,28 @@ apply_post_probing_filters (MMPlugin *self,
         return TRUE;
     }
 
+    /* The plugin may specify that only Xmm-based modems are supported.
+     * If that is the case, filter by allowed Xmm support */
+    if (self->priv->allowed_xmm &&
+        !mm_port_probe_is_xmm (probe)) {
+        /* Unsupported! */
+        mm_dbg ("(%s) [%s] filtered as modem is not XMM",
+                self->priv->name,
+                mm_port_probe_get_port_name (probe));
+        return TRUE;
+    }
+
+    /* The plugin may specify that Xmm-based modems are NOT supported.
+     * If that is the case, filter by forbidden Xmm support */
+    if (self->priv->forbidden_xmm &&
+        mm_port_probe_is_xmm (probe)) {
+        /* Unsupported! */
+        mm_dbg ("(%s) [%s] filtered as modem is XMM",
+                self->priv->name,
+                mm_port_probe_get_port_name (probe));
+        return TRUE;
+    }
+
     return FALSE;
 }
 
@@ -667,16 +696,24 @@ mm_plugin_supports_port_finish (MMPlugin      *self,
                                 GAsyncResult  *result,
                                 GError       **error)
 {
+    GError *inner_error = NULL;
+    gssize value;
+
     g_return_val_if_fail (MM_IS_PLUGIN (self), MM_PLUGIN_SUPPORTS_PORT_UNKNOWN);
     g_return_val_if_fail (G_IS_TASK (result), MM_PLUGIN_SUPPORTS_PORT_UNKNOWN);
 
-    return (MMPluginSupportsResult) g_task_propagate_int (G_TASK (result), error);
+    value = g_task_propagate_int (G_TASK (result), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return MM_PLUGIN_SUPPORTS_PORT_UNKNOWN;
+    }
+    return (MMPluginSupportsResult)value;
 }
 
 void
 mm_plugin_supports_port (MMPlugin            *self,
                          MMDevice            *device,
-                         GUdevDevice         *port,
+                         MMKernelDevice      *port,
                          GCancellable        *cancellable,
                          GAsyncReadyCallback  callback,
                          gpointer             user_data)
@@ -691,7 +728,7 @@ mm_plugin_supports_port (MMPlugin            *self,
 
     g_return_if_fail (MM_IS_PLUGIN (self));
     g_return_if_fail (MM_IS_DEVICE (device));
-    g_return_if_fail (G_UDEV_IS_DEVICE (port));
+    g_return_if_fail (MM_IS_KERNEL_DEVICE (port));
 
     /* Create new cancellable task */
     task = g_task_new (self, cancellable, callback, user_data);
@@ -718,16 +755,16 @@ mm_plugin_supports_port (MMPlugin            *self,
                                  MM_CORE_ERROR_FAILED,
                                  "(%s) Missing port probe for port (%s/%s)",
                                  self->priv->name,
-                                 g_udev_device_get_subsystem (port),
-                                 g_udev_device_get_name (port));
+                                 mm_kernel_device_get_subsystem (port),
+                                 mm_kernel_device_get_name (port));
         g_object_unref (task);
         return;
     }
 
     /* Before launching any probing, check if the port is a net device. */
-    if (g_str_equal (g_udev_device_get_subsystem (port), "net")) {
+    if (g_str_equal (mm_kernel_device_get_subsystem (port), "net")) {
         mm_dbg ("(%s) [%s] probing deferred until result suggested",
-                self->priv->name, g_udev_device_get_name (port));
+                self->priv->name, mm_kernel_device_get_name (port));
         g_task_return_int (task, MM_PLUGIN_SUPPORTS_PORT_DEFER_UNTIL_SUGGESTED);
         g_object_unref (task);
         return;
@@ -735,7 +772,7 @@ mm_plugin_supports_port (MMPlugin            *self,
 
     /* Build flags depending on what probing needed */
     probe_run_flags = MM_PORT_PROBE_NONE;
-    if (!g_str_has_prefix (g_udev_device_get_name (port), "cdc-wdm")) {
+    if (!g_str_has_prefix (mm_kernel_device_get_name (port), "cdc-wdm")) {
         /* Serial ports... */
         if (self->priv->at)
             probe_run_flags |= MM_PORT_PROBE_AT;
@@ -745,9 +782,9 @@ mm_plugin_supports_port (MMPlugin            *self,
             probe_run_flags |= MM_PORT_PROBE_QCDM;
     } else {
         /* cdc-wdm ports... */
-        if (self->priv->qmi && !g_strcmp0 (mm_device_utils_get_port_driver (port), "qmi_wwan"))
+        if (self->priv->qmi && !g_strcmp0 (mm_kernel_device_get_driver (port), "qmi_wwan"))
             probe_run_flags |= MM_PORT_PROBE_QMI;
-        else if (self->priv->mbim && !g_strcmp0 (mm_device_utils_get_port_driver (port), "cdc_mbim"))
+        else if (self->priv->mbim && !g_strcmp0 (mm_kernel_device_get_driver (port), "cdc_mbim"))
             probe_run_flags |= MM_PORT_PROBE_MBIM;
         else
             probe_run_flags |= MM_PORT_PROBE_AT;
@@ -761,6 +798,8 @@ mm_plugin_supports_port (MMPlugin            *self,
             probe_run_flags |= MM_PORT_PROBE_AT_PRODUCT;
         if (self->priv->icera_probe || self->priv->allowed_icera || self->priv->forbidden_icera)
             probe_run_flags |= MM_PORT_PROBE_AT_ICERA;
+        if (self->priv->xmm_probe || self->priv->allowed_xmm || self->priv->forbidden_xmm)
+            probe_run_flags |= MM_PORT_PROBE_AT_XMM;
     }
 
     /* If no explicit probing was required, just request to grab it without probing anything.
@@ -780,7 +819,7 @@ mm_plugin_supports_port (MMPlugin            *self,
         mm_dbg ("(%s) [%s] not setting up AT probing tasks: "
                 "modem already has the expected single AT port",
                 self->priv->name,
-                g_udev_device_get_name (port));
+                mm_kernel_device_get_name (port));
 
         /* Assuming it won't be an AT port. We still run the probe anyway, in
          * case we need to check for other port types (e.g. QCDM) */
@@ -800,7 +839,7 @@ mm_plugin_supports_port (MMPlugin            *self,
     probe_list_str = mm_port_probe_flag_build_string_from_mask (ctx->flags);
     mm_dbg ("(%s) [%s] probe required: '%s'",
             self->priv->name,
-            g_udev_device_get_name (port),
+            mm_kernel_device_get_name (port),
             probe_list_str);
     g_free (probe_list_str);
 
@@ -819,9 +858,9 @@ mm_plugin_supports_port (MMPlugin            *self,
 /*****************************************************************************/
 
 MMPluginSupportsHint
-mm_plugin_discard_port_early (MMPlugin *self,
-                              MMDevice *device,
-                              GUdevDevice *port)
+mm_plugin_discard_port_early (MMPlugin       *self,
+                              MMDevice       *device,
+                              MMKernelDevice *port)
 {
     gboolean need_vendor_probing = FALSE;
     gboolean need_product_probing = FALSE;
@@ -865,7 +904,7 @@ mm_plugin_create_modem (MMPlugin  *self,
 
     /* Let the plugin create the modem from the port probe results */
     modem = MM_PLUGIN_GET_CLASS (self)->create_modem (MM_PLUGIN (self),
-                                                      mm_device_get_path (device),
+                                                      mm_device_get_uid (device),
                                                       mm_device_get_drivers (device),
                                                       mm_device_get_vendor (device),
                                                       mm_device_get_product (device),
@@ -904,21 +943,17 @@ mm_plugin_create_modem (MMPlugin  *self,
                         mm_port_probe_get_port_subsys (probe),
                         mm_port_probe_get_port_name (probe));
                 grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_get_port_subsys (probe),
-                                                   mm_port_probe_get_port_name (probe),
-                                                   mm_port_probe_get_parent_path (probe),
+                                                   mm_port_probe_peek_port (probe),
                                                    MM_PORT_TYPE_IGNORED,
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
             }
 #if !defined WITH_QMI
             else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     !g_strcmp0 (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)), "qmi_wwan")) {
+                     !g_strcmp0 (mm_kernel_device_get_driver (mm_port_probe_peek_port (probe)), "qmi_wwan")) {
                 /* Try to generically grab the port, but flagged as ignored */
                 grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_get_port_subsys (probe),
-                                                   mm_port_probe_get_port_name (probe),
-                                                   mm_port_probe_get_parent_path (probe),
+                                                   mm_port_probe_peek_port (probe),
                                                    MM_PORT_TYPE_IGNORED,
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
@@ -926,12 +961,10 @@ mm_plugin_create_modem (MMPlugin  *self,
 #endif
 #if !defined WITH_MBIM
             else if (mm_port_probe_get_port_type (probe) == MM_PORT_TYPE_NET &&
-                     !g_strcmp0 (mm_device_utils_get_port_driver (mm_port_probe_peek_port (probe)), "cdc_mbim")) {
+                     !g_strcmp0 (mm_kernel_device_get_driver (mm_port_probe_peek_port (probe)), "cdc_mbim")) {
                 /* Try to generically grab the port, but flagged as ignored */
                 grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_get_port_subsys (probe),
-                                                   mm_port_probe_get_port_name (probe),
-                                                   mm_port_probe_get_parent_path (probe),
+                                                   mm_port_probe_peek_port (probe),
                                                    MM_PORT_TYPE_IGNORED,
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
@@ -944,9 +977,7 @@ mm_plugin_create_modem (MMPlugin  *self,
                                                                  &inner_error);
             else
                 grabbed = mm_base_modem_grab_port (modem,
-                                                   mm_port_probe_get_port_subsys (probe),
-                                                   mm_port_probe_get_port_name (probe),
-                                                   mm_port_probe_get_parent_path (probe),
+                                                   mm_port_probe_peek_port (probe),
                                                    mm_port_probe_get_port_type (probe),
                                                    MM_PORT_SERIAL_AT_FLAG_NONE,
                                                    &inner_error);
@@ -962,20 +993,38 @@ mm_plugin_create_modem (MMPlugin  *self,
         guint i;
 
         for (i = 0; virtual_ports[i]; i++) {
-            GError *inner_error = NULL;
+            GError                  *inner_error = NULL;
+            MMKernelDevice          *kernel_device;
+            MMKernelEventProperties *properties;
 
-            if (!mm_base_modem_grab_port (modem,
-                                          "virtual",
-                                          virtual_ports[i],
-                                          NULL,
-                                          MM_PORT_TYPE_AT,
-                                          MM_PORT_SERIAL_AT_FLAG_NONE,
-                                          &inner_error)) {
+            properties = mm_kernel_event_properties_new ();
+            mm_kernel_event_properties_set_action (properties, "add");
+            mm_kernel_event_properties_set_subsystem (properties, "virtual");
+            mm_kernel_event_properties_set_name (properties, virtual_ports[i]);
+
+            /* Give an empty set of rules, because we don't want them to be
+             * loaded from the udev rules path (as there may not be any
+             * installed yet). */
+            kernel_device = mm_kernel_device_generic_new_with_rules (properties, NULL, &inner_error);
+            if (!kernel_device) {
+                mm_warn ("Could not grab port (virtual/%s): '%s'",
+                         virtual_ports[i],
+                         inner_error ? inner_error->message : "unknown error");
+                g_clear_error (&inner_error);
+            } else if (!mm_base_modem_grab_port (modem,
+                                                 kernel_device,
+                                                 MM_PORT_TYPE_AT,
+                                                 MM_PORT_SERIAL_AT_FLAG_NONE,
+                                                 &inner_error)) {
                 mm_warn ("Could not grab port (virtual/%s): '%s'",
                          virtual_ports[i],
                          inner_error ? inner_error->message : "unknown error");
                 g_clear_error (&inner_error);
             }
+
+            if (kernel_device)
+                g_object_unref (kernel_device);
+            g_object_unref (properties);
         }
     }
 
@@ -1086,6 +1135,18 @@ set_property (GObject *object,
         /* Construct only */
         self->priv->forbidden_icera = g_value_get_boolean (value);
         break;
+    case PROP_XMM_PROBE:
+        /* Construct only */
+        self->priv->xmm_probe = g_value_get_boolean (value);
+        break;
+    case PROP_ALLOWED_XMM:
+        /* Construct only */
+        self->priv->allowed_xmm = g_value_get_boolean (value);
+        break;
+    case PROP_FORBIDDEN_XMM:
+        /* Construct only */
+        self->priv->forbidden_xmm = g_value_get_boolean (value);
+        break;
     case PROP_CUSTOM_AT_PROBE:
         /* Construct only */
         self->priv->custom_at_probe = g_value_dup_boxed (value);
@@ -1178,6 +1239,15 @@ get_property (GObject *object,
     case PROP_FORBIDDEN_ICERA:
         g_value_set_boolean (value, self->priv->forbidden_icera);
         break;
+    case PROP_XMM_PROBE:
+        g_value_set_boolean (value, self->priv->xmm_probe);
+        break;
+    case PROP_ALLOWED_XMM:
+        g_value_set_boolean (value, self->priv->allowed_xmm);
+        break;
+    case PROP_FORBIDDEN_XMM:
+        g_value_set_boolean (value, self->priv->forbidden_xmm);
+        break;
     case PROP_CUSTOM_AT_PROBE:
         g_value_set_boxed (value, self->priv->custom_at_probe);
         break;
@@ -1400,6 +1470,30 @@ mm_plugin_class_init (MMPluginClass *klass)
                                FALSE,
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+    g_object_class_install_property
+        (object_class, PROP_XMM_PROBE,
+         g_param_spec_boolean (MM_PLUGIN_XMM_PROBE,
+                               "XMM probe",
+                               "Request to probe for XMM support.",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property
+        (object_class, PROP_ALLOWED_XMM,
+         g_param_spec_boolean (MM_PLUGIN_ALLOWED_XMM,
+                               "Allowed XMM",
+                               "Whether XMM support is allowed in this plugin.",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+    g_object_class_install_property
+        (object_class, PROP_FORBIDDEN_XMM,
+         g_param_spec_boolean (MM_PLUGIN_FORBIDDEN_XMM,
+                               "Allowed XMM",
+                               "Whether XMM support is forbidden in this plugin.",
+                               FALSE,
+                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     g_object_class_install_property
         (object_class, PROP_CUSTOM_AT_PROBE,
          g_param_spec_boxed (MM_PLUGIN_CUSTOM_AT_PROBE,
