@@ -35,6 +35,7 @@
 
 #include "mm-port-serial.h"
 #include "mm-log.h"
+#include "mm-helper-enums-types.h"
 
 static gboolean port_serial_queue_process          (gpointer data);
 static void     port_serial_schedule_queue_process (MMPortSerial *self,
@@ -53,10 +54,10 @@ enum {
     PROP_BITS,
     PROP_PARITY,
     PROP_STOPBITS,
+    PROP_FLOW_CONTROL,
     PROP_SEND_DELAY,
     PROP_FD,
     PROP_SPEW_CONTROL,
-    PROP_RTS_CTS,
     PROP_FLASH_OK,
 
     LAST_PROP
@@ -95,9 +96,9 @@ struct _MMPortSerialPrivate {
     guint bits;
     char parity;
     guint stopbits;
+    MMFlowControl flow_control;
     guint64 send_delay;
     gboolean spew_control;
-    gboolean rts_cts;
     gboolean flash_ok;
 
     guint queue_id;
@@ -110,8 +111,8 @@ struct _MMPortSerialPrivate {
 
     guint connected_id;
 
-    gpointer flash_ctx;
-    gpointer reopen_ctx;
+    GTask *flash_task;
+    GTask *reopen_task;
 };
 
 /*****************************************************************************/
@@ -210,95 +211,13 @@ mm_port_serial_command (MMPortSerial *self,
 
 /*****************************************************************************/
 
-#if 0
-static const char *
-baud_to_string (int baud)
+static gboolean
+parse_baudrate (guint  baudrate_num,
+                guint *out_baudrate_speed)
 {
-    const char *speed = NULL;
+    guint speed;
 
-    switch (baud) {
-    case B0:
-        speed = "0";
-        break;
-    case B50:
-        speed = "50";
-        break;
-    case B75:
-        speed = "75";
-        break;
-    case B110:
-        speed = "110";
-        break;
-    case B150:
-        speed = "150";
-        break;
-    case B300:
-        speed = "300";
-        break;
-    case B600:
-        speed = "600";
-        break;
-    case B1200:
-        speed = "1200";
-        break;
-    case B2400:
-        speed = "2400";
-        break;
-    case B4800:
-        speed = "4800";
-        break;
-    case B9600:
-        speed = "9600";
-        break;
-    case B19200:
-        speed = "19200";
-        break;
-    case B38400:
-        speed = "38400";
-        break;
-    case B57600:
-        speed = "57600";
-        break;
-    case B115200:
-        speed = "115200";
-        break;
-    case B460800:
-        speed = "460800";
-        break;
-    default:
-        break;
-    }
-
-    return speed;
-}
-
-void
-mm_port_serial_print_config (MMPortSerial *port,
-                             const char *detail)
-{
-    struct termios stbuf;
-    int err;
-
-    err = tcgetattr (self->priv->fd, &stbuf);
-    if (err) {
-        mm_warn ("*** %s (%s): (%s) tcgetattr() error %d",
-                 __func__, detail, mm_port_get_device (MM_PORT (port)), errno);
-        return;
-    }
-
-    mm_info ("(%s): (%s) baud rate: %d (%s)",
-             detail, mm_port_get_device (MM_PORT (port)),
-             stbuf.c_cflag & CBAUD,
-             baud_to_string (stbuf.c_cflag & CBAUD));
-}
-#endif
-
-static int
-parse_baudrate (guint i)
-{
-    int speed;
-
-    switch (i) {
+    switch (baudrate_num) {
     case 0:
         speed = B0;
         break;
@@ -344,15 +263,22 @@ parse_baudrate (guint i)
     case 115200:
         speed = B115200;
         break;
+    case 230400:
+        speed = B230400;
+        break;
     case 460800:
         speed = B460800;
         break;
+    case 921600:
+        speed = B921600;
+        break;
     default:
-        mm_warn ("Invalid baudrate '%d'", i);
-        speed = B9600;
+        return FALSE;
     }
 
-    return speed;
+    if (out_baudrate_speed)
+        *out_baudrate_speed = speed;
+    return TRUE;
 }
 
 static int
@@ -427,20 +353,124 @@ parse_stopbits (guint i)
     return stopbits;
 }
 
+static gboolean
+internal_tcsetattr (MMPortSerial          *self,
+                    gint                   fd,
+                    const struct termios  *options,
+                    GError               **error)
+{
+    guint          count;
+    struct termios other;
+
+#define MAX_TCSETATTR_RETRIES 4
+
+    for (count = 0; count < MAX_TCSETATTR_RETRIES; count++) {
+        /* try to set the new port attributes */
+        errno = 0;
+        if (tcsetattr (fd, TCSANOW, options) == 0)
+            break;
+
+        /* hard error if not EAGAIN */
+        if (errno != EAGAIN) {
+            g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                         "couldn't set serial port attributes: %s", g_strerror (errno));
+            return FALSE;
+        }
+
+        /* try a few times if EAGAIN */
+        g_usleep (100000);
+    }
+
+    /* too many retries? */
+    if (count == MAX_TCSETATTR_RETRIES) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "couldn't set serial port attributes: too many retries (%u)", count);
+        return FALSE;
+    }
+
+    /* tcsetattr() returns 0 if any of the requested attributes could be set,
+     * so we should double-check that all were set and log if not. Just with
+     * debug level, as we're ignoring this issue all together anyway.
+     */
+    memset (&other, 0, sizeof (struct termios));
+    errno = 0;
+    if (tcgetattr (fd, &other) != 0)
+        mm_dbg ("(%s): couldn't get serial port attributes after setting them: %s",
+                mm_port_get_device (MM_PORT (self)), g_strerror (errno));
+    else if (memcmp (options, &other, sizeof (struct termios)) != 0)
+        mm_dbg ("(%s): port attributes not fully set",
+                mm_port_get_device (MM_PORT (self)));
+
+#undef MAX_TCSETATTR_RETRIES
+
+    return TRUE;
+}
+
+static gboolean
+set_flow_control_termios (MMPortSerial    *self,
+                          MMFlowControl    flow_control,
+                          struct termios  *options)
+{
+    gboolean had_xon_xoff;
+    gboolean had_rts_cts;
+    tcflag_t iflag_orig, cflag_orig;
+
+    iflag_orig = options->c_iflag;
+    cflag_orig = options->c_cflag;
+
+    had_xon_xoff = !!(options->c_iflag & (IXON | IXOFF));
+    options->c_iflag &= ~(IXON | IXOFF | IXANY);
+
+    had_rts_cts  = !!(options->c_cflag & (CRTSCTS));
+    options->c_cflag &= ~(CRTSCTS);
+
+    /* setup the requested flags */
+    switch (flow_control) {
+        case MM_FLOW_CONTROL_XON_XOFF:
+            mm_dbg ("(%s): enabling XON/XOFF flow control", mm_port_get_device (MM_PORT (self)));
+            options->c_iflag |= (IXON | IXOFF | IXANY);
+            break;
+        case MM_FLOW_CONTROL_RTS_CTS:
+            mm_dbg ("(%s): enabling RTS/CTS flow control", mm_port_get_device (MM_PORT (self)));
+            options->c_cflag |= (CRTSCTS);
+            break;
+        case MM_FLOW_CONTROL_NONE:
+        case MM_FLOW_CONTROL_UNKNOWN:
+            if (had_xon_xoff)
+                mm_dbg ("(%s): disabling XON/XOFF flow control", mm_port_get_device (MM_PORT (self)));
+            if (had_rts_cts)
+                mm_dbg ("(%s): disabling RTS/CTS flow control", mm_port_get_device (MM_PORT (self)));
+            break;
+        default:
+            g_assert_not_reached ();
+    }
+
+    return iflag_orig != options->c_iflag || cflag_orig != options->c_cflag;
+}
+
 static gboolean
 real_config_fd (MMPortSerial *self, int fd, GError **error)
 {
-    struct termios stbuf, other;
-    int speed;
-    int bits;
-    int parity;
-    int stopbits;
+    struct termios stbuf;
+    guint speed;
+    gint bits;
+    gint parity;
+    gint stopbits;
 
     /* No setup if not a tty */
     if (mm_port_get_subsys (MM_PORT (self)) != MM_PORT_SUBSYS_TTY)
         return TRUE;
 
-    speed = parse_baudrate (self->priv->baud);
+    mm_dbg ("(%s): setting up baudrate: %u",
+            mm_port_get_device (MM_PORT (self)),
+            self->priv->baud);
+    if (!parse_baudrate (self->priv->baud, &speed) || speed == B0) {
+        mm_warn ("(%s): baudrate invalid: %u; defaulting to 57600",
+                 mm_port_get_device (MM_PORT (self)),
+                 self->priv->baud);
+        speed = B57600;
+    }
+
     bits = parse_bits (self->priv->bits);
     parity = parse_parity (self->priv->parity);
     stopbits = parse_stopbits (self->priv->stopbits);
@@ -452,21 +482,19 @@ real_config_fd (MMPortSerial *self, int fd, GError **error)
                  errno);
     }
 
-    stbuf.c_iflag &= ~(IGNCR | ICRNL | IUCLC | INPCK | IXON | IXANY );
+    stbuf.c_cflag &= ~(CBAUD | CSIZE | CSTOPB | PARENB | PARODD | CRTSCTS);
+    stbuf.c_iflag &= ~(IGNCR | ICRNL | IUCLC | INPCK | IXON | IXOFF | IXANY );
     stbuf.c_oflag &= ~(OPOST | OLCUC | OCRNL | ONLCR | ONLRET);
     stbuf.c_lflag &= ~(ICANON | ECHO | ECHOE | ECHONL);
     stbuf.c_cc[VMIN] = 1;
     stbuf.c_cc[VTIME] = 0;
     stbuf.c_cc[VEOF] = 1;
 
-    /* Use software handshaking and ignore parity/framing errors */
-    stbuf.c_iflag |= (IXON | IXOFF | IXANY | IGNPAR);
+    /* Ignore parity/framing errors */
+    stbuf.c_iflag |= IGNPAR;
 
-    /* Set up port speed and serial attributes; also ignore modem control
-     * lines since most drivers don't implement RTS/CTS anyway.
-     */
-    stbuf.c_cflag &= ~(CBAUD | CSIZE | CSTOPB | PARENB | CRTSCTS);
-    stbuf.c_cflag |= (bits | CREAD | 0 | parity | stopbits | CLOCAL);
+    /* Set up port speed and serial attributes and enable receiver in local mode */
+    stbuf.c_cflag |= (bits | parity | stopbits | CLOCAL | CREAD);
 
     errno = 0;
     if (cfsetispeed (&stbuf, speed) != 0) {
@@ -488,32 +516,21 @@ real_config_fd (MMPortSerial *self, int fd, GError **error)
         return FALSE;
     }
 
-    if (tcsetattr (fd, TCSANOW, &stbuf) < 0) {
-        g_set_error (error,
-                     MM_CORE_ERROR,
-                     MM_CORE_ERROR_FAILED,
-                     "%s: failed to set serial port attributes; errno %d",
-                     __func__, errno);
-        return FALSE;
-    }
+    if (self->priv->flow_control != MM_FLOW_CONTROL_UNKNOWN) {
+        gchar *str;
 
-    /* tcsetattr() returns 0 if any of the requested attributes could be set,
-     * so we should double-check that all were set and log a warning if not.
-     */
-    memset (&other, 0, sizeof (struct termios));
-    errno = 0;
-    if (tcgetattr (fd, &other) != 0) {
-        mm_warn ("(%s): tcgetattr() error: %d",
-                 mm_port_get_device (MM_PORT (self)),
-                 errno);
-    }
+        str = mm_flow_control_build_string_from_mask (self->priv->flow_control);
+        mm_dbg ("(%s): flow control explicitly requested for device is: %s",
+                mm_port_get_device (MM_PORT (self)),
+                str ? str : "unknown");
+        g_free (str);
+    } else
+        mm_dbg ("(%s): no flow control explicitly requested for device",
+                mm_port_get_device (MM_PORT (self)));
 
-    if (memcmp (&stbuf, &other, sizeof (other)) != 0) {
-        mm_warn ("(%s): port attributes not fully set",
-                 mm_port_get_device (MM_PORT (self)));
-    }
+    set_flow_control_termios (self, self->priv->flow_control, &stbuf);
 
-    return TRUE;
+    return internal_tcsetattr (self, fd, &stbuf, error);
 }
 
 static void
@@ -634,7 +651,7 @@ port_serial_process_command (MMPortSerial *self,
             /* Just keep on, will retry... */
             written = 0;
         } else
-          written = bytes_sent;
+            written = bytes_sent;
 
         ctx->idx += written;
     } else
@@ -850,21 +867,25 @@ port_serial_queue_process (gpointer data)
 
     /* Setup the cancellable so that we can stop waiting for a response */
     if (ctx->cancellable) {
+        gulong cancellable_id;
+
         self->priv->cancellable = g_object_ref (ctx->cancellable);
-        self->priv->cancellable_id = (g_cancellable_connect (
-                                          ctx->cancellable,
-                                          (GCallback)port_serial_response_wait_cancelled,
-                                          self,
-                                          NULL));
-        if (!self->priv->cancellable_id) {
-            error = g_error_new (MM_CORE_ERROR,
-                                 MM_CORE_ERROR_CANCELLED,
-                                 "Won't wait for the reply");
-            /* Note: may complete last operation and unref the MMPortSerial */
-            port_serial_got_response (self, NULL, error);
-            g_error_free (error);
+
+        /* If the GCancellable is already cancelled here, the callback will be
+         * called right away, and a GError will be propagated as response. In
+         * this case we need to completely avoid doing anything else with the
+         * MMPortSerial, as it may already be disposed.
+         * So, use an intermediate variable to store the cancellable id, and
+         * just return without further processing if we're already cancelled.
+         */
+        cancellable_id = g_cancellable_connect (ctx->cancellable,
+                                                (GCallback)port_serial_response_wait_cancelled,
+                                                self,
+                                                NULL);
+        if (!cancellable_id)
             return G_SOURCE_REMOVE;
-        }
+
+        self->priv->cancellable_id = cancellable_id;
     }
 
     /* If the command is finished being sent, schedule the timeout */
@@ -1145,7 +1166,7 @@ mm_port_serial_open (MMPortSerial *self, GError **error)
         return FALSE;
     }
 
-    if (self->priv->reopen_ctx) {
+    if (self->priv->reopen_task) {
         g_set_error (error,
                      MM_SERIAL_ERROR,
                      MM_SERIAL_ERROR_OPEN_FAILED,
@@ -1154,6 +1175,15 @@ mm_port_serial_open (MMPortSerial *self, GError **error)
         return FALSE;
     }
 
+    if (mm_port_get_connected (MM_PORT (self))) {
+        g_set_error (error,
+                     MM_SERIAL_ERROR,
+                     MM_SERIAL_ERROR_OPEN_FAILED,
+                     "Could not open serial device %s: port is connected",
+                     device);
+        return FALSE;
+    }
+
     if (self->priv->open_count) {
         /* Already open */
         goto success;
@@ -1301,7 +1331,6 @@ error:
     mm_warn ("(%s) failed to open serial device", device);
 
     if (self->priv->iochannel) {
-        g_io_channel_shutdown (self->priv->iochannel, FALSE, NULL);
         g_io_channel_unref (self->priv->iochannel);
         self->priv->iochannel = NULL;
     }
@@ -1389,7 +1418,9 @@ _close_internal (MMPortSerial *self, gboolean force)
         /* Destroy channel */
         if (self->priv->iochannel) {
             data_watch_enable (self, FALSE);
-            g_io_channel_shutdown (self->priv->iochannel, TRUE, NULL);
+            /* unref() without g_io_channel_shutdown() to destroy the channel
+             * without closing the fd. The close() is called explicitly after.
+             */
             g_io_channel_unref (self->priv->iochannel);
             self->priv->iochannel = NULL;
         }
@@ -1494,20 +1525,15 @@ port_serial_close_force (MMPortSerial *self)
 /* Reopen */
 
 typedef struct {
-    MMPortSerial *self;
-    GSimpleAsyncResult *result;
     guint initial_open_count;
     guint reopen_id;
 } ReopenContext;
 
 static void
-reopen_context_complete_and_free (ReopenContext *ctx)
+reopen_context_free (ReopenContext *ctx)
 {
     if (ctx->reopen_id)
         g_source_remove (ctx->reopen_id);
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
     g_slice_free (ReopenContext, ctx);
 }
 
@@ -1516,54 +1542,56 @@ mm_port_serial_reopen_finish (MMPortSerial *port,
                               GAsyncResult *res,
                               GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static void
 port_serial_reopen_cancel (MMPortSerial *self)
 {
-    ReopenContext *ctx;
+    GTask *task;
 
-    if (!self->priv->reopen_ctx)
+    if (!self->priv->reopen_task)
         return;
 
-    /* Recover context */
-    ctx = (ReopenContext *)self->priv->reopen_ctx;
-    self->priv->reopen_ctx = NULL;
+    /* Recover task */
+    task = self->priv->reopen_task;
+    self->priv->reopen_task = NULL;
 
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_CANCELLED,
-                                     "Reopen cancelled");
-    reopen_context_complete_and_free (ctx);
+    g_task_return_new_error (task,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_CANCELLED,
+                             "Reopen cancelled");
+    g_object_unref (task);
 }
 
 static gboolean
 reopen_do (MMPortSerial *self)
 {
+    GTask *task;
     ReopenContext *ctx;
     GError *error = NULL;
     guint i;
 
-    /* Recover context */
-    g_assert (self->priv->reopen_ctx != NULL);
-    ctx = (ReopenContext *)self->priv->reopen_ctx;
-    self->priv->reopen_ctx = NULL;
+    /* Recover task */
+    g_assert (self->priv->reopen_task != NULL);
+    task = self->priv->reopen_task;
+    self->priv->reopen_task = NULL;
 
+    ctx = g_task_get_task_data (task);
     ctx->reopen_id = 0;
 
     for (i = 0; i < ctx->initial_open_count; i++) {
-        if (!mm_port_serial_open (ctx->self, &error)) {
+        if (!mm_port_serial_open (self, &error)) {
             g_prefix_error (&error, "Couldn't reopen port (%u): ", i);
             break;
         }
     }
 
     if (error)
-        g_simple_async_result_take_error (ctx->result, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    reopen_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 
     return G_SOURCE_REMOVE;
 }
@@ -1575,35 +1603,34 @@ mm_port_serial_reopen (MMPortSerial *self,
                        gpointer user_data)
 {
     ReopenContext *ctx;
+    GTask *task;
     guint i;
 
     g_return_if_fail (MM_IS_PORT_SERIAL (self));
 
     /* Setup context */
     ctx = g_slice_new0 (ReopenContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_port_serial_reopen);
     ctx->initial_open_count = self->priv->open_count;
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)reopen_context_free);
+
     if (self->priv->forced_close) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Serial port has been forced close.");
-        reopen_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "Serial port has been forced close.");
+        g_object_unref (task);
         return;
     }
 
     /* If already reopening, halt */
-    if (self->priv->reopen_ctx) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_IN_PROGRESS,
-                                         "Modem is already being reopened");
-        reopen_context_complete_and_free (ctx);
+    if (self->priv->reopen_task) {
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_IN_PROGRESS,
+                                 "Modem is already being reopened");
+        g_object_unref (task);
         return;
     }
 
@@ -1620,7 +1647,7 @@ mm_port_serial_reopen (MMPortSerial *self,
         ctx->reopen_id = g_idle_add ((GSourceFunc)reopen_do, self);
 
     /* Store context in private info */
-    self->priv->reopen_ctx = ctx;
+    self->priv->reopen_task = task;
 }
 
 static gboolean
@@ -1648,15 +1675,11 @@ static gboolean
 set_speed (MMPortSerial *self, speed_t speed, GError **error)
 {
     struct termios options;
-    int fd, count = 4;
-    gboolean success = FALSE;
 
     g_assert (self->priv->fd >= 0);
 
-    fd = self->priv->fd;
-
     memset (&options, 0, sizeof (struct termios));
-    if (tcgetattr (fd, &options) != 0) {
+    if (tcgetattr (self->priv->fd, &options) != 0) {
         g_set_error (error,
                      MM_CORE_ERROR,
                      MM_CORE_ERROR_FAILED,
@@ -1669,60 +1692,22 @@ set_speed (MMPortSerial *self, speed_t speed, GError **error)
     cfsetospeed (&options, speed);
     options.c_cflag |= (CLOCAL | CREAD);
 
-    /* Configure flow control as well here */
-    if (self->priv->rts_cts)
-        options.c_cflag |= (CRTSCTS);
-
-    while (count-- > 0) {
-        if (tcsetattr (fd, TCSANOW, &options) == 0) {
-            success = TRUE;
-            break;  /* Operation successful */
-        }
-
-        /* Try a few times if EAGAIN */
-        if (errno == EAGAIN)
-            g_usleep (100000);
-        else {
-            /* If not EAGAIN, hard error */
-            g_set_error (error,
-                            MM_CORE_ERROR,
-                            MM_CORE_ERROR_FAILED,
-                            "%s: tcsetattr() error %d",
-                            __func__, errno);
-            return FALSE;
-        }
-    }
-
-    if (!success) {
-        g_set_error (error,
-                        MM_CORE_ERROR,
-                        MM_CORE_ERROR_FAILED,
-                        "%s: tcsetattr() retry timeout",
-                        __func__);
-        return FALSE;
-    }
-
-    return TRUE;
+    return internal_tcsetattr (self, self->priv->fd, &options, error);
 }
 
 /*****************************************************************************/
 /* Flash */
 
 typedef struct {
-    GSimpleAsyncResult *result;
-    MMPortSerial *self;
     speed_t current_speed;
     guint flash_id;
 } FlashContext;
 
 static void
-flash_context_complete_and_free (FlashContext *ctx)
+flash_context_free (FlashContext *ctx)
 {
     if (ctx->flash_id)
         g_source_remove (ctx->flash_id);
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->self);
     g_slice_free (FlashContext, ctx);
 }
 
@@ -1731,44 +1716,46 @@ mm_port_serial_flash_finish (MMPortSerial *port,
                              GAsyncResult *res,
                              GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 void
 mm_port_serial_flash_cancel (MMPortSerial *self)
 {
-    FlashContext *ctx;
+    GTask *task;
 
-    if (!self->priv->flash_ctx)
+    if (!self->priv->flash_task)
         return;
 
-    /* Recover context */
-    ctx = (FlashContext *)self->priv->flash_ctx;
-    self->priv->flash_ctx = NULL;
+    /* Recover task */
+    task = self->priv->flash_task;
+    self->priv->flash_task = NULL;
 
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_CANCELLED,
-                                     "Flash cancelled");
-    flash_context_complete_and_free (ctx);
+    g_task_return_new_error (task,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_CANCELLED,
+                             "Flash cancelled");
+    g_object_unref (task);
 }
 
 static gboolean
 flash_do (MMPortSerial *self)
 {
+    GTask *task;
     FlashContext *ctx;
     GError *error = NULL;
 
-    /* Recover context */
-    g_assert (self->priv->flash_ctx != NULL);
-    ctx = (FlashContext *)self->priv->flash_ctx;
-    self->priv->flash_ctx = NULL;
+    /* Recover task */
+    g_assert (self->priv->flash_task != NULL);
+    task = self->priv->flash_task;
+    self->priv->flash_task = NULL;
 
+    ctx = g_task_get_task_data (task);
     ctx->flash_id = 0;
 
     if (self->priv->flash_ok && mm_port_get_subsys (MM_PORT (self)) == MM_PORT_SUBSYS_TTY) {
         if (ctx->current_speed) {
-            if (!set_speed (ctx->self, ctx->current_speed, &error))
+            if (!set_speed (self, ctx->current_speed, &error))
                 g_assert (error);
         } else {
             error = g_error_new_literal (MM_SERIAL_ERROR,
@@ -1778,10 +1765,10 @@ flash_do (MMPortSerial *self)
     }
 
     if (error)
-        g_simple_async_result_take_error (ctx->result, error);
+        g_task_return_error (task, error);
     else
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    flash_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 
     return G_SOURCE_REMOVE;
 }
@@ -1794,6 +1781,7 @@ mm_port_serial_flash (MMPortSerial *self,
                       gpointer user_data)
 {
     FlashContext *ctx;
+    GTask *task;
     GError *error = NULL;
     gboolean success;
 
@@ -1801,33 +1789,31 @@ mm_port_serial_flash (MMPortSerial *self,
 
     /* Setup context */
     ctx = g_slice_new0 (FlashContext);
-    ctx->self = g_object_ref (self);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             mm_port_serial_flash);
+
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)flash_context_free);
 
     if (!mm_port_serial_is_open (self)) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_SERIAL_ERROR,
-                                         MM_SERIAL_ERROR_NOT_OPEN,
-                                         "The serial port is not open.");
-        flash_context_complete_and_free (ctx);
+        g_task_return_new_error (task,
+                                 MM_SERIAL_ERROR,
+                                 MM_SERIAL_ERROR_NOT_OPEN,
+                                 "The serial port is not open.");
+        g_object_unref (task);
         return;
     }
 
-    if (self->priv->flash_ctx) {
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_IN_PROGRESS,
-                                         "Modem is already being flashed.");
-        flash_context_complete_and_free (ctx);
+    if (self->priv->flash_task) {
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_IN_PROGRESS,
+                                 "Modem is already being flashed.");
+        g_object_unref (task);
         return;
     }
 
     /* Flashing only in TTY */
     if (!self->priv->flash_ok || mm_port_get_subsys (MM_PORT (self)) != MM_PORT_SUBSYS_TTY) {
-        self->priv->flash_ctx = ctx;
+        self->priv->flash_task = task;
         ctx->flash_id = g_idle_add ((GSourceFunc)flash_do, self);
         return;
     }
@@ -1835,26 +1821,76 @@ mm_port_serial_flash (MMPortSerial *self,
     /* Grab current speed so we can reset it after flashing */
     success = get_speed (self, &ctx->current_speed, &error);
     if (!success && !ignore_errors) {
-        g_simple_async_result_take_error (ctx->result, error);
-        flash_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
     g_clear_error (&error);
 
     success = set_speed (self, B0, &error);
     if (!success && !ignore_errors) {
-        g_simple_async_result_take_error (ctx->result, error);
-        flash_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
     g_clear_error (&error);
 
-    self->priv->flash_ctx = ctx;
+    self->priv->flash_task = task;
     ctx->flash_id = g_timeout_add (flash_time, (GSourceFunc)flash_do, self);
 }
 
 /*****************************************************************************/
 
+gboolean
+mm_port_serial_set_flow_control (MMPortSerial   *self,
+                                 MMFlowControl   flow_control,
+                                 GError        **error)
+{
+    struct termios options;
+    gchar *flow_control_str = NULL;
+    GError *inner_error = NULL;
+
+    /* retrieve current settings */
+    memset (&options, 0, sizeof (struct termios));
+    if (tcgetattr (self->priv->fd, &options) != 0) {
+        inner_error = g_error_new (MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                                   "couldn't get serial port attributes: %s", g_strerror (errno));
+        goto out;
+    }
+
+    flow_control_str = mm_flow_control_build_string_from_mask (flow_control);
+
+    /* Return if current settings are already what we want */
+    if (!set_flow_control_termios (self, flow_control, &options)) {
+        mm_dbg ("(%s): no need to change flow control settings: already %s",
+                mm_port_get_device (MM_PORT (self)), flow_control_str);
+        goto out;
+    }
+
+    if (!internal_tcsetattr (self, self->priv->fd, &options, &inner_error))
+        goto out;
+
+    mm_dbg ("(%s): flow control settings updated to %s",
+            mm_port_get_device (MM_PORT (self)), flow_control_str);
+
+ out:
+    g_free (flow_control_str);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+MMFlowControl
+mm_port_serial_get_flow_control (MMPortSerial *self)
+{
+    return self->priv->flow_control;
+}
+
+/*****************************************************************************/
+
 MMPortSerial *
 mm_port_serial_new (const char *name, MMPortType ptype)
 {
@@ -1919,6 +1955,7 @@ mm_port_serial_init (MMPortSerial *self)
     self->priv->bits = 8;
     self->priv->parity = 'n';
     self->priv->stopbits = 1;
+    self->priv->flow_control = MM_FLOW_CONTROL_UNKNOWN;
     self->priv->send_delay = 1000;
 
     self->priv->queue = g_queue_new ();
@@ -1949,15 +1986,15 @@ set_property (GObject *object,
     case PROP_STOPBITS:
         self->priv->stopbits = g_value_get_uint (value);
         break;
+    case PROP_FLOW_CONTROL:
+        self->priv->flow_control = g_value_get_flags (value);
+        break;
     case PROP_SEND_DELAY:
         self->priv->send_delay = g_value_get_uint64 (value);
         break;
     case PROP_SPEW_CONTROL:
         self->priv->spew_control = g_value_get_boolean (value);
         break;
-    case PROP_RTS_CTS:
-        self->priv->rts_cts = g_value_get_boolean (value);
-        break;
     case PROP_FLASH_OK:
         self->priv->flash_ok = g_value_get_boolean (value);
         break;
@@ -1991,15 +2028,15 @@ get_property (GObject *object,
     case PROP_STOPBITS:
         g_value_set_uint (value, self->priv->stopbits);
         break;
+    case PROP_FLOW_CONTROL:
+        g_value_set_flags (value, self->priv->flow_control);
+        break;
     case PROP_SEND_DELAY:
         g_value_set_uint64 (value, self->priv->send_delay);
         break;
     case PROP_SPEW_CONTROL:
         g_value_set_boolean (value, self->priv->spew_control);
         break;
-    case PROP_RTS_CTS:
-        g_value_set_boolean (value, self->priv->rts_cts);
-        break;
     case PROP_FLASH_OK:
         g_value_set_boolean (value, self->priv->flash_ok);
         break;
@@ -2091,6 +2128,15 @@ mm_port_serial_class_init (MMPortSerialClass *klass)
                             1, 2, 1,
                             G_PARAM_READWRITE));
 
+    g_object_class_install_property
+        (object_class, PROP_FLOW_CONTROL,
+         g_param_spec_flags (MM_PORT_SERIAL_FLOW_CONTROL,
+                             "FlowControl",
+                             "Select flow control",
+                             MM_TYPE_FLOW_CONTROL,
+                             MM_FLOW_CONTROL_UNKNOWN,
+                             G_PARAM_READWRITE));
+
     g_object_class_install_property
         (object_class, PROP_SEND_DELAY,
          g_param_spec_uint64 (MM_PORT_SERIAL_SEND_DELAY,
@@ -2107,14 +2153,6 @@ mm_port_serial_class_init (MMPortSerialClass *klass)
                                FALSE,
                                G_PARAM_READWRITE));
 
-    g_object_class_install_property
-        (object_class, PROP_RTS_CTS,
-         g_param_spec_boolean (MM_PORT_SERIAL_RTS_CTS,
-                               "RTSCTS",
-                               "Enable RTS/CTS flow control",
-                               FALSE,
-                               G_PARAM_READWRITE));
-
     g_object_class_install_property
         (object_class, PROP_FLASH_OK,
          g_param_spec_boolean (MM_PORT_SERIAL_FLASH_OK,
