@@ -91,6 +91,106 @@ ensure_qmi_client (GTask       *task,
 /*****************************************************************************/
 /* Load SIM ID (ICCID) */
 
+static GArray *
+uim_read_finish (QmiClientUim  *client,
+                 GAsyncResult  *res,
+                 GError       **error)
+{
+    return g_task_propagate_pointer (G_TASK (res), error);
+}
+
+static void
+uim_read_ready (QmiClientUim *client,
+                GAsyncResult *res,
+                GTask        *task)
+{
+    QmiMessageUimReadTransparentOutput *output;
+    GError *error = NULL;
+
+    output = qmi_client_uim_read_transparent_finish (client, res, &error);
+    if (!output) {
+        g_prefix_error (&error, "QMI operation failed: ");
+        g_task_return_error (task, error);
+    } else if (!qmi_message_uim_read_transparent_output_get_result (output, &error)) {
+        g_prefix_error (&error, "Couldn't read data from UIM: ");
+        g_task_return_error (task, error);
+    } else {
+        GArray *read_result = NULL;
+
+        qmi_message_uim_read_transparent_output_get_read_result (output, &read_result, NULL);
+        if (read_result)
+            g_task_return_pointer (task,
+                                   g_array_ref (read_result),
+                                   (GDestroyNotify) g_array_unref);
+        else
+            g_task_return_new_error (task,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Read malformed data from UIM");
+    }
+
+    if (output)
+        qmi_message_uim_read_transparent_output_unref (output);
+
+    g_object_unref (task);
+}
+
+static void
+uim_read (MMSimQmi            *self,
+          guint16              file_id,
+          const guint16       *file_path,
+          gsize                file_path_len,
+          GAsyncReadyCallback  callback,
+          gpointer             user_data)
+{
+    GTask *task;
+    QmiClient *client = NULL;
+    GArray *file_path_bytes;
+    gsize i;
+    QmiMessageUimReadTransparentInput *input;
+
+    task = g_task_new (self, NULL, callback, user_data);
+
+    if (!ensure_qmi_client (task,
+                            self,
+                            QMI_SERVICE_UIM, &client))
+        return;
+
+    file_path_bytes = g_array_sized_new (FALSE, FALSE, 1, file_path_len * 2);
+    for (i = 0; i < file_path_len; ++i) {
+        guint8 byte;
+
+        byte = file_path[i] & 0xFF;
+        g_array_append_val (file_path_bytes, byte);
+        byte = (file_path[i] >> 8) & 0xFF;
+        g_array_append_val (file_path_bytes, byte);
+    }
+
+    input = qmi_message_uim_read_transparent_input_new ();
+    qmi_message_uim_read_transparent_input_set_session_information (
+        input,
+        QMI_UIM_SESSION_TYPE_PRIMARY_GW_PROVISIONING,
+        "",
+        NULL);
+    qmi_message_uim_read_transparent_input_set_file (input,
+                                                     file_id,
+                                                     file_path_bytes,
+                                                     NULL);
+    qmi_message_uim_read_transparent_input_set_read_information (input,
+                                                                 0,
+                                                                 0,
+                                                                 NULL);
+    g_array_unref (file_path_bytes);
+
+    qmi_client_uim_read_transparent (QMI_CLIENT_UIM (client),
+                                     input,
+                                     10,
+                                     NULL,
+                                     (GAsyncReadyCallback)uim_read_ready,
+                                     task);
+    qmi_message_uim_read_transparent_input_unref (input);
+}
+
 static gchar *
 load_sim_identifier_finish (MMBaseSim     *self,
                             GAsyncResult  *res,
@@ -99,6 +199,44 @@ load_sim_identifier_finish (MMBaseSim     *self,
     return g_task_propagate_pointer (G_TASK (res), error);
 }
 
+static void
+uim_get_iccid_ready (QmiClientUim *client,
+                     GAsyncResult *res,
+                     GTask        *task)
+{
+    GError *error = NULL;
+    GArray *read_result;
+    gchar *iccid;
+
+    read_result = uim_read_finish (client, res, &error);
+    if (!read_result) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+
+    iccid = mm_bcd_to_string ((const guint8 *) read_result->data, read_result->len);
+    g_assert (iccid);
+    g_task_return_pointer (task, iccid, g_free);
+    g_object_unref (task);
+
+    g_array_unref (read_result);
+}
+
+static void
+uim_get_iccid (MMSimQmi *self,
+               GTask    *task)
+{
+    static const guint16 file_path[] = { 0x3F00 };
+
+    uim_read (self,
+              0x2FE2,
+              file_path,
+              G_N_ELEMENTS (file_path),
+              (GAsyncReadyCallback)uim_get_iccid_ready,
+              task);
+}
+
 static void
 dms_uim_get_iccid_ready (QmiClientDms *client,
                          GAsyncResult *res,
@@ -118,7 +256,7 @@ dms_uim_get_iccid_ready (QmiClientDms *client,
         const gchar *str = NULL;
 
         qmi_message_dms_uim_get_iccid_output_get_iccid (output, &str, NULL);
-        g_task_return_pointer (task, g_strdup (str), (GDestroyNotify) g_free);
+        g_task_return_pointer (task, g_strdup (str), g_free);
     }
 
     if (output)
@@ -128,20 +266,16 @@ dms_uim_get_iccid_ready (QmiClientDms *client,
 }
 
 static void
-load_sim_identifier (MMBaseSim           *self,
-                     GAsyncReadyCallback  callback,
-                     gpointer             user_data)
+dms_uim_get_iccid (MMSimQmi *self,
+                   GTask    *task)
 {
-    GTask *task;
     QmiClient *client = NULL;
 
-    task = g_task_new (self, NULL, callback, user_data);
     if (!ensure_qmi_client (task,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_DMS, &client))
         return;
 
-    mm_dbg ("loading SIM identifier...");
     qmi_client_dms_uim_get_iccid (QMI_CLIENT_DMS (client),
                                   NULL,
                                   5,
@@ -150,6 +284,24 @@ load_sim_identifier (MMBaseSim           *self,
                                   task);
 }
 
+static void
+load_sim_identifier (MMBaseSim           *_self,
+                     GAsyncReadyCallback  callback,
+                     gpointer             user_data)
+{
+    MMSimQmi *self;
+    GTask *task;
+
+    self = MM_SIM_QMI (_self);
+    task = g_task_new (self, NULL, callback, user_data);
+
+    mm_dbg ("loading SIM identifier...");
+    if (!self->priv->dms_uim_deprecated)
+        dms_uim_get_iccid (self, task);
+    else
+        uim_get_iccid (self, task);
+}
+
 /*****************************************************************************/
 /* Load IMSI */
 
@@ -161,6 +313,55 @@ load_imsi_finish (MMBaseSim     *self,
     return g_task_propagate_pointer (G_TASK (res), error);
 }
 
+static void
+uim_get_imsi_ready (QmiClientUim *client,
+                    GAsyncResult *res,
+                    GTask        *task)
+{
+    GError *error = NULL;
+    GArray *read_result;
+    gchar *imsi;
+
+    read_result = uim_read_finish (client, res, &error);
+    if (!read_result) {
+        g_task_return_error (task, error);
+        g_object_unref (task);
+        return;
+    }
+
+    imsi = mm_bcd_to_string ((const guint8 *) read_result->data, read_result->len);
+    g_assert (imsi);
+    if (strlen (imsi) < 3)
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "IMSI is malformed");
+    else
+        /* EFimsi contains a length byte, follwed by a nibble for parity,
+         * and then followed by the actual IMSI in BCD. After converting
+         * the BCD into a decimal string, we simply skip the first 3
+         * decimal digits to obtain the IMSI. */
+        g_task_return_pointer (task, g_strdup (imsi + 3), g_free);
+    g_object_unref (task);
+
+    g_free (imsi);
+    g_array_unref (read_result);
+}
+
+static void
+uim_get_imsi (MMSimQmi *self,
+              GTask    *task)
+{
+    static const guint16 file_path[] = { 0x3F00, 0x7FFF };
+
+    uim_read (self,
+              0x6F07,
+              file_path,
+              G_N_ELEMENTS (file_path),
+              (GAsyncReadyCallback)uim_get_imsi_ready,
+              task);
+}
+
 static void
 dms_uim_get_imsi_ready (QmiClientDms *client,
                         GAsyncResult *res,
@@ -180,7 +381,7 @@ dms_uim_get_imsi_ready (QmiClientDms *client,
         const gchar *str = NULL;
 
         qmi_message_dms_uim_get_imsi_output_get_imsi (output, &str, NULL);
-        g_task_return_pointer (task, g_strdup (str), (GDestroyNotify) g_free);
+        g_task_return_pointer (task, g_strdup (str), g_free);
     }
 
     if (output)
@@ -190,20 +391,16 @@ dms_uim_get_imsi_ready (QmiClientDms *client,
 }
 
 static void
-load_imsi (MMBaseSim           *self,
-           GAsyncReadyCallback callback,
-           gpointer            user_data)
+dms_uim_get_imsi (MMSimQmi *self,
+                  GTask    *task)
 {
-    GTask *task;
     QmiClient *client = NULL;
 
-    task = g_task_new (self, NULL, callback, user_data);
     if (!ensure_qmi_client (task,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_DMS, &client))
         return;
 
-    mm_dbg ("loading IMSI...");
     qmi_client_dms_uim_get_imsi (QMI_CLIENT_DMS (client),
                                  NULL,
                                  5,
@@ -212,6 +409,24 @@ load_imsi (MMBaseSim           *self,
                                  task);
 }
 
+static void
+load_imsi (MMBaseSim           *_self,
+           GAsyncReadyCallback  callback,
+           gpointer             user_data)
+{
+    MMSimQmi *self;
+    GTask *task;
+
+    self = MM_SIM_QMI (_self);
+    task = g_task_new (self, NULL, callback, user_data);
+
+    mm_dbg ("loading IMSI...");
+    if (!self->priv->dms_uim_deprecated)
+        dms_uim_get_imsi (self, task);
+    else
+        uim_get_imsi (self, task);
+}
+
 /*****************************************************************************/
 /* Load operator identifier */
 
@@ -304,7 +519,7 @@ load_operator_identifier_ready (QmiClientNas *client,
         g_string_append_printf (aux, "%.3" G_GUINT16_FORMAT, mnc);
     else
         g_string_append_printf (aux, "%.2" G_GUINT16_FORMAT, mnc);
-    g_task_return_pointer (task, g_string_free (aux, FALSE), (GDestroyNotify) g_free);
+    g_task_return_pointer (task, g_string_free (aux, FALSE), g_free);
     g_object_unref (task);
 }
 
@@ -353,7 +568,7 @@ load_operator_name_ready (QmiClientNas *client,
     if (!get_home_network (client, res, NULL, NULL, NULL, &operator_name, &error))
         g_task_return_error (task, error);
     else
-        g_task_return_pointer (task, operator_name, (GDestroyNotify) g_free);
+        g_task_return_pointer (task, operator_name, g_free);
     g_object_unref (task);
 }
 
@@ -449,7 +664,7 @@ uim_verify_pin (MMSimQmi *self,
     QmiClient *client = NULL;
 
     if (!ensure_qmi_client (task,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_UIM, &client))
         return;
 
@@ -480,9 +695,6 @@ dms_uim_verify_pin_ready (QmiClientDms *client,
 {
     QmiMessageDmsUimVerifyPinOutput *output = NULL;
     GError *error = NULL;
-    MMSimQmi *self;
-
-    self = g_task_get_source_object (task);
 
     output = qmi_client_dms_uim_verify_pin_finish (client, res, &error);
     if (!output) {
@@ -507,7 +719,7 @@ dms_uim_verify_pin (MMSimQmi *self,
     QmiClient *client = NULL;
 
     if (!ensure_qmi_client (NULL,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_DMS, &client)) {
         /* Very unlikely that this will ever happen, but anyway, try with
          * UIM service instead */
@@ -543,7 +755,7 @@ send_pin (MMBaseSim           *_self,
     self = MM_SIM_QMI (_self);
     task = g_task_new (self, NULL, callback, user_data);
 
-    g_task_set_task_data (task, g_strdup (pin), (GDestroyNotify) g_free);
+    g_task_set_task_data (task, g_strdup (pin), g_free);
 
     mm_dbg ("Verifying PIN...");
     if (!self->priv->dms_uim_deprecated)
@@ -608,7 +820,7 @@ uim_unblock_pin (MMSimQmi *self,
     UnblockPinContext *ctx;
 
     if (!ensure_qmi_client (task,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_UIM, &client))
         return;
 
@@ -642,9 +854,6 @@ dms_uim_unblock_pin_ready (QmiClientDms *client,
 {
     QmiMessageDmsUimUnblockPinOutput *output = NULL;
     GError *error = NULL;
-    MMSimQmi *self;
-
-    self = g_task_get_source_object (task);
 
     output = qmi_client_dms_uim_unblock_pin_finish (client, res, &error);
     if (!output) {
@@ -670,7 +879,7 @@ dms_uim_unblock_pin (MMSimQmi *self,
     UnblockPinContext *ctx;
 
     if (!ensure_qmi_client (NULL,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_DMS, &client)) {
         /* Very unlikely that this will ever happen, but anyway, try with
          * UIM service instead */
@@ -778,7 +987,7 @@ uim_change_pin (MMSimQmi *self,
     ChangePinContext *ctx;
 
     if (!ensure_qmi_client (task,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_UIM, &client))
         return;
 
@@ -812,9 +1021,6 @@ dms_uim_change_pin_ready (QmiClientDms *client,
 {
     QmiMessageDmsUimChangePinOutput *output = NULL;
     GError *error = NULL;
-    MMSimQmi *self;
-
-    self = g_task_get_source_object (task);
 
     output = qmi_client_dms_uim_change_pin_finish (client, res, &error);
     if (!output) {
@@ -840,7 +1046,7 @@ dms_uim_change_pin (MMSimQmi *self,
     ChangePinContext *ctx;
 
     if (!ensure_qmi_client (NULL,
-                            MM_SIM_QMI (self),
+                            self,
                             QMI_SERVICE_DMS, &client)) {
         /* Very unlikely that this will ever happen, but anyway, try with
          * UIM service instead */
@@ -981,9 +1187,6 @@ dms_uim_set_pin_protection_ready (QmiClientDms *client,
 {
     QmiMessageDmsUimSetPinProtectionOutput *output = NULL;
     GError *error = NULL;
-    MMSimQmi *self;
-
-    self = g_task_get_source_object (task);
 
     output = qmi_client_dms_uim_set_pin_protection_finish (client, res, &error);
     if (!output) {
