@@ -53,12 +53,13 @@ peek_device (gpointer self,
 
         port = mm_base_modem_peek_port_mbim (modem);
         if (!port) {
-            g_simple_async_report_error_in_idle (G_OBJECT (self),
-                                                 callback,
-                                                 user_data,
-                                                 MM_CORE_ERROR,
-                                                 MM_CORE_ERROR_FAILED,
-                                                 "Couldn't peek MBIM port");
+            g_task_report_new_error (self,
+                                     callback,
+                                     user_data,
+                                     peek_device,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Couldn't peek MBIM port");
             g_object_unref (modem);
             return FALSE;
         }
@@ -74,21 +75,16 @@ peek_device (gpointer self,
 /* Send the SMS */
 
 typedef struct {
-    MMBaseSms *self;
     MMBaseModem *modem;
     MbimDevice *device;
-    GSimpleAsyncResult *result;
     GList *current;
 } SmsSendContext;
 
 static void
-sms_send_context_complete_and_free (SmsSendContext *ctx)
+sms_send_context_free (SmsSendContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
     g_object_unref (ctx->device);
     g_object_unref (ctx->modem);
-    g_object_unref (ctx->self);
     g_slice_free (SmsSendContext, ctx);
 }
 
@@ -97,20 +93,22 @@ sms_send_finish (MMBaseSms *self,
                  GAsyncResult *res,
                  GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static void sms_send_next_part (SmsSendContext *ctx);
+static void sms_send_next_part (GTask *task);
 
 static void
 sms_send_set_ready (MbimDevice *device,
                     GAsyncResult *res,
-                    SmsSendContext *ctx)
+                    GTask *task)
 {
+    SmsSendContext *ctx;
     MbimMessage *response;
     GError *error = NULL;
     guint32 message_reference;
 
+    ctx = g_task_get_task_data (task);
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
         mbim_message_response_get_result (response, MBIM_MESSAGE_TYPE_COMMAND_DONE, &error) &&
@@ -127,19 +125,20 @@ sms_send_set_ready (MbimDevice *device,
 
     if (error) {
         g_prefix_error (&error, "Couldn't send SMS part: ");
-        g_simple_async_result_take_error (ctx->result, error);
-        sms_send_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
     /* Go on with next part */
     ctx->current = g_list_next (ctx->current);
-    sms_send_next_part (ctx);
+    sms_send_next_part (task);
 }
 
 static void
-sms_send_next_part (SmsSendContext *ctx)
+sms_send_next_part (GTask *task)
 {
+    SmsSendContext *ctx;
     MbimMessage *message;
     guint8 *pdu;
     guint pdulen = 0;
@@ -147,18 +146,19 @@ sms_send_next_part (SmsSendContext *ctx)
     GError *error = NULL;
     MbimSmsPduSendRecord send_record;
 
+    ctx = g_task_get_task_data (task);
     if (!ctx->current) {
         /* Done we are */
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-        sms_send_context_complete_and_free (ctx);
+        g_task_return_boolean (task, TRUE);
+        g_object_unref (task);
         return;
     }
 
     /* Get PDU */
     pdu = mm_sms_part_3gpp_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
     if (!pdu) {
-        g_simple_async_result_take_error (ctx->result, error);
-        sms_send_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -174,7 +174,7 @@ sms_send_next_part (SmsSendContext *ctx)
                          30,
                          NULL,
                          (GAsyncReadyCallback)sms_send_set_ready,
-                         ctx);
+                         task);
     mbim_message_unref (message);
     g_free (pdu);
 }
@@ -186,45 +186,39 @@ sms_send (MMBaseSms *self,
 {
     SmsSendContext *ctx;
     MbimDevice *device;
+    GTask *task;
 
     if (!peek_device (self, &device, callback, user_data))
         return;
 
     /* Setup the context */
     ctx = g_slice_new0 (SmsSendContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             sms_send);
-    ctx->self = g_object_ref (self);
     ctx->device = g_object_ref (device);
     g_object_get (self,
                   MM_BASE_SMS_MODEM, &ctx->modem,
                   NULL);
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)sms_send_context_free);
+
     ctx->current = mm_base_sms_get_parts (self);;
-    sms_send_next_part (ctx);
+    sms_send_next_part (task);
 }
 
 /*****************************************************************************/
 
 typedef struct {
-    MMBaseSms *self;
     MMBaseModem *modem;
     MbimDevice *device;
-    GSimpleAsyncResult *result;
     GList *current;
     guint n_failed;
 } SmsDeletePartsContext;
 
 static void
-sms_delete_parts_context_complete_and_free (SmsDeletePartsContext *ctx)
+sms_delete_parts_context_free (SmsDeletePartsContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
     g_object_unref (ctx->device);
     g_object_unref (ctx->modem);
-    g_object_unref (ctx->self);
     g_slice_free (SmsDeletePartsContext, ctx);
 }
 
@@ -233,19 +227,21 @@ sms_delete_finish (MMBaseSms *self,
                    GAsyncResult *res,
                    GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static void delete_next_part (SmsDeletePartsContext *ctx);
+static void delete_next_part (GTask *task);
 
 static void
 sms_delete_set_ready (MbimDevice *device,
                       GAsyncResult *res,
-                      SmsDeletePartsContext *ctx)
+                      GTask *task)
 {
+    SmsDeletePartsContext *ctx;
     MbimMessage *response;
     GError *error = NULL;
 
+    ctx = g_task_get_task_data (task);
     response = mbim_device_command_finish (device, res, &error);
     if (response &&
         mbim_message_response_get_result (response, MBIM_MESSAGE_TYPE_COMMAND_DONE, &error))
@@ -266,14 +262,16 @@ sms_delete_set_ready (MbimDevice *device,
     mm_sms_part_set_index ((MMSmsPart *)ctx->current->data, SMS_PART_INVALID_INDEX);
 
     ctx->current = g_list_next (ctx->current);
-    delete_next_part (ctx);
+    delete_next_part (task);
 }
 
 static void
-delete_next_part (SmsDeletePartsContext *ctx)
+delete_next_part (GTask *task)
 {
+    SmsDeletePartsContext *ctx;
     MbimMessage *message;
 
+    ctx = g_task_get_task_data (task);
     /* Skip non-stored parts */
     while (ctx->current &&
            mm_sms_part_get_index ((MMSmsPart *)ctx->current->data) == SMS_PART_INVALID_INDEX)
@@ -282,15 +280,15 @@ delete_next_part (SmsDeletePartsContext *ctx)
     /* If all removed, we're done */
     if (!ctx->current) {
         if (ctx->n_failed > 0)
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_FAILED,
-                                             "Couldn't delete %u parts from this SMS",
-                                             ctx->n_failed);
+            g_task_return_new_error (task,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Couldn't delete %u parts from this SMS",
+                                     ctx->n_failed);
         else
-            g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+            g_task_return_boolean (task, TRUE);
 
-        sms_delete_parts_context_complete_and_free (ctx);
+        g_object_unref (task);
         return;
     }
 
@@ -302,7 +300,7 @@ delete_next_part (SmsDeletePartsContext *ctx)
                          10,
                          NULL,
                          (GAsyncReadyCallback)sms_delete_set_ready,
-                         ctx);
+                         task);
     mbim_message_unref (message);
 
 }
@@ -314,24 +312,23 @@ sms_delete (MMBaseSms *self,
 {
     SmsDeletePartsContext *ctx;
     MbimDevice *device;
+    GTask *task;
 
     if (!peek_device (self, &device, callback, user_data))
         return;
 
     ctx = g_slice_new0 (SmsDeletePartsContext);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             sms_delete);
-    ctx->self = g_object_ref (self);
     ctx->device = g_object_ref (device);
     g_object_get (self,
                   MM_BASE_SMS_MODEM, &ctx->modem,
                   NULL);
 
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)sms_delete_parts_context_free);
+
     /* Go on deleting parts */
     ctx->current = mm_base_sms_get_parts (self);
-    delete_next_part (ctx);
+    delete_next_part (task);
 }
 
 /*****************************************************************************/
