@@ -23,6 +23,7 @@
 #define _LIBMM_INSIDE_MM
 #include <libmm-glib.h>
 
+#include "mm-helper-enums-types.h"
 #include "mm-sms-part-3gpp.h"
 #include "mm-charsets.h"
 #include "mm-log.h"
@@ -132,10 +133,10 @@ sms_decode_address (const guint8 *address, int len)
     if (addrtype == SMS_NUMBER_TYPE_ALPHA) {
         guint8 *unpacked;
         guint32 unpacked_len;
-        unpacked = gsm_unpack (address, (len * 4) / 7, 0, &unpacked_len);
+        unpacked = mm_charset_gsm_unpack (address, (len * 4) / 7, 0, &unpacked_len);
         utf8 = (char *)mm_charset_gsm_unpacked_to_utf8 (unpacked,
                                                         unpacked_len);
-        g_free(unpacked);
+        g_free (unpacked);
     } else if (addrtype == SMS_NUMBER_TYPE_INTL &&
                addrplan == SMS_NUMBER_PLAN_TELEPHONE) {
         /* International telphone number, format as "+1234567890" */
@@ -241,17 +242,38 @@ sms_decode_text (const guint8 *text, int len, MMSmsEncoding encoding, int bit_of
     guint32 unpacked_len;
 
     if (encoding == MM_SMS_ENCODING_GSM7) {
-        mm_dbg ("Converting SMS part text from GSM7 to UTF8...");
-        unpacked = gsm_unpack ((const guint8 *) text, len, bit_offset, &unpacked_len);
+        mm_dbg ("Converting SMS part text from GSM-7 to UTF-8...");
+        unpacked = mm_charset_gsm_unpack ((const guint8 *) text, len, bit_offset, &unpacked_len);
         utf8 = (char *) mm_charset_gsm_unpacked_to_utf8 (unpacked, unpacked_len);
         mm_dbg ("   Got UTF-8 text: '%s'", utf8);
         g_free (unpacked);
     } else if (encoding == MM_SMS_ENCODING_UCS2) {
-        mm_dbg ("Converting SMS part text from UCS-2BE to UTF8...");
-        utf8 = g_convert ((char *) text, len, "UTF8", "UCS-2BE", NULL, NULL, NULL);
-        mm_dbg ("   Got UTF-8 text: '%s'", utf8);
+        /* Despite 3GPP TS 23.038 specifies that Unicode SMS messages are
+         * encoded in UCS-2, UTF-16 encoding is commonly used instead on many
+         * modern platforms to allow encoding code points that fall outside the
+         * Basic Multilingual Plane (BMP), such as Emoji. Most of the UCS-2
+         * code points are identical to their equivalent UTF-16 code points.
+         * In UTF-16, non-BMP code points are encoded in a pair of surrogate
+         * code points (i.e. a high surrogate in 0xD800..0xDBFF, followed by a
+         * low surrogate in 0xDC00..0xDFFF). An isolated surrogate code point
+         * has no general interpretation in UTF-16, but could be a valid
+         * (though unmapped) code point in UCS-2. Here we first try to decode
+         * the SMS message in UTF-16BE, and if that fails, fall back to decode
+         * in UCS-2BE.
+         */
+        mm_dbg ("Converting SMS part text from UTF-16BE to UTF-8...");
+        utf8 = g_convert ((const gchar *) text, len, "UTF8", "UTF16BE", NULL, NULL, NULL);
+        if (!utf8) {
+            mm_dbg ("Converting SMS part text from UCS-2BE to UTF8...");
+            utf8 = g_convert ((const gchar *) text, len, "UTF8", "UCS-2BE", NULL, NULL, NULL);
+        }
+        if (!utf8) {
+            mm_warn ("Couldn't convert SMS part contents from UTF-16BE/UCS-2BE to UTF-8: not decoding any text");
+            utf8 = g_strdup ("");
+        } else
+            mm_dbg ("   Got UTF-8 text: '%s'", utf8);
     } else {
-        g_warn_if_reached ();
+        mm_warn ("Unexpected encoding '%s': not decoding any text", mm_sms_encoding_get_string (encoding));
         utf8 = g_strdup ("");
     }
 
@@ -523,7 +545,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
                 break;
             default:
                 /* Cannot happen as we AND with the 0x18 mask */
-                g_assert_not_reached();
+                g_assert_not_reached ();
             }
         }
 
@@ -580,7 +602,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
         PDU_SIZE_CHECK (tp_dcs_offset + 1, "cannot read TP-DCS");
 
         /* Encoding given in the 'alphabet' bits */
-        user_data_encoding = sms_encoding_type(pdu[tp_dcs_offset]);
+        user_data_encoding = sms_encoding_type (pdu[tp_dcs_offset]);
         switch (user_data_encoding) {
         case MM_SMS_ENCODING_GSM7:
             mm_dbg ("  user data encoding is GSM7");
@@ -729,7 +751,7 @@ mm_sms_part_3gpp_new_from_binary_pdu (guint index,
  * @address: the phone number to encode
  * @buf: the buffer to encode @address in
  * @buflen: the size  of @buf
- * @is_smsc: if %TRUE encode size as number of octets of address infromation,
+ * @is_smsc: if %TRUE encode size as number of octets of address information,
  *   otherwise if %FALSE encode size as number of digits of @address
  *
  * Returns: the size in bytes of the data added to @buf
@@ -956,7 +978,7 @@ mm_sms_part_3gpp_get_submit_pdu (MMSmsPart *part,
                 *udl_ptr,
                 mm_sms_part_get_concat_sequence (part) ? "with" : "without");
 
-        packed = gsm_pack (unpacked, unlen, shift, &packlen);
+        packed = mm_charset_gsm_pack (unpacked, unlen, shift, &packlen);
         g_free (unpacked);
         if (!packed || packlen == 0) {
             g_free (packed);
@@ -1026,7 +1048,6 @@ gchar **
 mm_sms_part_3gpp_util_split_text (const gchar *text,
                                   MMSmsEncoding *encoding)
 {
-    guint gsm_unsupported = 0;
     gchar **out;
     guint n_chunks;
     guint i;
@@ -1058,10 +1079,7 @@ mm_sms_part_3gpp_util_split_text (const gchar *text,
      */
 
     /* Check if we can do GSM encoding */
-    mm_charset_get_encoded_len (text,
-                                MM_MODEM_CHARSET_GSM,
-                                &gsm_unsupported);
-    if (gsm_unsupported > 0) {
+    if (!mm_charset_can_convert_to (text, MM_MODEM_CHARSET_GSM)) {
         /* If cannot do it in GSM encoding, do it in UCS-2 */
         GByteArray *array;
 
