@@ -15,12 +15,19 @@
 
 #include <glib.h>
 #include <string.h>
+#include <locale.h>
 
 #include "mm-modem-helpers.h"
 #include "mm-log.h"
 
+#if defined ENABLE_TEST_MESSAGE_TRACES
+#define trace(message, ...) g_print (message, ##__VA_ARGS__)
+#else
+#define trace(...)
+#endif
+
 static void
-test_def_chars (void *f, gpointer d)
+test_gsm7_default_chars (void)
 {
     /* Test that a string with all the characters in the GSM 03.38 charset
      * are converted from UTF-8 to GSM and back to UTF-8 successfully.
@@ -44,7 +51,7 @@ test_def_chars (void *f, gpointer d)
 }
 
 static void
-test_esc_chars (void *f, gpointer d)
+test_gsm7_extended_chars (void)
 {
     /* Test that a string with all the characters in the extended GSM 03.38
      * charset are converted from UTF-8 to GSM and back to UTF-8 successfully.
@@ -68,7 +75,7 @@ test_esc_chars (void *f, gpointer d)
 }
 
 static void
-test_mixed_chars (void *f, gpointer d)
+test_gsm7_mixed_chars (void)
 {
     /* Test that a string with a mix of GSM 03.38 default and extended characters
      * is converted from UTF-8 to GSM and back to UTF-8 successfully.
@@ -92,14 +99,14 @@ test_mixed_chars (void *f, gpointer d)
 }
 
 static void
-test_unpack_gsm7 (void *f, gpointer d)
+test_gsm7_unpack_basic (void)
 {
     static const guint8 gsm[] = { 0xC8, 0xF7, 0x1D, 0x14, 0x96, 0x97, 0x41, 0xF9, 0x77, 0xFD, 0x07 };
     static const guint8 expected[] = { 0x48, 0x6f, 0x77, 0x20, 0x61, 0x72, 0x65, 0x20, 0x79, 0x6f, 0x75, 0x3f };
     guint8 *unpacked;
     guint32 unpacked_len = 0;
 
-    unpacked = gsm_unpack (gsm, (sizeof (gsm) * 8) / 7, 0, &unpacked_len);
+    unpacked = mm_charset_gsm_unpack (gsm, (sizeof (gsm) * 8) / 7, 0, &unpacked_len);
     g_assert (unpacked);
     g_assert_cmpint (unpacked_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (unpacked, expected, unpacked_len), ==, 0);
@@ -108,7 +115,7 @@ test_unpack_gsm7 (void *f, gpointer d)
 }
 
 static void
-test_unpack_gsm7_7_chars (void *f, gpointer d)
+test_gsm7_unpack_7_chars (void)
 {
     static const guint8 gsm[] = { 0xF1, 0x7B, 0x59, 0x4E, 0xCF, 0xD7, 0x01 };
     static const guint8 expected[] = { 0x71, 0x77, 0x65, 0x72, 0x74, 0x79, 0x75};
@@ -120,7 +127,7 @@ test_unpack_gsm7_7_chars (void *f, gpointer d)
      * we expect to get the number of characters that were specified.
      */
 
-    unpacked = gsm_unpack (gsm, 7 , 0, &unpacked_len);
+    unpacked = mm_charset_gsm_unpack (gsm, 7 , 0, &unpacked_len);
     g_assert (unpacked);
     g_assert_cmpint (unpacked_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (unpacked, expected, unpacked_len), ==, 0);
@@ -129,7 +136,7 @@ test_unpack_gsm7_7_chars (void *f, gpointer d)
 }
 
 static void
-test_unpack_gsm7_all_chars (void *f, gpointer d)
+test_gsm7_unpack_all_chars (void)
 {
     /* Packed array of all chars in GSM default and extended charset */
     static const guint8 gsm[] = {
@@ -153,7 +160,7 @@ test_unpack_gsm7_all_chars (void *f, gpointer d)
     guint32 unpacked_len = 0;
     int i;
 
-    unpacked = gsm_unpack (gsm, (sizeof (gsm) * 8) / 7, 0, &unpacked_len);
+    unpacked = mm_charset_gsm_unpack (gsm, (sizeof (gsm) * 8) / 7, 0, &unpacked_len);
     g_assert (unpacked);
     g_assert_cmpint (unpacked_len, ==, 148);
 
@@ -168,14 +175,14 @@ test_unpack_gsm7_all_chars (void *f, gpointer d)
 }
 
 static void
-test_pack_gsm7 (void *f, gpointer d)
+test_gsm7_pack_basic (void)
 {
     static const guint8 unpacked[] = { 0x48, 0x6f, 0x77, 0x20, 0x61, 0x72, 0x65, 0x20, 0x79, 0x6f, 0x75, 0x3f };
     static const guint8 expected[] = { 0xC8, 0xF7, 0x1D, 0x14, 0x96, 0x97, 0x41, 0xF9, 0x77, 0xFD, 0x07 };
     guint8 *packed;
     guint32 packed_len = 0;
 
-    packed = gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (packed, expected, packed_len), ==, 0);
@@ -184,7 +191,7 @@ test_pack_gsm7 (void *f, gpointer d)
 }
 
 static void
-test_pack_gsm7_7_chars (void *f, gpointer d)
+test_gsm7_pack_7_chars (void)
 {
     static const guint8 unpacked[] = { 0x71, 0x77, 0x65, 0x72, 0x74, 0x79, 0x75 };
     static const guint8 expected[] = { 0xF1, 0x7B, 0x59, 0x4E, 0xCF, 0xD7, 0x01 };
@@ -197,7 +204,7 @@ test_pack_gsm7_7_chars (void *f, gpointer d)
      * the intended message to remove it when required.
      */
 
-    packed = gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (packed, expected, packed_len), ==, 0);
@@ -205,27 +212,8 @@ test_pack_gsm7_7_chars (void *f, gpointer d)
     g_free (packed);
 }
 
-#if 0
 static void
-print_array (const guint8 *array, guint32 len)
-{
-    int col;
-    guint8 c;
-
-    g_print ("\n");
-    for (c = 0, col = 0; c < len; c++) {
-        g_print ("0x%02X, ", array[c] & 0xFF);
-        if (col++ == 11) {
-            col = 0;
-            g_print ("\n");
-        }
-    }
-    g_print ("\n");
-}
-#endif
-
-static void
-test_pack_gsm7_all_chars (void *f, gpointer d)
+test_gsm7_pack_all_chars (void)
 {
     /* Packed array of all chars in GSM default and extended charset */
     static const guint8 expected[] = {
@@ -255,7 +243,7 @@ test_pack_gsm7_all_chars (void *f, gpointer d)
     for (c = 0; c < sizeof (ext); c++)
         g_byte_array_append (unpacked, &ext[c], 1);
 
-    packed = gsm_pack (unpacked->data, unpacked->len, 0, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked->data, unpacked->len, 0, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (packed, expected, packed_len), ==, 0);
@@ -265,7 +253,7 @@ test_pack_gsm7_all_chars (void *f, gpointer d)
 }
 
 static void
-test_pack_gsm7_24_chars (void *f, gpointer d)
+test_gsm7_pack_24_chars (void)
 {
     static const guint8 unpacked[] = {
         0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B,
@@ -278,7 +266,7 @@ test_pack_gsm7_24_chars (void *f, gpointer d)
      * are packed into an exact number of bytes.
      */
 
-    packed = gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, 21);
 
@@ -286,7 +274,7 @@ test_pack_gsm7_24_chars (void *f, gpointer d)
 }
 
 static void
-test_pack_gsm7_last_septet_alone (void *f, gpointer d)
+test_gsm7_pack_last_septet_alone (void)
 {
     static const guint8 unpacked[] = {
         0x54, 0x68, 0x69, 0x73, 0x20, 0x69, 0x73, 0x20, 0x72, 0x65, 0x61, 0x6C,
@@ -304,7 +292,7 @@ test_pack_gsm7_last_septet_alone (void *f, gpointer d)
      * septet will be in an octet by itself) packs correctly.
      */
 
-    packed = gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked, sizeof (unpacked), 0, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, sizeof (expected));
 
@@ -312,14 +300,14 @@ test_pack_gsm7_last_septet_alone (void *f, gpointer d)
 }
 
 static void
-test_pack_gsm7_7_chars_offset (void *f, gpointer d)
+test_gsm7_pack_7_chars_offset (void)
 {
     static const guint8 unpacked[] = { 0x68, 0x65, 0x6C, 0x6C, 0x6F, 0x10, 0x2F };
     static const guint8 expected[] = { 0x00, 0x5D, 0x66, 0xB3, 0xDF, 0x90, 0x17 };
     guint8 *packed;
     guint32 packed_len = 0;
 
-    packed = gsm_pack (unpacked, sizeof (unpacked), 5, &packed_len);
+    packed = mm_charset_gsm_pack (unpacked, sizeof (unpacked), 5, &packed_len);
     g_assert (packed);
     g_assert_cmpint (packed_len, ==, sizeof (expected));
     g_assert_cmpint (memcmp (packed, expected, packed_len), ==, 0);
@@ -328,7 +316,7 @@ test_pack_gsm7_7_chars_offset (void *f, gpointer d)
 }
 
 static void
-test_take_convert_ucs2_hex_utf8 (void *f, gpointer d)
+test_take_convert_ucs2_hex_utf8 (void)
 {
     gchar *src, *converted;
 
@@ -340,7 +328,7 @@ test_take_convert_ucs2_hex_utf8 (void *f, gpointer d)
 }
 
 static void
-test_take_convert_ucs2_bad_ascii (void *f, gpointer d)
+test_take_convert_ucs2_bad_ascii (void)
 {
     gchar *src, *converted;
 
@@ -352,7 +340,7 @@ test_take_convert_ucs2_bad_ascii (void *f, gpointer d)
 }
 
 static void
-test_take_convert_ucs2_bad_ascii2 (void *f, gpointer d)
+test_take_convert_ucs2_bad_ascii2 (void)
 {
     gchar *src, *converted;
 
@@ -362,6 +350,66 @@ test_take_convert_ucs2_bad_ascii2 (void *f, gpointer d)
     g_assert (converted == NULL);
 }
 
+struct charset_can_convert_to_test_s {
+    const char *utf8;
+    gboolean    to_gsm;
+    gboolean    to_ira;
+    gboolean    to_8859_1;
+    gboolean    to_ucs2;
+    gboolean    to_pccp437;
+    gboolean    to_pcdn;
+};
+
+static void
+test_charset_can_covert_to (void)
+{
+    static const struct charset_can_convert_to_test_s charset_can_convert_to_test[] = {
+        {
+            .utf8 = "",
+            .to_gsm = TRUE, .to_ira = TRUE, .to_8859_1 = TRUE, .to_ucs2 = TRUE, .to_pccp437 = TRUE, .to_pcdn = TRUE,
+        },
+        {
+            .utf8 = " ",
+            .to_gsm = TRUE, .to_ira = TRUE, .to_8859_1 = TRUE, .to_ucs2 = TRUE, .to_pccp437 = TRUE, .to_pcdn = TRUE,
+        },
+        {
+            .utf8 = "some basic ascii",
+            .to_gsm = TRUE, .to_ira = TRUE, .to_8859_1 = TRUE, .to_ucs2 = TRUE, .to_pccp437 = TRUE, .to_pcdn = TRUE,
+        },
+        {
+            .utf8 = "ホモ・サピエンス 喂人类 katakana, chinese, english: UCS2 takes it all",
+            .to_gsm = FALSE, .to_ira = FALSE, .to_8859_1 = FALSE, .to_ucs2 = TRUE, .to_pccp437 = FALSE, .to_pcdn = FALSE,
+        },
+        {
+            .utf8 = "Some from the GSM7 basic set: a % Ψ Ω ñ ö è æ",
+            .to_gsm = TRUE, .to_ira = FALSE, .to_8859_1 = FALSE, .to_ucs2 = TRUE, .to_pccp437 = FALSE, .to_pcdn = FALSE,
+        },
+        {
+            .utf8 = "More from the GSM7 extended set: {} [] ~ € |",
+            .to_gsm = TRUE, .to_ira = FALSE, .to_8859_1 = FALSE, .to_ucs2 = TRUE, .to_pccp437 = FALSE, .to_pcdn = FALSE,
+        },
+        {
+            .utf8 = "patín cannot be encoded in GSM7 or IRA, but is valid UCS2, ISO-8859-1, CP437 and CP850",
+            .to_gsm = FALSE, .to_ira = FALSE, .to_8859_1 = TRUE, .to_ucs2 = TRUE, .to_pccp437 = TRUE, .to_pcdn = TRUE,
+        },
+        {
+            .utf8 = "ècole can be encoded in multiple ways, but not in IRA",
+            .to_gsm = TRUE, .to_ira = FALSE, .to_8859_1 = TRUE, .to_ucs2 = TRUE, .to_pccp437 = TRUE, .to_pcdn = TRUE,
+        },
+    };
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (charset_can_convert_to_test); i++) {
+        trace ("testing charset conversion: '%s'\n", charset_can_convert_to_test[i].utf8);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_GSM)     == charset_can_convert_to_test[i].to_gsm);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_IRA)     == charset_can_convert_to_test[i].to_ira);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_8859_1)  == charset_can_convert_to_test[i].to_8859_1);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_UCS2)    == charset_can_convert_to_test[i].to_ucs2);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_PCCP437) == charset_can_convert_to_test[i].to_pccp437);
+        g_assert (mm_charset_can_convert_to (charset_can_convert_to_test[i].utf8, MM_MODEM_CHARSET_PCDN)    == charset_can_convert_to_test[i].to_pcdn);
+    }
+}
+
 void
 _mm_log (const char *loc,
          const char *func,
@@ -382,40 +430,30 @@ _mm_log (const char *loc,
 #endif
 }
 
-typedef GTestFixtureFunc TCFunc;
-
-#define TESTCASE(t, d) g_test_create_case (#t, 0, d, NULL, (TCFunc) t, NULL)
-
 int main (int argc, char **argv)
 {
-    GTestSuite *suite;
-    gint result;
+    setlocale (LC_ALL, "");
 
     g_test_init (&argc, &argv, NULL);
 
-    suite = g_test_get_root ();
+    g_test_add_func ("/MM/charsets/gsm7/default-chars",          test_gsm7_default_chars);
+    g_test_add_func ("/MM/charsets/gsm7/extended-chars",         test_gsm7_extended_chars);
+    g_test_add_func ("/MM/charsets/gsm7/mixed-chars",            test_gsm7_mixed_chars);
+    g_test_add_func ("/MM/charsets/gsm7/unpack/basic",           test_gsm7_unpack_basic);
+    g_test_add_func ("/MM/charsets/gsm7/unpack/7-chars",         test_gsm7_unpack_7_chars);
+    g_test_add_func ("/MM/charsets/gsm7/unpack/all-chars",       test_gsm7_unpack_all_chars);
+    g_test_add_func ("/MM/charsets/gsm7/pack/basic",             test_gsm7_pack_basic);
+    g_test_add_func ("/MM/charsets/gsm7/pack/7-chars",           test_gsm7_pack_7_chars);
+    g_test_add_func ("/MM/charsets/gsm7/pack/all-chars",         test_gsm7_pack_all_chars);
+    g_test_add_func ("/MM/charsets/gsm7/pack/24-chars",          test_gsm7_pack_24_chars);
+    g_test_add_func ("/MM/charsets/gsm7/pack/last-septet-alone", test_gsm7_pack_last_septet_alone);
+    g_test_add_func ("/MM/charsets/gsm7/pack/7-chars-offset",    test_gsm7_pack_7_chars_offset);
 
-    g_test_suite_add (suite, TESTCASE (test_def_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_esc_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_mixed_chars, NULL));
+    g_test_add_func ("/MM/charsets/take-convert/ucs2/hex",         test_take_convert_ucs2_hex_utf8);
+    g_test_add_func ("/MM/charsets/take-convert/ucs2/bad-ascii",   test_take_convert_ucs2_bad_ascii);
+    g_test_add_func ("/MM/charsets/take-convert/ucs2/bad-ascii-2", test_take_convert_ucs2_bad_ascii2);
 
-    g_test_suite_add (suite, TESTCASE (test_unpack_gsm7, NULL));
-    g_test_suite_add (suite, TESTCASE (test_unpack_gsm7_7_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_unpack_gsm7_all_chars, NULL));
+    g_test_add_func ("/MM/charsets/can-convert-to", test_charset_can_covert_to);
 
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7, NULL));
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7_7_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7_all_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7_24_chars, NULL));
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7_last_septet_alone, NULL));
-
-    g_test_suite_add (suite, TESTCASE (test_pack_gsm7_7_chars_offset, NULL));
-
-    g_test_suite_add (suite, TESTCASE (test_take_convert_ucs2_hex_utf8, NULL));
-    g_test_suite_add (suite, TESTCASE (test_take_convert_ucs2_bad_ascii, NULL));
-    g_test_suite_add (suite, TESTCASE (test_take_convert_ucs2_bad_ascii2, NULL));
-
-    result = g_test_run ();
-
-    return result;
+    return g_test_run ();
 }
